/**
 * $Id$
 *
 *<center>
 *               Avocent Corporation. Proprietary Information.
 * \n<em>
 *      This software is supplied under the terms of a license agreement or
 *      nondisclosure agreement with Avocent Corporation, or  one of its 
 *      affiliates, andaffiliates, and may not be used, disseminated, or 
 *      distributed except in accordance with the terms of that agreement.
 *
 *      3541 Gateway Boulevard Fremont, San Jose, CA 94538 U.S.A.
 *\n
 *                  US phone: 510-771-6100
 *
 *        Copyright &copy; 2001-2009 Avocent Corporation. All rights reserved.
 *</em> </center>
 *----------------------------------------------------------------------------\n
 *  MODULES     Linux device Driver                                           \n
 *----------------------------------------------------------------------------\n
 *  @file   aess_biospostdrv.c
 *  @brief  biospost driver and interrupt service routine
 *  @internal
 *  $RCSfile: aess_biospostdrv.c,v $
 *  $Revision: 1.1.1.1 $
 *  $Date: 2009/03/10 13:44:48 $
 *  $Author: Taiyi Kuo $
 *  $Source: //tpe-subversion/AESSTPE/puti/puti_fw/ProjectBase/trunk/platform/ast2050/evb/src/drivers/biospost/biospost_driver,v $
 *  $Name:  $
 *----------------------------------------------------------------------------*/
/******************************************************************************
* Content
* ----------------
*   aess_biospost_init()            - The function is invoked when biospost module is loaded into kernel.
*   aess_biospost_exit()            - The function is invoked when biospost module is removed from kernel. 
*   aess_biospost_open()            - Open the device note .
*   aess_biospost_close()           - Close the device note .
*   aess_biospost_clearall()        - Clean the S_aBIOSPostData structure and un-register the IRQ.
*   aess_biospost_ioctl()           - The API is invoked when user space issues an IOCTL command. 
*   aess_biospost_drv_init()        - Command handler for AESS_ BIOSPOSTDRV_INIT, used to initialize the I/O address.
*   aess_biospost_read()            - Command handler for AESS_BIOSPOSTDRV_READ uses to pass the incoming data to user space.
*   aess_biospost_isr()             - The function is the entry function of interrupt handler.
*   aess_biospost_reset()           - Command handler for AESS_BIOSPOSTDRV_RESET uses to clean biospost FIFO.  
******************************************************************************/

#define AESSBIOSPOSTDRV_C

#include <linux/errno.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/cdev.h>
#include <linux/interrupt.h>
#include <asm/uaccess.h>
#include <asm/arch/hardware.h>
#include <asm/io.h>
//#include <asm/arch/map.h>
#include <linux/spinlock.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <asm/string.h>

#include <linux/spinlock.h>
#include <asm/arch/irqs.h>

#include "aess_biospostdrv.h"

/* implement debug message with debugfs */
#include <linux/aess_debugfs.h>

#include <asm-arm/arch-ast2050/hw_regs.h>

static u8   u8tmpcounter = 0;
static u16 u16tmpcounter = 0;
static u32 u32tmpcounter = 0;

#define SNP_CTRL_REG LPC_HICR5_REG
#define EN_SNP0W            0x00000001
#define ENINT_SNP0W         0x00000002
#define EN_SNP1W            0x00000004
#define ENINT_SNP1W         0x00000008
#define ENABLE_SNP0         0x00000003
#define DISENABLE_SNP0      0xFFFFFFFC
#define ENABLE_SNP1         0x0000000C 
#define DISENABLE_SNP1      0xFFFFFFF3 
#define DISENABLE_SNP_ALL   0xFFFFFFF0

#define SNP_INT_STATUS_REG LPC_HICR6_REG
#define INT_SNP0      0x00000001 
#define INT_SNP1      0x00000002 
#define INT_SNP_ALL   0x00000003  

#define DeviceSNP_Unused 0 
#define DeviceSNP0_ID 0x01 
#define DeviceSNP1_ID 0x02 

#define MAX_REPEAT 10
volatile UINT8 u8LastDate[IO_NUMBER]={0,0};
volatile int iRepeatCount[IO_NUMBER]={0,0};
  
/* driver name will passed by insmod command */
static char *driver_name="aess_biospostdrv";

/* Linux device data structure */
dev_t biospost_dev;
struct cdev *biospost_cdev;
static spinlock_t S_spinlock;


/******************************************************************************
*   FUNCTION        :   aess_biospost_open
******************************************************************************/
/**
 *  @brief      The function is invoked when application open driver note.
 *
 *  @return     STATUS_OK never fail.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function \n
 *
 *****************************************************************************/
static int aess_biospost_open(
                             /** inode structure point */
                             struct inode *inode, 

                             /** file structure point */
                             struct file *filp
                              )
{
    /* There is nothing need to do */
    DRV_INFO("biospost_open():: return(STATUS_OK)\n" ); 
    return (STATUS_OK);  
}


/******************************************************************************
*   FUNCTION        :   aess_biospost_close
******************************************************************************/
/**
 *  @brief      The function is invoked when application close the device note.
 *
 *  @return     STATUS_OK never fail.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function \n
 *
 *****************************************************************************/
static int aess_biospost_close(
                             /** inode structure point */
                             struct inode *inode, 

                             /** file structure point */
                             struct file *filp
                              )
{
    sBIOSPostStruct *pu8Data = (sBIOSPostStruct *)filp->private_data;
    UINT32 u32Tmp   =0 ;
    UINT32 u32Debug1 = 0;
    UINT32 u32Debug2 = 0;
    UINT32 u32Debug3 = 0;
    
    if ( pu8Data == NULL)  
    {
        /* return the applications except which call inital Snoop1 0/1 success that record application's ID */
        DRV_ERROR("biospost_close():: invalid private data.\n");       
        return 0;
    }
    
    /* if the appllcation's ID match  */   
    if( &S_aBIOSPostData[0] == pu8Data )
    {
        /* disable Snooping#0 and Snooping Interrupt */
        u32Tmp = readl( SNP_CTRL_REG ) ;
        iowrite32( ( DISENABLE_SNP0 & u32Tmp ) ,SNP_CTRL_REG );

        /* clear snooping address#0 interrupt status */
        u32Tmp = ioread32 ( SNP_INT_STATUS_REG );
        iowrite32( INT_SNP0 ,  SNP_INT_STATUS_REG );            

        memset( &S_aBIOSPostData[0], 0 , sizeof(sBIOSFIFOStruct) );
    
        DRV_URGENT("biospost_close()::SnoopingAddress#0 \n");
    }

    if( &S_aBIOSPostData[1] == pu8Data )
    {
        /* disable Snooping#1 and Snooping Interrupt */
        u32Tmp = readl( SNP_CTRL_REG ) ;
        iowrite32( ( DISENABLE_SNP1 & u32Tmp ) ,SNP_CTRL_REG );

        /* clear snooping address#0 interrupt status */
        u32Tmp = readl ( SNP_INT_STATUS_REG );
        iowrite32( INT_SNP1 ,  SNP_INT_STATUS_REG );            

        memset( &S_aBIOSPostData[1], 0 , sizeof(sBIOSFIFOStruct) );
        DRV_URGENT("biospost_close()::SnoopingAddress#1 \n" );
    }
    
    /* clear driver private data */
    filp->private_data = NULL ; 

    /* debug message */
    u32Debug1 = ioread32( SNP_CTRL_REG );
    u32Debug2 = ioread32( SNP_INT_STATUS_REG );
    u32Debug3 = ioread32( LPC_SNPWADR_REG );
    DRV_URGENT("SNP_CTRL_REG= 0x%08x , SNP_INT_STATUS_REG=0x%08x, LPC_SNPWADR_REG=0x%08x \n",
                (unsigned int)u32Debug1,(unsigned int)u32Debug2 ,(unsigned int)u32Debug3 );
    
    DRV_INFO("biospost_close():: return(STATUS_OK)\n" ); 
    return (STATUS_OK);
}


/******************************************************************************
*   FUNCTION        :   aess_biospost_drv_init
******************************************************************************/
/**
 *  @brief      Command handler for AESS_ BIOSPOSTDRV_INIT, used to initialize the I/O address.
 *
 *  @return     STATUS_OK never fail.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Internal function \n
 *
 *****************************************************************************/
static int aess_biospost_drv_init(
                                  /** piont to sBIOSPostInfo structure comes from ioctl fucntion.*/
                                  sBIOSPostInfo *psInfo,
                                  
                                  /** file structure point */
                                  struct file *filp 
                                  )
{
    UINT32 u32Debug1 = 0;
    UINT32 u32Debug2 = 0;
    UINT32 u32Debug3 = 0;
    UINT32 u32Tmp = 0;
    int    i = 0  ;

    if( (sBIOSPostStruct *)filp->private_data !=NULL )
    {
        /* the fd already be occupied */ 
        DRV_INFO("biospost_drv_init()::return (-EACCES):: fd already in used.\n" );
        return -EACCES;
    }
 
    for( i = 0 ; i < IO_NUMBER ; i++ )
    {
        if( S_aBIOSPostData[i].u8IOAddrInitFlag == SNP_NOT_INITIALED )
        {
            S_aBIOSPostData[i].u8IOAddrInitFlag = SNP_INITIALED ;         
            S_aBIOSPostData[i].u8BIOSPostAddressLSB = psInfo->u8BIOSPostAddressLSB;
            S_aBIOSPostData[i].u8BIOSPostAddressMSB = psInfo->u8BIOSPostAddressMSB;
            S_aBIOSPostData[i].sIOAddrBuf.u16In =0;
            S_aBIOSPostData[i].sIOAddrBuf.u16Out =0;
            S_aBIOSPostData[i].sIOAddrBuf.u16Size =0;
            
            if( i == 0 )
            {
                /* Device Snooping_Address#0 */ 
                psInfo->u8IOAddrEnFlag = DeviceSNP0_ID ;
                
                /* Set the address to Snooping_Address#0 */
                u32Tmp = ioread32( LPC_SNPWADR_REG );
                u32Tmp &= 0xFFFF0000 ;
                u32Tmp |= (S_aBIOSPostData[i].u8BIOSPostAddressMSB<<8) ;
                u32Tmp |= S_aBIOSPostData[i].u8BIOSPostAddressLSB ;
                iowrite32( u32Tmp , LPC_SNPWADR_REG );

                /* enable Snooping_Address#0 and interrupt */                                    
                u32Tmp = ioread32( SNP_CTRL_REG );
                iowrite32( u32Tmp | ENABLE_SNP0 ,SNP_CTRL_REG );
            }
                
            if( i == 1 )
            {
                /* Device Snooping_Address#0 */ 
                psInfo->u8IOAddrEnFlag = DeviceSNP1_ID ;

                /* Set the address to Snooping_Address#1 */
                u32Tmp = ioread32( LPC_SNPWADR_REG );
                u32Tmp &= 0x0000FFFF ;
                u32Tmp |= (S_aBIOSPostData[i].u8BIOSPostAddressMSB<<24) ;
                u32Tmp |= (S_aBIOSPostData[i].u8BIOSPostAddressLSB<<16) ;
                iowrite32( u32Tmp , LPC_SNPWADR_REG );
                
                /* enable Snooping_Address#1 and interrupt */                                    
                u32Tmp = ioread32( SNP_CTRL_REG );
                iowrite32( u32Tmp | ENABLE_SNP1 ,SNP_CTRL_REG );
            }
            
            /* store the snooping device index to caller */
            filp->private_data =&S_aBIOSPostData[i] ;

            /* debug message */
            u32Debug1 = ioread32( SNP_CTRL_REG );
            u32Debug2 = ioread32( SNP_INT_STATUS_REG );
            u32Debug3 = ioread32( LPC_SNPWADR_REG );
            DRV_URGENT("biospost_drv_init()::SnoopingAddress#%d \n", i);
            DRV_URGENT("SNP_CTRL_REG= 0x%08x, SNP_INT_STATUS_REG= 0x%08x, LPC_SNPWADR_REG= 0x%08x \n",
                        (unsigned int)u32Debug1, (unsigned int)u32Debug2 , (unsigned int)u32Debug3 );

            DRV_INFO("biospost_drv_init(): init Snooping_Address#%d)::STATUS_OK \n",i );
            return 	STATUS_OK;
        }
    }
    return -EACCES;
}


/******************************************************************************
*   FUNCTION        :   aess_biospost_read
******************************************************************************/
/**
 *  @brief      Command handler for AESS_BIOSPOSTDRV_READ uses to pass the incoming data to user space.
 *
 *  @return     STATUS_OK if read data and transfer it to user space success.
 *              Others if read or transfer data fail.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Internal function \n
 *
 *****************************************************************************/	
static int aess_biospost_read(
                              /** point to sBIOSPostInfo pass comes from ioctl function */  
                              sBIOSPostInfo *psInfo, 
                             
                             /** file structure point */
                              struct file *filp
                              )
{
    UINT32 u32Index = 0;
    UINT16 u16Len = 0;
    UINT16 i=0;
    UINT16 u16tmp1, u16tmp2;

    for(i = 0 ; i < IO_NUMBER; i++)
    {
        /* check match caller and the Snooping_Address#n */
        if( S_aBIOSPostData[i].u8IOAddrInitFlag == SNP_INITIALED && 
           (sBIOSPostStruct *)filp->private_data == &S_aBIOSPostData[i]  )
        {
            if( S_aBIOSPostData[i].sIOAddrBuf.u16Size == 0 ||  
                psInfo->u16MaxReadLen == 0 )
            {
                /* if data_len ==0 or read length ==0 */
                psInfo->u16CopyLen = 0;
            }        
            else
            {
                /* use the minimum value */
                u16Len = min(S_aBIOSPostData[i].sIOAddrBuf.u16Size, psInfo->u16MaxReadLen);
                
                u32Index = S_aBIOSPostData[i].sIOAddrBuf.u16Out;

                if( (S_aBIOSPostData[i].sIOAddrBuf.u16Out+u16Len ) > (BIOSPOST_KFIFO_SIZE) )
                {
                    /* if index over 1K size */
                    
                    u16tmp1 = BIOSPOST_KFIFO_SIZE - u32Index ;// the date that index value under 1K
                    u16tmp2 = u16Len - u16tmp1; // // the date that index value over 1K 
                    
                    /* copy data */
                    memcpy( psInfo->pu8Data , S_aBIOSPostData[i].sIOAddrBuf.au8Buffer+u32Index,u16tmp1 );
                    memcpy( psInfo->pu8Data+u16tmp1 , S_aBIOSPostData[i].sIOAddrBuf.au8Buffer,u16tmp2 );
                    
                    /* adjust the output data index */
                    S_aBIOSPostData[i].sIOAddrBuf.u16Out = u16tmp2 ;
                }
                else
                {
                    /* copy data */
                    memcpy( psInfo->pu8Data , S_aBIOSPostData[i].sIOAddrBuf.au8Buffer+u32Index, u16Len );                   

                    /* adjust the output data index */
                    S_aBIOSPostData[i].sIOAddrBuf.u16Out += u16Len ;
                }
                S_aBIOSPostData[i].sIOAddrBuf.u16Size -= u16Len ;
                
                psInfo->u16CopyLen =  u16Len ;
            }
            
            /* debug message */
            DRV_URGENT("biospost_read()::SnoopingAddress#%d \n", i);
            DRV_URGENT("DataSize=%d, u16In=%d, u16Out=%d ,u16Len=%d \n",
                        S_aBIOSPostData[i].sIOAddrBuf.u16Size , S_aBIOSPostData[i].sIOAddrBuf.u16In,
                        S_aBIOSPostData[i].sIOAddrBuf.u16Out , u16Len );
            
        	DRV_INFO("biospost_read()::Snooping_Address#%d::return( STATUS_OK)\n",i ); 	
            return (STATUS_OK);                
        }
    } 
    
    /* No match  date :: (sBIOSPostStruct *)filp->private_data == &S_aBIOSPostData[i] */ 
    DRV_ERROR("biospost_read()::return( STATUS_FAIL)::Unknow Snooping_Address device \n" ); 	
    return (STATUS_FAIL);
}


/******************************************************************************
*   FUNCTION        :   aess_biospost_reset
******************************************************************************/
/**
 *  @brief      Command handler for AESS_BIOSPOSTDRV_RESET uses to clean biospost FIFO.
 *
 *  @return     STATUS_OK if read data and transfer it to user space success.
 *              Others if read or transfer data fail.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function \n
 *
 *****************************************************************************/	
static int aess_biospost_reset(
                              /** point to sBIOSPostInfo pass comes from ioctl function */  
                              sBIOSPostInfo *psInfo, 
                             
                             /** file structure point */
                              struct file *filp

                               )
{
    int i ;
    for(i = 0 ; i < IO_NUMBER; i++)
    {
        /* check match caller and the Snooping_Address#n */
        if( S_aBIOSPostData[i].u8IOAddrInitFlag == SNP_INITIALED && 
           (sBIOSPostStruct *)filp->private_data == &S_aBIOSPostData[i]  )
        {
            /* reset the input index */
            S_aBIOSPostData[i].sIOAddrBuf.u16In = 0;
            
            /* reset the output index */
            S_aBIOSPostData[i].sIOAddrBuf.u16Out = 0;
            
            /* reset the buffer size */
            S_aBIOSPostData[i].sIOAddrBuf.u16Size = 0;
            
            DRV_URGENT("biospost_reset()::psInfo->u8IOAddrEnFlag=%d :: SNP#%d ",psInfo->u8IOAddrEnFlag,i);
            DRV_INFO("biospost_reset()::return( STATUS_OK)\n" ); 	
            return (STATUS_OK);
        }
    }
    /* IO Address invalid */
    DRV_ERROR("biospost_reset()::return(-EINVAL )::Unknow Snooping_Address device \n" ); 	
    return ( -EINVAL );
}


/******************************************************************************
*   FUNCTION        :   aess_biospost_clearall
******************************************************************************/
/**
 *  @brief      Clean the S_aBIOSPostData structure and un-register the IRQ .
 *
 *  @return     STATUS_OK never fail.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    Before remove kcs driver , it's necessary to free irq .
 *
 *  @note       None
 *
 *  @internal   Function Type: Internal function \n
 *
 *****************************************************************************/
static int aess_biospost_clearall( void )
{
    UINT32 u32Debug1 = 0;
    UINT32 u32Debug2 = 0;
    UINT32 u32Debug3 = 0;    
    UINT32 u32Tmp = 0;
    
    /* disable Snooping_Address#0~1 and interrupt */
    u32Tmp = ioread32( SNP_CTRL_REG );
    iowrite32( u32Tmp & DISENABLE_SNP_ALL , SNP_CTRL_REG );
    
    /* clear Snooping_Address#0~1 interrupt status */
    u32Tmp = ioread32( SNP_INT_STATUS_REG );
    iowrite32( u32Tmp | INT_SNP_ALL , SNP_INT_STATUS_REG );
    
    /* Set the initial value for data structure */
    memset(S_aBIOSPostData, 0, sizeof(sBIOSPostStruct) * IO_NUMBER);  
    
    /* Release IRQ */
    free_irq( IRQ_LPC, (void *) &S_aBIOSPostData);
	
    /* debug message */
    u32Debug1 = ioread32( SNP_CTRL_REG );
    u32Debug2 = ioread32( SNP_INT_STATUS_REG );
    u32Debug3 = ioread32( LPC_SNPWADR_REG );
    DRV_URGENT("biospost_clearall() \n");
    DRV_URGENT("SNP_CTRL_REG=0x%08x, SNP_INT_STATUS_REG=0x%08x, LPC_SNPWADR_REG= 0x%08x\n",
                (unsigned int)u32Debug1,(unsigned int)u32Debug2 ,(unsigned int)u32Debug3 );
	
    DRV_INFO("biospost_clearall()::return( STATUS_OK)\n" ); 	
    return (STATUS_OK);
}


/******************************************************************************
*   FUNCTION        :   aess_biospost_ioctl
******************************************************************************/
/**
 *  @brief      The API is invoked when user space application issues an IOCTL command. 
 *              The following is the list of commands that supported currently.
 *              1.	AESS_BIOSPOSTDRV_INIT
 *                  Initialize Linux BIOS Post Codes driver according to the given parameters.
 *              2.	AESS_BIOSPOSTDRV_READ
 *                  Copy data from driver to user space application.
 *              3.	AESS_BIOSPOSTDRV_RESET
 *                  Reset index and size of a data buffer to zero.
 *
 *  @return     STATUS_OK if driver execute the command success.
 *              Others if execute the command fail.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function \n
 *
 *****************************************************************************/
static int aess_biospost_ioctl(
                              /** inode structure point */
                              struct inode *inode, 
           
                              /** file structure point */
                              struct file *filp,
           
                              /** command from user space */
                              unsigned int cmd, 
    
                              /** sBIOSPostInfo structure point */
                              unsigned long arg
                              )
{   
    int err_check;
    sBIOSPostInfo *psInfo = (sBIOSPostInfo *) arg;

    /* Check the structure address is OK */
    if(!access_ok(VERIFY_WRITE, (void *) psInfo, sizeof(sBIOSPostInfo)))
    {
        /* error occur */
        DRV_ERROR("biospost_ioctl():: struct access error.\n");       
        return -EFAULT;
    }
  
    switch(cmd)
    {
        case AESS_BIOSPOSTDRV_INIT:
            err_check = aess_biospost_drv_init(psInfo, filp);         
            break;
    
        case AESS_BIOSPOSTDRV_READ:
            err_check = aess_biospost_read(psInfo, filp);
            break;

        case AESS_BIOSPOSTDRV_RESET:
            err_check = aess_biospost_reset(psInfo, filp);
            break;

        default:
            DRV_ERROR("biospost_ioctl()::cmd(Unknown)::return( -EINVAL)\n" ); 	
            err_check = -EINVAL;
            break;
    }
    
    /* 0->ok, minus->fail */
    return err_check;
}


/******************************************************************************
*   FUNCTION        :   aess_biospost_isr
******************************************************************************/
/**
 *  @brief      The function is the entry function of interrupt handler
 *
 *  @return     IRQ_NONE if the interrupt not occured biospost .
 *              IRQ_HANDLED if biospsot interrupt occured.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Internal function \n
 *
 *****************************************************************************/
static irqreturn_t aess_biospost_isr (
                                      /** irq number  */
                                      int irq, 
                                      
                                      /**  point to current dev data structure  */
                                      void *dev_id
                                      )
{
    volatile unsigned long u32Flag ;
    volatile UINT32 u32Status;
    volatile UINT32 u32index;
    volatile UINT16 u16data ;
    volatile UINT8  u8data;
    volatile UINT32 u32Tmp;
    volatile int SNP_Index ;
    //int iStopInt[IO_NUMBER]={0,0};
    
    
    /* get the inetrrupt status */
    u32Status = ioread32( SNP_INT_STATUS_REG ) ;
    
    /* check the inetrrupt status */
    if( ( (u32Status & INT_SNP0) ==0 ) && ( (u32Status&INT_SNP1)==0 ) )
    {
        /* No Snooping_Address#0 interrupt arrived */
        DRV_INFO("biospost_isr()::return( IRQ_NONE)::u32Status= 0x%08x\n", (unsigned int)u32Status ); 	    
        return IRQ_NONE;
    }

    /* locked */
    spin_lock_irqsave(&S_spinlock, u32Flag);  
    
    for( SNP_Index = 0 ; SNP_Index < IO_NUMBER; SNP_Index++ )
    {
        if( (u32Status & (0x1<<SNP_Index) )== INT_SNP0 ) 
        {
            /* get data */
            u16data = ioread16( LPC_SNPWDR_REG );
            u16data &= 0x00FF;
            u8data = (UINT8) u16data ;
                            
            /* clear interrupt status */
            iowrite32( u32Status | INT_SNP0 , SNP_INT_STATUS_REG );
        }        
        else if( (u32Status & (0x1<<SNP_Index))== INT_SNP1 )
        {
            /* get data */
            u16data = ioread16( LPC_SNPWDR_REG );
            u16data = (u16data>>8) ;
            u8data = (UINT8) u16data ;
        
            /* clear interrupt status */
            iowrite32( u32Status | INT_SNP1 , SNP_INT_STATUS_REG );
        }
        else
            continue;
            
        /* if data repeat over than checkpoint , stop the interrupt*/
        if( u8LastDate[SNP_Index] == u8data )
        {
            if( ++iRepeatCount[SNP_Index] >= MAX_REPEAT )
            {
                u32Tmp = readl( SNP_CTRL_REG ) ;
                
                if( SNP_Index ==0 )
                    iowrite32( ( DISENABLE_SNP0 & u32Tmp ) ,SNP_CTRL_REG );
                else if( SNP_Index ==1 )
                    iowrite32( ( DISENABLE_SNP1 & u32Tmp ) ,SNP_CTRL_REG );
            }
        }
        else
        {
            u8LastDate[SNP_Index] = u8data; 
            iRepeatCount[SNP_Index] = 0;
        }


        if( S_aBIOSPostData[SNP_Index].u8IOAddrInitFlag  != SNP_INITIALED )
        {
            /* No Snooping_Address#0 interrupt arrived */
            DRV_INFO("biospost_isr():: SNP_NOT_INITIALED->ad#=%d::return(IRQ_NONE)::u32Status=0x%08x\n",
                                                                	          SNP_Index ,(unsigned int)u32Status ); 	    
        	
            /* unlocked */
            spin_unlock_irqrestore(&S_spinlock, u32Flag);
        	
            return IRQ_NONE;
        }
        
        /* check the data size is over max. size */    
        if(S_aBIOSPostData[SNP_Index].sIOAddrBuf.u16Size == BIOSPOST_KFIFO_SIZE)
        {
            /* increase the ouput index */ 
            S_aBIOSPostData[SNP_Index].sIOAddrBuf.u16Out++;

            /* adjust the output index when it over max. size.*/
            S_aBIOSPostData[SNP_Index].sIOAddrBuf.u16Out %= BIOSPOST_KFIFO_SIZE;
            
            /* prepare operation for increasing size later */
            S_aBIOSPostData[SNP_Index].sIOAddrBuf.u16Size--;   
        }
        
        u32index =  S_aBIOSPostData[SNP_Index].sIOAddrBuf.u16In ;
        
        /* fill data */
        S_aBIOSPostData[SNP_Index].sIOAddrBuf.au8Buffer[u32index]= u8data;
        
        /* increase input index */
        S_aBIOSPostData[SNP_Index].sIOAddrBuf.u16In++;

        /* adjust the input index when it over max. size.*/
        S_aBIOSPostData[SNP_Index].sIOAddrBuf.u16In %= BIOSPOST_KFIFO_SIZE;
    
        /* increase data size */
        S_aBIOSPostData[SNP_Index].sIOAddrBuf.u16Size++;
    
        /* debug message */        
        DRV_URGENT("biospost_isr()::SnoopingAddress#%d \n", SNP_Index);
        DRV_URGENT("DataSize=%d, u16In=%d, u16Out=%d \n",
                    S_aBIOSPostData[SNP_Index].sIOAddrBuf.u16Size , S_aBIOSPostData[SNP_Index].sIOAddrBuf.u16In,
                    S_aBIOSPostData[SNP_Index].sIOAddrBuf.u16Out );
    }    
    
    /* unlocked */
    spin_unlock_irqrestore(&S_spinlock, u32Flag);
    
    /* retuen */
    DRV_INFO("biospost_isr()::return(IRQ_HANDLED):: done.\n"); 	    
    return IRQ_HANDLED;
    
}

struct file_operations aess_biospost_fops = {
    .open = aess_biospost_open, 
    .ioctl = aess_biospost_ioctl,
    .release = aess_biospost_close,        
};


/******************************************************************************
*   FUNCTION        :   aess_biospost_init
******************************************************************************/
/**
 *  @brief      The function is invoked when biospost module is loaded into the kernel.
 *
 *  @return     STATUS_OK if driver initial success.
 *              Others if initial fail.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function  \n
 *
 *****************************************************************************/
int __init aess_biospost_init(void)
{
    int result;
    biospost_cdev = cdev_alloc();
    
    biospost_cdev->ops = &aess_biospost_fops;
    
    /* allocate device numbers */
    result = alloc_chrdev_region(&biospost_dev, 0, 1, driver_name);
    
    if (result < 0) 
    {
        printk (KERN_ERR "KN:Registering the BIOSPOST device failed with %d\n", MAJOR(biospost_dev));
        return result;
    }
    
    cdev_add(biospost_cdev, biospost_dev, 1);
    
    /* Register IRQ*/
    if (request_irq( IRQ_LPC, aess_biospost_isr,IRQF_SAMPLE_RANDOM|IRQF_SHARED,"BIOSPOST", (void *) &S_aBIOSPostData)) 
    {
        printk (KERN_ERR "KN:Registering the BIOSPOST IRQ failed with %d\n", MAJOR(biospost_dev));
        return -EBUSY;
    }  

    /* initial S_spinlok */
    spin_lock_init(&S_spinlock);
    
    /* initial S_aBIOSPostData data structure */
    memset(S_aBIOSPostData, 0, sizeof(sBIOSPostStruct)*IO_NUMBER);
 
    printk("mknod /dev/aess_biospostdrv c %d 0\n", MAJOR(biospost_dev));
    
    /* initial debugfs related resource */
    aess_debugfs_default_create(driver_name,NULL,0,0);
    aess_debugfs_create_file(driver_name,"u32",(u32 *)&u32tmpcounter,DBG_TYPE32,0);
    aess_debugfs_create_file(driver_name,"u16",(u16 *)&u16tmpcounter,DBG_TYPE16,0);
    aess_debugfs_create_file(driver_name,"u8",(u8 *)&u8tmpcounter,DBG_TYPE8,0);	
        
    DRV_INFO("driver_init()::return( STATUS_OK)\n" ); 	    
    return (STATUS_OK);
}


/******************************************************************************
*   FUNCTION        :   aess_biospost_exit
******************************************************************************/
/**
 *  @brief      The function is invoked when biospost driver be removed from kernal.
 *
 *  @return     None.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function \n
 *
 *****************************************************************************/

static void aess_biospost_exit(void)
{
    /* disable Snoopring_Address#n */
    aess_biospost_clearall();
   
    /* remove device driver */
    cdev_del(biospost_cdev);
    
    /* unregister device driver */
    unregister_chrdev_region(biospost_dev, 1);    
    
    /* remove debugfs module */
    aess_debugfs_remove(driver_name,NULL,0,0);
    
    DRV_INFO("driver_exit()::return( STATUS_OK)\n" ); 	 
}

MODULE_DESCRIPTION("BIOSPOST Driver");
MODULE_AUTHOR("Taiyi Kuo<taiyi.kuo@avocent.com>");
MODULE_LICENSE("GPL");
MODULE_VERSION("0.1.0.0");
module_param(driver_name, charp, S_IRUGO);
module_init(aess_biospost_init);
module_exit(aess_biospost_exit);

/* End of code */
