/**
 * $Id$
 *
 *<center>
 *               Avocent Corporation. Proprietary Information.
 * \n<em>
 *      This software is supplied under the terms of a license agreement or
 *      nondisclosure agreement with Avocent Corporation, or  one of its 
 *      affiliates, andaffiliates, and may not be used, disseminated, or 
 *      distributed except in accordance with the terms of that agreement.
 *
 *      3541 Gateway Boulevard Fremont, San Jose, CA 94538 U.S.A.
 *\n
 *                  US phone: 510-771-6100
 *
 *        Copyright &copy; 2001-2009 Avocent Corporation. All rights reserved.
 *</em> </center>
 *----------------------------------------------------------------------------\n
 *  MODULES     Linux device Driver                                           \n
 *----------------------------------------------------------------------------\n
 *  @file   aess_biospostdrv.h
 *  @brief  biospost driver and interrupt service routine
 *  @internal
 *  $RCSfile: aess_biospostdrv.h,v $
 *  $Revision: 1.1.1.1 $
 *  $Date: 2009/03/10 13:44:48 $
 *  $Author: Taiyi Kuo $
 *  $Source: //tpe-subversion/AESSTPE/puti/puti_fw/ProjectBase/trunk/platform/ast2050/evb/src/drivers/biospost/biospost_driver,v $
 *  $Name:  $
 *----------------------------------------------------------------------------*/
 
#ifndef AESSBIOSPOSTDRV_H
#define AESSBIOSPOSTDRV_H

/* IOCTL command */
#define AESS_BIOSPOSTDRV_IOC_MAGIC    0xCF
#define AESS_BIOSPOSTDRV_INIT         _IOWR(AESS_BIOSPOSTDRV_IOC_MAGIC, 0, int)
#define AESS_BIOSPOSTDRV_READ         _IOWR(AESS_BIOSPOSTDRV_IOC_MAGIC, 1, int)
#define AESS_BIOSPOSTDRV_RESET        _IOWR(AESS_BIOSPOSTDRV_IOC_MAGIC, 2, int)


/******************************************************************************
*   STRUCT      :    sBIOSPostInfo
******************************************************************************/
/**
 *  @brief    BIOS Post Parameters definition from User space
 *
 *****************************************************************************/
typedef struct 
{
  /** the Max. data length can copy to */
  UINT16 u16MaxReadLen;
  
  /**  data size copied to user space */
  UINT16 u16CopyLen;    
  
  /** BIOS Post Codes FIFO address(LSB) */
  UINT8 u8BIOSPostAddressLSB;
  
  /** BIOS Post Codes FIFO address(MSB) */
  UINT8 u8BIOSPostAddressMSB;
  
  /** Snooping addre#n index : 1-> snooping_address#0 ; 2->snooping_address#1. */
  UINT8 u8IOAddrEnFlag;
  
  /** Reserved */
  UINT8 u8Reserved;
  
  /** Data buffer one byte length */
  UINT8 *pu8Data;
		
} sBIOSPostInfo;

#define SNP_NOT_INITIALED   0 
#define SNP_INITIALED 1 


#ifdef AESSBIOSPOSTDRV_C 


#define IO_NUMBER           0x2

/* BIOS interface package and structure definition */
//#define BIOSPOST_KFIFO_SIZE     0x400
#define BIOSPOST_KFIFO_SIZE     0x100
#define BIOSPOST_KFIFO_SIZE_MINUS_1     (BIOSPOST_KFIFO_SIZE-1)


/* For temporary use, because no type.h now */
#define STATUS_OK     0
#define STATUS_FAIL   1


/******************************************************************************
*   STRUCT      :   sBIOSFIFOStruct
******************************************************************************/
/**
 *  @brief   Structure to BIOS FIFO driver related buffer, 
 *           used by linux driver.
 *
 *****************************************************************************/
typedef struct{

  /** data is input index */
  UINT16 u16In;	
  
  /** data is output index */
  UINT16 u16Out;	
  
  /** data size */
  UINT16 u16Size;	
  
  /** Reserved */
  UINT16 Reserved;	
  
  /** data buffer */
  UINT8 au8Buffer[BIOSPOST_KFIFO_SIZE];

} sBIOSFIFOStruct;


/******************************************************************************
*   STRUCT      :   sBIOSPostStruct
******************************************************************************/
/**
 *  @brief   Structure to BIOS driver related data parameter, 
 *           used by linux driver.
 *
 *****************************************************************************/
typedef struct 
{
  /** File ID*/
  UINT8 u8FileID;
  
  /** Record whether IO Address initialized */
  UINT8  u8IOAddrInitFlag;
  
  /** BIOS Post Codes FIFO address(LSB) */
  UINT8 u8BIOSPostAddressLSB;
  
  /** BIOS Post Codes FIFO address(MSB) */
  UINT8 u8BIOSPostAddressMSB;
  
  /** FIFO for I/O port Address */
  sBIOSFIFOStruct sIOAddrBuf;

}sBIOSPostStruct;

/* Declear BIOS Post Codes data structure */
static sBIOSPostStruct S_aBIOSPostData[IO_NUMBER];

#endif   /* AESSBIOSPOSTDRV_C */
#endif   /* AESSBIOSPOSTDRV_H */

/* End of code */
