/**
 * $Id$
 *
 *<center>
 *               Avocent Corporation. Proprietary Information.
 * \n<em>
 *      This software is supplied under the terms of a license agreement or
 *      nondisclosure agreement with Avocent Corporation, or  one of its 
 *      affiliates, andaffiliates, and may not be used, disseminated, or 
 *      distributed except in accordance with the terms of that agreement.
 *
 *      3541 Gateway Boulevard Fremont, San Jose, CA 94538 U.S.A.
 *\n
 *                  US phone: 510-771-6100
 *
 *        Copyright &copy; 2001-2009 Avocent Corporation. All rights reserved.
 *</em> </center>
 *----------------------------------------------------------------------------\n
 *  MODULES     Linux device Driver                                           \n
 *----------------------------------------------------------------------------\n
 *  @file   aess_dynairqdrv.h
 *  @brief  KCS driver and interrupt service routine
 *  @internal
 *  $RCSfile: aess_dynairqdrv.h,v $
 *  $Revision: 0.1.0.0 $
 *  $Date: 2009/03/10 13:44:48 $
 *  $Author: Taiyi Kuo $
 *  $Source: $
 *  $Name:  $
 *----------------------------------------------------------------------------*/
 
 
#if !defined (AESSDYNAIRQDRV_H)
#define AESSDYNAIRQDRV_H

#define	GPIO_MAX_PIN_NUM   64

/* ioctl definitions */
#define AESS_IRQDRV_IOC_MAGIC		0xB9
#define AESS_IOCTL_DRIVER_INIT      _IOWR(AESS_IRQDRV_IOC_MAGIC, 0, int)
#define AESS_IOCTL_DYNAIRQ_INIT     _IOWR(AESS_IRQDRV_IOC_MAGIC, 1, int)
#define AESS_IOCTL_DYNAIRQ_CONFIG   _IOWR(AESS_IRQDRV_IOC_MAGIC, 2, int)
#define AESS_IOCTL_DYNAIRQ_CLEAR    _IOWR(AESS_IRQDRV_IOC_MAGIC, 3, int)
#define AESS_IOCTL_GENEISR_INIT     _IOWR(AESS_IRQDRV_IOC_MAGIC, 4, int)
#define AESS_IOCTL_USERMODE_ISRID   _IOWR(AESS_IRQDRV_IOC_MAGIC, 5, int)


enum INT_TRIGGER_INFO {
	INTT_RISING_EDGE =0,	/* No use in vitesse */
	INTT_FALLING_EDGE,
	INTT_HIGH_LEVEL,
	INTT_LOW_LEVEL ,
	INTT_BOTH_EDGE
};


/******************************************************************************
*   STRUCT       :   sIRQInfoType
******************************************************************************/
/**
 *  @brief   Dynamic IRQ Paramters Structure
 *
 *****************************************************************************/
typedef struct
{
    
    union
    /** Parameter2 */
    {
        
        /** data */
        ushort data;

        /** IRQ number */
        ushort u16IRQNum;
    }param1;
    
    /** Parameter2 */
    union
    {
        /** data */
        ushort data;

        /** Reserved */
        ushort u16Reserved;
    }param2;

    /** Parameter3 */
    UINT32 param3;

    /** ISR name */
    char *pISRName;

} sIRQInfoType;


#ifdef AESSDYNAIRQDRV_C

#include <linux/types.h>
#include <asm/types.h>
#include <asm/ioctl.h>
#include <linux/interrupt.h>
#include <linux/spinlock.h>
#include <asm/arch/irqs.h>
#include <asm-arm/arch-ast2050/hw_regs.h>

#ifdef DIRQ_UNITTEST
#define AESS_IOCTL_GENEISR_TEST     _IOWR(AESS_IRQDRV_IOC_MAGIC, 10, int)
#endif

/* Dynamic IRQ List Structure */
#ifdef	__KERNEL__

/******************************************************************************
*   STRUCT       :   sIrqNum
******************************************************************************/
/**
 *  @brief   sIrqNu
 *
 *****************************************************************************/
struct sIrqNum
{
    /** point to next*/
    struct sIrqNum *next;

    /** irq number*/
    int irq_num;
};


/******************************************************************************
*   STRUCT       :   sUserModeRecordID
******************************************************************************/
/**
 *  @brief   sUserModeRecordID
 *
 *****************************************************************************/
typedef struct
{
    /** irq number */    
    UINT16 u16NumIRQ;
    
    /** event id */    
    UINT32 u32EventID;
    
} sUserModeRecordID;


/******************************************************************************
*   STRUCT       :   sIRQList
******************************************************************************/
/**
 *  @brief   sIRQList
 *
 *****************************************************************************/
typedef struct
{
    /** point to struct sIrqNum*/
    struct sIrqNum *pstIRQNum;
    
    /** device name */
    const char * devname;

    /** device id */
    void * dev_id;
    
    /** irqflags */
    unsigned long irqflags;
    
    /** interrupt service routing */
    irqreturn_t (*handler)(int, void *, struct pt_regs *);

} sIRQList;

#define MAX_DIRQ_LIST	50

int aess_register_dynamic_irq_list(const char * devname,
								   void * dev_id,
								   unsigned long irqflags,
								   irqreturn_t (*handler)(int, void *, struct pt_regs *)
								   );

int aess_dynamic_irq_int_clear(int irq);

#ifdef DIRQ_DEBUG
void aess_show_dynamic_irq_list();
int aess_call_dynamic_irq(int);
#endif

#endif /* __KERNEL__ */

#endif /* AESSDYNAIRQDRV_C */

#endif   /* AESSDYNAIRQDRV_H */


/* End of code */
