/**
 * ASPEED 2050 I2C driver.
 * Copyright (C) 2008 Avocent Corp.
 * Avocent Corporation. Proprietary Information.
 *
 * This file is subject to the terms and conditions of the GNU 
 * General Public License. This program is distributed in the hope 
 * that it will be useful, but WITHOUT ANY WARRANTY; without even 
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 *
 *
 *----------------------------------------------------------------------------\n
 *  MODULES     Linux device Driver                                           \n
 *----------------------------------------------------------------------------\n
 *  @file   aess_i2cdrv.h
 *  @brief  i2c driver header file
 *
 *  @internal
 * $RCSfile: aess_i2cdrv.h,v $
 * $Revision:  $
 * $Date:  $
 * $Author:  $
 * $Source:  $
 * $Name:  $
 *----------------------------------------------------------------------------*/

#if !defined (AESSI2CDRV_H)
#define AESSI2CDRV_H

#define I2C_DRV_MAX_BUS             7
#define I2C_DRV_MAX_PI2C_BUS        I2C_DRV_MAX_BUS
#define I2C_DRV_MAX_IPMB_BUS        I2C_DRV_MAX_BUS

#define I2C_DRV_BUS_0               0
#define I2C_DRV_BUS_1               1
#define I2C_DRV_BUS_2               2
#define I2C_DRV_BUS_3               3
#define I2C_DRV_BUS_4               4
#define I2C_DRV_BUS_5               5
#define I2C_DRV_BUS_6               6

#define I2C_DRV_MAX_MSG_SIZE        64
#define I2C_DRV_IPMB_MSG_MIN        7

#define I2C_DRV_CLK_STR_WAIT_50US   50

/* definition for I2C frequency */
#define I2C_DRV_FREQ_100    0
#define I2C_DRV_FREQ_400    1
#define I2C_DRV_FREQ_50     2

#define I2C_DRV_TRANS_NORMAL        0
#define I2C_DRV_TRANS_MUX           1
#define I2C_DRV_TRANS_SMBUS_BLK     2

/* definitions for control hardware */
#define I2C_DRV_REC_RESET_SMBUS     0x08
#define I2C_DRV_REC_CLK_PULSE       0x04
#define I2C_DRV_REC_RESET_OUT       0x02
#define I2C_DRV_REC_FORCE_STOP      0x01

/* definitions for I2C CLK line control */
#define I2C_SCL_O                   1 << 12
#define I2C_SDA_O                   1 << 14
/* Output direct control 1:output */
#define I2C_SCL_OE                  1 << 13
#define I2C_SDA_OE                  1 << 15

#define I2C_MASTER_ENABLE           1 << 0
#define I2C_SLAVE_ENABLE            1 << 1

/* IOCTL command */
#define AESS_I2CDRV_IOC_MAGIC       0xB7
#define AESS_I2CDRV_INIT            _IOWR(AESS_I2CDRV_IOC_MAGIC, 0, \
                                          sI2CDrvBusInfoType)
#define AESS_I2CDRV_CONFIG          _IOWR(AESS_I2CDRV_IOC_MAGIC, 1, \
                                          sI2CDrvBusInfoType)
#define AESS_I2CDRV_WR              _IOWR(AESS_I2CDRV_IOC_MAGIC, 2, \
                                          sI2CDrvBufInfoType)
#define AESS_I2CDRV_GET_MSG         _IOWR(AESS_I2CDRV_IOC_MAGIC, 3, \
                                          sI2CDrvBufInfoType)
#define AESS_I2CDRV_RESET           _IOWR(AESS_I2CDRV_IOC_MAGIC, 4, \
                                          sI2CDrvBusInfoType)
#define AESS_I2CDRV_GET_STATUS      _IOWR(AESS_I2CDRV_IOC_MAGIC, 5, \
                                          sI2CDrvBusInfoType)
#define AESS_I2CDRV_GET_HW_STATUS   _IOWR(AESS_I2CDRV_IOC_MAGIC, 6, \
                                          sI2CDrvBusInfoType)
#define AESS_I2CDRV_CTRL_HW         _IOWR(AESS_I2CDRV_IOC_MAGIC, 7, \
                                          sI2CDrvBusInfoType)

/******************************************************************************
*   Enum      :   eI2CDrvErrorType
******************************************************************************/
/**
 *  @brief   I2C driver error enumeration type
 *
 *****************************************************************************/
typedef enum
{
    /** Successful */
    I2C_DRV_OK = 0,
    
    /** General failure */
    I2C_DRV_FAIL,
    
    /** Memory allocation failed */
    I2C_DRV_MEM_FAIL,
    
    /** Memory pool creation failed */
    I2C_DRV_POOL_FAIL,
    
    /** Queue creation failed */
    I2C_DRV_Q_FAIL,
    
    /** Event flags group creation failed */
    I2C_DRV_FLAGS_FAIL,
    
    /** Semaphore creation failed */
    I2C_DRV_SEM_FAIL,
    
    /** Timer creation failed */
    I2C_DRV_TIMER_FAIL
} eI2CDrvErrorType;

/******************************************************************************
*   Enum      :   eI2CDrvErrorStatusType
******************************************************************************/
/**
 *  @brief   I2C driver error status enumeration type
 *
 *****************************************************************************/
typedef enum
{
    /** No errors */
    I2C_DRV_ERROR_NONE = 0,
    
    /** NACK error */
    I2C_DRV_ERROR_NACK,
    
    /** Arbitration loss error */
    I2C_DRV_ERROR_AL,
    
    /** Start error */
    I2C_DRV_ERROR_START,
    
    /** Stop error */
    I2C_DRV_ERROR_STOP,
    
    /** bus busy */
    I2C_DRV_ERROR_BUSY,
    
    /** Timeout error */
    I2C_DRV_ERROR_TIMEOUT,
    
    /** General bus error */
    I2C_DRV_ERROR_BUS,
    
    /** Buffer overflow error */
    I2C_DRV_ERROR_OVRFL,
    
    /** Bus reet error */
    I2C_DRV_ERROR_RESET
} eI2CDrvErrorStatusType;

/******************************************************************************
*   Enum      :   eI2CDrvModeType
******************************************************************************/
/**
 *  @brief   I2C driver mode enumeration type
 *
 *****************************************************************************/
typedef enum
{
    /** Private I2C mode */
    I2C_DRV_MODE_PI2C = 0,
    
    /** IPMB mode */
    I2C_DRV_MODE_IPMB
} eI2CDrvModeType;


/******************************************************************************
*   Enum      :   eI2CDrvCtrlHardwareType
******************************************************************************/
/**
 *  @brief   I2C driver control hardware enumeration type
 *
 *****************************************************************************/
typedef enum
{
    /** Pulse the SCL manually and generate a stop condition */
    I2C_DRV_CLK_PULSE,
    
    /** Generate a stop condition */
    I2C_DRV_FORCE_STOP,
    
    /** Issue SMBus 2.0 reset pulse (SCL low for > 30ms */
    I2C_DRV_RESET_SMBUS
} eI2CDrvCtrlHardwareType;

/******************************************************************************
*   STRUCT      :   sI2CDrvBufInfoType
******************************************************************************/
/**
 *  @brief   structure of I2C driver message buffer information
 *
 *****************************************************************************/
typedef struct
{
    /** Msg transmit buffer */
    UINT8 *pu8MsgSendBuffer;

    /** Msg receive buffer */
    UINT8 *pu8MsgRecBuffer;

    /** I2C bus channel */
    UINT8 u8Channel;
    
    /** device slave address */
    UINT8 u8DeviceAddr;
    
    /** Bus error status */
    UINT8 u8ErrorStatus;
    
    /** Msg transmit data length */
    UINT8 u8MsgSendDataSize;
    
    /** Msg receive data length */
    UINT8 u8MsgRecDataSize;

    /** Transaction type: 0:Normal, 1:Mux, 2:SMBus Blk */
    UINT8 u8TransType;  
    
} sI2CDrvBufInfoType;

/******************************************************************************
*   STRUCT      :   sI2CDrvBusInfoType
******************************************************************************/
/**
 *  @brief   structure of I2C driver bus information
 *
 *****************************************************************************/
typedef struct
{
    /** IPMB msg received flag */
    UINT32 u32RecFlag;
    
    /** ID for kernel event handler */
    UINT16 u16DriverID;

    /** Current start tracking count */
    UINT16 u16CurStartCount;
    
    /** Current stop tracking count */
    UINT16 u16CurStopCount;

    /** Reserved */
    UINT16 u16Rsv;

    /** I2C bus channel */
    UINT8 u8Channel;
    
    /** Initial mode (IPMB or PI2C) */
    UINT8 u8InitMode;
    
    /** Current mode (IPMB or PI2C) */
    UINT8 u8CurMode;
    
    /** IPMB slave address */
    UINT8 u8SlaveAddr;
    
    /** I2C bus frequency selection */
    UINT8 u8Frequency;
    
    /** Bus error status */
    UINT8 u8ErrorStatus;
    
    /** Bus status */
    UINT8 u8BusStatus;
    
    /** Control hardware */
    UINT8 u8CtrlHW;
    
} sI2CDrvBusInfoType;


#ifdef AESSI2CDRV_C

/* ioctl definitions */
#define DRIVER_NAME "aess_i2cdrv"
#define DRIVER_MODULE "aess_i2cdrv.ko"
#define DRIVER_TYPE "CHAR"


/******************************************************************************
*   STRUCT      :   Function Prototype 
******************************************************************************/
/**
 *  @brief   Prototype for each private function.
 *
 *****************************************************************************/


#endif   /* AESSI2CDRV_C */

#endif   /* AESSI2CDRV_H */
/* End of code */
