/**
 * $Id$
 *
 *<center>
 *               Avocent Corporation. Proprietary Information.
 * \n<em>
 *      This software is supplied under the terms of a license agreement or
 *      nondisclosure agreement with Avocent Corporation, or  one of its 
 *      affiliates, andaffiliates, and may not be used, disseminated, or 
 *      distributed except in accordance with the terms of that agreement.
 *
 *      3541 Gateway Boulevard Fremont, San Jose, CA 94538 U.S.A.
 *\n
 *                  US phone: 510-771-6100
 *
 *        Copyright &copy; 2001-2009 Avocent Corporation. All rights reserved.
 *</em> </center>
 *----------------------------------------------------------------------------\n
 *  MODULES     Linux device Driver                                           \n
 *----------------------------------------------------------------------------\n
 *  @file   aess_kcsdrv.c
 *  @brief  KCS driver and interrupt service routine
 *  @internal
 *  $RCSfile: aess_kcsdrv.c,v $
 *  $Revision: 1.1.1.1 $
 *  $Date: 2009/03/10 13:44:48 $
 *  $Author: Taiyi Kuo $
 *  $Source: //tpe-subversion/AESSTPE/puti/puti_fw/ProjectBase/trunk/platform/ast2050/evb/src/drivers/kcs/kcs_driver,v $
 *  $Name:  $
 *----------------------------------------------------------------------------*/
/******************************************************************************
* Content
* ----------------
*   aess_kcs_init()                 - The function is invoked when KCS module is loaded into kernel.
*   aess_kcs_exit()                 - The function is invoked when KCS module is removed from kernel. 
*   aess_kcs_open()                 - Open the device note .
*   aess_kcs_close()                - Close the device note .
*   aess_kcs_clearall()             - Free irq and clean KCS related data structures.         
*   aess_kcs_ioctl()                - The API is invoked when user space issues an IOCTL command. 
*   aess_kcs_read()                 - Command handler for AESS_KCSDRV_READ uses to pass the incoming KCS data to user space.
*   aess_kcs_write()                - Command handler for AESS_KCSDRV_WRITE uses to set the system 
*                                     attention bit or to send out the response KCS data from user space.
*   aess_kcs_isr()                  - The function is the entry function of interrupt handler.
*   aess_kcs_isr_handler()          - The function is an internal function for interrupt handler, aess_kcs_isr.
*   aess_kcs_errorhandler()         - The error handler for Linux KCS driver.
*   aess_kcs_error_isr_handler()    - The error handler for Linux KCS driver
*   aess_kcs_channel_init()         - Base on the channel index to initial related information.
*   aess_kcs_hw_init()              - Base on the channel index to initial the H/W register and related information.
*   aess_kcs_setstate()             - It used to change the KCS state.
*   aess_kcs_getreg()               - It used to read the KCS register value. 
*   aess_kcs_setreg()               - It used to set the KCS register value. 
*   aess_kcs_hw_reset()             - It used to reset H/W KCS channel.  
*   aess_kcs_polling_func()         - The function is for debug to dump register value.
*   aess_kcsdrv_bh()                - Notify event handler driver.
******************************************************************************/

#define AESSKCSDRV_C
#include <linux/errno.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/cdev.h>
#include <linux/interrupt.h>
#include <asm/uaccess.h>
#include <asm/arch/hardware.h>
#include <asm/io.h>

#include <linux/aess_debugfs.h>

static u8   u8tmpcounter = 0;
static u16 u16tmpcounter = 0;
static u32 u32tmpcounter = 0;

#include "aess_kcsdrv.h"
#include "../../eventhandler/eventhandler_driver/aess_eventhandlerdrv.h"

#include <linux/timer.h>
#include <linux/jiffies.h>

#include <asm-arm/arch-ast2050/hw_regs.h>


/* change to the highest message level */
#if 0
#define LPC_TESTING
#else
#define LPC_TESTING KERN_DEBUG
#endif

/* enable to print out message */
#if 0
#define DEBUGP(format, args...) printk(LPC_TESTING format, ## args)
#else
#define DEBUGP(format, args...)
#endif

/* enable a background timer to poll all hardware registers */
#if 0
#define KCS_POLLING
#endif


#if 0
#define SUPPORT_KCS_TX_EVENT
#endif


#define __REG(x)	(*((volatile u32 *)x))


#include <linux/spinlock.h>
#include <asm/arch/irqs.h>


#define SCU_WR_PROT_KEY        	(ASPEED_SCU_VIRTUAL+0x00)
#define SCU_RESET_CTRL         	(ASPEED_SCU_VIRTUAL+0x04)
#define SCU_CLOCK_STOP_CTRL    	(ASPEED_SCU_VIRTUAL+0x0c)

/* Values for SCU_WR_PROT_KEY */
#define SCU_WR_KEY_EN          	0x1688A8A8
#define SCU_WR_KEY_DIS         	0x00000000

/* Bits of SCU_CLOCK_STOP_CTRL */
#define SCU_CLOCK_STOP_LPC     	(1<<8)

/* Bits of SCU_RESET_CTRL */
#define SCU_RESET_LPC        	(1<<5)

/* definitions for HICR0 */
#define LPC_LPC3E   0x80
#define LPC_LPC2E   0x40
#define LPC_LPC1E   0x20

/* definitions for HICR1 */
#define LPC_LRSTB   0x10
#define LPC_SDWNB   0x08

/* definitions for HICR2 */
#define LPC_LRST    0x40
#define LPC_SDWN    0x20
#define LPC_ABRT    0x10
#define LPC_IBFIF3  0x08
#define LPC_IBFIF2  0x04
#define LPC_IBFIF1  0x02
#define LPC_ERRIE   0x01

/* definitions for HICR3 */

/* definitions for HICR4 */
#define LPC_LADR12ASEL_SEL1  0x00
#define LPC_LADR12ASEL_SEL2  0x80
#define LPC_KCSENBL          0x04
#define LPC_BTENBL           0x01


typedef enum
{
    KCS_CH_1 = 0,
    KCS_CH_2,
    KCS_CH_3
} eKCSChannelList;


/* MICS */
#define KCS_DUMMY_DATA              0x00
#define KCS_CLEAR_OBF               0x00
#define SET_SMA_BIT                 0x01

/* KCS Register Definition */
#define KCS_REG_FOCTL               0x00
#define KCS_REG_ODR                 0x01
#define KCS_REG_IDR                 0x02
#define KCS_REG_STATUS              0x03
#define KCS_REG_IBF                 0x04
#define KCS_REG_OBF                 0x05
#define KCS_REG_CD                  0x06


/* KCS interface state definition */
#define KCS_STATUS_IDLE_STATE          0x0
#define KCS_STATUS_READ_STATE          0x1
#define KCS_STATUS_WRITE_STATE         0x2
#define KCS_STATUS_ERROR_STATE         0x3

/* KCS interface control codes definition */
#define KCS_ABORT_CMD           0x60
#define KCS_GET_STATUS          0x60
#define KCS_WRITE_START         0x61
#define KCS_WRITE_END           0x62
#define KCS_READ                0x68
#define KCS_ZERO_DATA           0x00

/* KCS interface status register bits definition */
#define KCS_DATA                    0
#define KCS_COMMAND                 1
#define SMS_ATN_SET                 1
#define SMS_ATN_CLEAR               0

/* KCS interface status codes definition */
#define KCS_NO_ERROR                0x00
#define KCS_ABORTED_BY_COMMAND      0x01
#define KCS_ILLEGAL_CONTROL_CODE    0x02
#define KCS_LENGTH_ERROR            0x06
#define KCS_UNSPECIFIED_ERROR       0xFF

/* KCS Seq No */
#define KCS_SEQ_NO_INIT             0

/******************************************************************************
*   STRUCT      :   sLPCSTRRegType
******************************************************************************/
/**
 *  @brief   structure of LPC Status Register
 *
 *****************************************************************************/
typedef struct
{
    union
    {
        /** BYTE */
        unsigned char BYTE;
        struct
        {
            /** DBU */
            unsigned char s1  :1;        /** STATUS */
            unsigned char s0  :1;        /** STATUS */
            unsigned char OEM2:1;        /** OEM */
            unsigned char OEM1:1;        /** OEM */
            unsigned char CD  :1;        /** C/D */
            unsigned char SMS :1;        /** SMS */
            unsigned char IBF :1;        /** IBF */
            unsigned char OBF :1;        /** OBF */
        } BIT;
    } EQU;
} sLPCSTRRegType;


/******************************************************************************
*   STRUCT      :   sLPCRegsPtrType
******************************************************************************/
/**
 *  @brief   structure of LPC Register pointers
 *
 *****************************************************************************/
typedef struct
{
    /** Pointer to IDR */
    volatile UINT8          *const pIDR;
    
    /** Pointer to ODR */
    volatile UINT8          *const pODR;
    
    /** Pointer to STR */
    volatile sLPCSTRRegType  *const pSTR;
} sLPCRegsPtrType;


/******************************************************************************
*   ENUM        :   eHIFStateTag
******************************************************************************/
/**
 *  @brief    The type state of KCS state machine
 *
 *****************************************************************************/
typedef enum
{
    /** The KCS driver has been initialized and is ready to process any KCS
        interface interrupt. */
    KCS_IDLE_STATE,

    /** When receive WRITE_START command then transition to this state. To store
        the receive data into local buffer in this state. */
    KCS_RECEIVE_DATA_STATE,

    /** When receive WRITE_END command and already in the RECEIVE_DATA state
        then transition to this state. To store the last one receives data
        in this state. */
    KCS_RECEIVE_DATA_END_STATE,

    /** When KCSWriteStart() API be called then transition to this state.
        Or already in this state and receive READ command, it still stay
        in this state. To put transfer data into ODR register in this state. */
    KCS_TRANSFER_DATA_STATE,

    /** When data length counter count down to zero and already
        in the TRANSFER_DATA state then transition to this state.
        To put the dummy data to ODR register in this state. */
    KCS_TRANSFER_DATA_END_STATE,

    /** When receive GET_STATUS/ABORT command then transition to this state. */
    KCS_RESPONSE_STATUS_PHASE1_STATE,

    /** When get the dummy data H'00 and already in the
        RESPONSE_STATUS_PHASE1 state then transition to this state.
        At the same time write HIF status to ODR register. */
    KCS_RESPONSE_STATUS_PHASE2_STATE,

    /** Error state */
    KCS_ERROR_STATE

} eKCSStateTag;


/******************************************************************************
*   STRUCT      :   sKCSStruct
******************************************************************************/
/**
 *  @brief   Structure to KCS driver related data parameter, 
 *           used by linux driver.
 *
 *****************************************************************************/
typedef struct
{
    /** Receive buffer */
    UINT8   au8HIFReceiveBuf[KCS_PACKAGE_MAX];

    /** Transmit buffer */
    UINT8   au8HIFTransferBuf[KCS_PACKAGE_MAX];

    /** Receive complete event */
    UINT32  u32KCSRxOKEvent;

    /** Transmit done event */
    UINT32  u32KCSTxOKEvent;

    /** Transmit fail event */
    UINT32  u32KCSTxFailEvent;

    /** Transmit data pointer */
    UINT8   *pu8HIFOutDataPtr;

    /** Address base */
    UINT16  u16AddrBase;

    /** Event group Driver ID */
    UINT16 u16DriverID;

    /** Channel number */
    UINT8 u8Channel;    
        
    /** ISR state  */
    UINT8   u8HIFISRState;

    /** Receive data counter */
    UINT8   u8HIFInDataSize;

    /** Transmit data counter */
    UINT8   u8HIFOutDataSize;

    /** KCS error status code */
    UINT8   u8HIFStatusCode;
  
    /** KCS sequence No*/
    UINT8 u8KCSSeqNo;

    /** Reserved */
    UINT8 uReserve[2];
    
} sKCSStruct;


/* Declear KCS data structure */
static sKCSStruct S_aKCSData[KCS_TOTAL_CHANNEL];

/* Set the address for KCS related register, 
   input data register as same as output data register */
static const sLPCRegsPtrType LPCRegs[KCS_TOTAL_CHANNEL] =
{
    {
        (volatile UINT8 *) LPC_IDR1_REG,
        (volatile UINT8 *) LPC_ODR1_REG,
        (volatile sLPCSTRRegType *) LPC_STR1_REG
    },
    {
        (volatile UINT8 *) LPC_IDR2_REG,
        (volatile UINT8 *) LPC_ODR2_REG,
        (volatile sLPCSTRRegType *) LPC_STR2_REG
    },
    {
        (volatile UINT8 *) LPC_IDR3_REG,
        (volatile UINT8 *) LPC_ODR3_REG,
        (volatile sLPCSTRRegType *) LPC_STR3_REG
    }
};

/* KCS Register */
#define REG_IDR  (*(LPCRegs[u8Channel].pIDR))
#define REG_ODR  (*(LPCRegs[u8Channel].pODR))
#define REG_STR  (*(LPCRegs[u8Channel].pSTR))

/* driver name will passed by insmod command */
static char *driver_name="aess_kcsdrv";

/* Linux device data structure */
dev_t kcs_dev;
struct cdev *kcs_cdev;

/* LPC initial status */
static UINT8 Su8LPCInirSata = 0;    /* bit0: LPC1, bit1: LPC2, bit2: LPC3 */

#ifdef KCS_POLLING
struct timer_list kcs_polling;
#endif

/******************************************************************************
*   STRUCT      :    sLPCResetCBIDStruct
******************************************************************************/
/**
 *  @brief    Structure for LPC reset
 *
 *****************************************************************************/
typedef struct
{
  /** Callback Function event ID*/
  UINT32 u32CBFunEventID;

  /** Callback Function event group Driver ID*/
  UINT16  u16CBFunDriverID;

  /** Reserve */
  UINT16 u16Reserve;
} sLPCResetCBIDStruct;
volatile static sLPCResetCBIDStruct S_sLPCResetCBID;

DECLARE_TASKLET(lpcreset_tasklet, aess_kcsdrv_bh, 0);

#ifdef AST2100
/******************************************************************************
*   FUNCTION        :   aess_kcs_hw_reset
******************************************************************************/
/**
 *  @brief      It used to reset the H/W KCS channel.
 *
 *  @return     None                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Internal function \n
 *
 *****************************************************************************/
static void aess_kcs_hw_reset(void)
{
    __REG(SCU_WR_PROT_KEY) = SCU_WR_KEY_EN;    /* Write enable key */
    __REG(SCU_RESET_CTRL) |= SCU_RESET_LPC;		/* Reset LPC */
    nop();
    __REG(SCU_RESET_CTRL) &= ~SCU_RESET_LPC; 	/* Clear reset LPC */
    __REG(SCU_CLOCK_STOP_CTRL) &= ~SCU_CLOCK_STOP_LPC; /* Enable clock */
    __REG(SCU_WR_PROT_KEY) = SCU_WR_KEY_DIS;   /* Write disable key */
    
}
#endif


/******************************************************************************
*   FUNCTION        :   aess_kcs_setreg
******************************************************************************/
/**
 *  @brief      It used to set the KCS register value.
 *
 *  @return     return the value if function success.
 *              return STATUS_FAIL if function fail.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Internal function \n
 *
 *****************************************************************************/
static inline void aess_kcs_setreg(
                                   /** KCS interface channel */
                                   UINT8 u8Channel,
                               
                                   /** KCS parameters definition */
                                   UINT8 u8Register ,
                               
                                   /** Data for setting KCS parameters */
                                   UINT8 u8Data
                                  )
{            
    volatile UINT8 u8Tmp = 0;
    
    if (u8Channel < KCS_TOTAL_CHANNEL)
    {
        switch (u8Register)
        { 
            case KCS_REG_OBF:
                
                u8Tmp = REG_STR.EQU.BYTE;
                REG_STR.EQU.BYTE = ((u8Tmp & 0xFE) | (u8Data & 0x01));
                
                break;
                
            case KCS_REG_FOCTL:
                
                u8Tmp = REG_STR.EQU.BYTE;
                REG_STR.EQU.BYTE = ((u8Tmp & 0xFB) | ((u8Data & 0x01) << 2));
               
                break;
                
            case KCS_REG_ODR:
               
                REG_ODR = u8Data;

                break;
                
            case KCS_REG_STATUS:
                
                u8Tmp = REG_STR.EQU.BYTE;
                REG_STR.EQU.BYTE = ((u8Tmp & 0x3F) | (u8Data << 6));
                
                break;
                
            default:
                break;
        }
    }
    
    return;
}


/******************************************************************************
*   FUNCTION        :   aess_kcs_getreg
******************************************************************************/
/**
 *  @brief      It used to read the KCS register value.
 *
 *  @return     return the value if function success.
 *              return STATUS_FAIL if function fail.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Internal function \n
 *
 *****************************************************************************/
static inline UINT8 aess_kcs_getreg(
                                    /** KCS interface channel */
                                    UINT8 u8Channel,
                     
                                    /** KCS parameters definition */
                                    UINT8 u8Register
                                   )
{
    volatile UINT8 u8Data = 0;
    
    if (u8Channel < KCS_TOTAL_CHANNEL)
    {
        switch (u8Register)
        {
            case KCS_REG_IBF:
                
                u8Data = ((REG_STR.EQU.BYTE & 0x02) >> 1);//REG_STR.EQU.BIT.IBF;
                
                break;
                
            case KCS_REG_CD:
                
                u8Data = ((REG_STR.EQU.BYTE & 0x08) >> 3);//REG_STR.EQU.BIT.CD;
                
                break;
                
            case KCS_REG_IDR:
                
                u8Data = REG_IDR;
                
                break;
                
            default:
                
                break;
        }
        
        return (u8Data);            
    }
    
    DRV_ERROR("aess_kcs_getreg()::return(STATUS_FAIL)::channel no error.\n");
    return STATUS_FAIL;
}


/******************************************************************************
*   FUNCTION        :   aess_kcs_setstate
******************************************************************************/
/**
 *  @brief      It used to change the KCS state.
 *
 *  @return     None.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Internal function \n
 *
 *****************************************************************************/
static void aess_kcs_setstate(
                              /** Channel number */
                              UINT8 u8Channel,
                              
                              /** BMC Status */
                              UINT8 u8State
                             )
{
    aess_kcs_setreg(u8Channel, KCS_REG_STATUS, u8State);
}

/******************************************************************************
*   FUNCTION        :   aess_kcs_hw_init
******************************************************************************/
/**
 *  @brief      Base on the channel index to initial the H/W register and related information.
 *
 *  @return     None                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Internal function \n
 *
 *****************************************************************************/	
static void aess_kcs_hw_init(
                              /** Channel number */
                              UINT8 u8Channel
                             )
{
    UINT8 u8Tmp = 0;

    /* initial register of host interface */
    switch(u8Channel)
    {
        case KCS_CH_1:
            
            /* LADR 1 is selected */
            u8Tmp = *((UINT8 *) LPC_HICR4_REG);
            *((UINT8 *) LPC_HICR4_REG) = ((u8Tmp & 0x7F) | LPC_LADR12ASEL_SEL1);
            
            /* LPC channel 1 address register */
            *((UINT8 *) LPC_LADR12H_REG) 
            = (UINT8) (S_aKCSData[u8Channel].u16AddrBase >> 8);
            *((UINT8 *) LPC_LADR12L_REG) 
            = (UINT8) (S_aKCSData[u8Channel].u16AddrBase & 0x00FF);
            
            /* LPC enable */
            u8Tmp = *((UINT8 *) LPC_HICR0_REG);
            *((UINT8 *) LPC_HICR0_REG) = (u8Tmp | LPC_LPC1E);
            
            /* clear the OBF */
            aess_kcs_setreg(u8Channel, KCS_REG_OBF, KCS_CLEAR_OBF);
            
            /* dummy read IDR register */
            aess_kcs_getreg(u8Channel , KCS_REG_IDR);
            
            /* enable interrupt */
            u8Tmp = *((UINT8 *) LPC_HICR2_REG);
            *((UINT8 *) LPC_HICR2_REG) = (u8Tmp | 0xF0 | LPC_IBFIF1 | LPC_ERRIE);
            
            break;
            
        case KCS_CH_2:
            
            /* LADR 2 is selected */
            u8Tmp = *((UINT8 *) LPC_HICR4_REG);
            *((UINT8 *) LPC_HICR4_REG) = ((u8Tmp & 0x7F) | LPC_LADR12ASEL_SEL2);
            
            /* LPC channel 2 address register */
            *((UINT8 *) LPC_LADR12H_REG) 
            = (UINT8) (S_aKCSData[u8Channel].u16AddrBase >> 8);
            *((UINT8 *) LPC_LADR12L_REG) 
            = (UINT8) (S_aKCSData[u8Channel].u16AddrBase & 0x00FF);
            
            /* LPC enable */
            u8Tmp = *((UINT8 *) LPC_HICR0_REG);
            *((UINT8 *) LPC_HICR0_REG) = (u8Tmp | LPC_LPC2E);
            
            /* clear the OBF */
            aess_kcs_setreg(u8Channel, KCS_REG_OBF, KCS_CLEAR_OBF);
            
            /* dummy read IDR register */
            aess_kcs_getreg(u8Channel , KCS_REG_IDR);
            
            /* enable interrupt */
            u8Tmp = *((UINT8 *) LPC_HICR2_REG);
            *((UINT8 *) LPC_HICR2_REG) = (u8Tmp | 0xF0 | LPC_IBFIF2 | LPC_ERRIE);
            
            break;
            
        case KCS_CH_3:
            
            /* LPC channel 3 address register */
            *((UINT8 *) LPC_LADR3H_REG) 
            = (UINT8) (S_aKCSData[u8Channel].u16AddrBase >> 8);
            *((UINT8 *) LPC_LADR3L_REG) 
            = (UINT8) (S_aKCSData[u8Channel].u16AddrBase & 0x00FF);
            
            /* LPC enable */
            u8Tmp = *((UINT8 *) LPC_HICR0_REG);
            *((UINT8 *) LPC_HICR0_REG) = (u8Tmp | LPC_LPC3E);
            
            /* enable KCS interface */
            u8Tmp = *((UINT8 *) LPC_HICR4_REG);
            *((UINT8 *) LPC_HICR4_REG) = (u8Tmp | LPC_KCSENBL);
            
            /* clear the OBF */
            aess_kcs_setreg(u8Channel, KCS_REG_OBF, KCS_CLEAR_OBF);
            
            /* dummy read IDR register */
            aess_kcs_getreg(u8Channel , KCS_REG_IDR);
            
            /* enable interrupt */
            u8Tmp = *((UINT8 *) LPC_HICR2_REG);
            *((UINT8 *) LPC_HICR2_REG) = (u8Tmp | 0xF0 | LPC_IBFIF3 | LPC_ERRIE);
            
            break;
            
        default:
            
            /* Won't happen */
            break;
    }
    /* debug message */
    DRV_URGENT("%s :: show the register value.\n", __FUNCTION__ );
    DRV_URGENT("LPC_HICR0_REG   %xh \n", *((UINT8 *) LPC_HICR0_REG)); 
    DRV_URGENT("LPC_HICR1_REG   %xh \n", *((UINT8 *) LPC_HICR1_REG)); 
    DRV_URGENT("LPC_HICR2_REG   %xh \n", *((UINT8 *) LPC_HICR2_REG)); 
    DRV_URGENT("LPC_HICR3_REG   %xh \n", *((UINT8 *) LPC_HICR3_REG)); 
    DRV_URGENT("LPC_HICR4_REG   %xh \n", *((UINT8 *) LPC_HICR4_REG)); 
    DRV_URGENT("LPC_LADR3H_REG  %xh \n",  *((UINT8 *) LPC_LADR3H_REG)); 
    DRV_URGENT("LPC_LADR3L_REG  %xh \n",  *((UINT8 *) LPC_LADR3L_REG)); 
    DRV_URGENT("LPC_LADR12H_REG %xh \n",  *((UINT8 *) LPC_LADR12H_REG)); 
    DRV_URGENT("LPC_LADR12L_REG %xh \n",  *((UINT8 *) LPC_LADR12L_REG)); 
    DRV_URGENT("LPC_IDR1_REG    %xh \n",  *((UINT8 *) LPC_IDR1_REG)); 
    DRV_URGENT("LPC_IDR2_REG    %xh \n",  *((UINT8 *) LPC_IDR2_REG)); 
    DRV_URGENT("LPC_IDR3_REG    %xh \n",  *((UINT8 *) LPC_IDR3_REG)); 
    DRV_URGENT("LPC_ODR1_REG    %xh \n",  *((UINT8 *) LPC_ODR1_REG)); 
    DRV_URGENT("LPC_ODR2_REG    %xh \n",  *((UINT8 *) LPC_ODR2_REG)); 
    DRV_URGENT("LPC_ODR3_REG    %xh \n",  *((UINT8 *) LPC_ODR3_REG)); 
    DRV_URGENT("LPC_STR1_REG    %xh \n",  *((UINT8 *) LPC_STR1_REG)); 
    DRV_URGENT("LPC_STR2_REG    %xh \n",  *((UINT8 *) LPC_STR2_REG)); 
    DRV_URGENT("LPC_STR3_REG    %xh \n",  *((UINT8 *) LPC_STR3_REG)); 
}


/******************************************************************************
*   FUNCTION        :   aess_kcs_channel_init
******************************************************************************/
/**
 *  @brief      Base on the channel index to initial related information.
 *
 *  @return     STATUS_OK if initial success.                   
 *              Others if initial fail.  
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Internal function \n
 *
 *****************************************************************************/	
static int aess_kcs_channel_init(
                                 /** sKCSInfo structure */
                                 sKCSInfo *psInfo
                                 )
{
    UINT8 u8Channel;
    
    /* Check channel number */
    if (psInfo->u8Channel >= KCS_TOTAL_CHANNEL)
    {
        DRV_ERROR("%s::return(-ECHRNG):: Invalid channel number.\n",__FUNCTION__);
        /* u8Channel invalid */
    	return -ECHRNG;
	}
	
    /* get channel number for accessing register */
    u8Channel = psInfo->u8Channel;
    
    /* Save Channel ID */
    S_aKCSData[u8Channel].u8Channel = u8Channel;        
    
    /* Save Event ID */
    S_aKCSData[u8Channel].u16DriverID = psInfo->u16DriverID;
    
    S_aKCSData[u8Channel].u32KCSRxOKEvent = psInfo->u32KCSRxOKEvent;
    
    S_aKCSData[u8Channel].u32KCSTxOKEvent = psInfo->u32KCSTxOKEvent;
    
    S_aKCSData[u8Channel].u32KCSTxFailEvent = psInfo->u32KCSTxFailEvent;
    
    /* Save KCS base address and offset */
    S_aKCSData[u8Channel].u16AddrBase = psInfo->u16BaseAddress;
    
    /* set HIF state machine to idle state */
    S_aKCSData[u8Channel].u8HIFISRState = KCS_IDLE_STATE;
    
    /* clear error code */
    S_aKCSData[u8Channel].u8HIFStatusCode = KCS_NO_ERROR;
    
    /* reset seq no */
    S_aKCSData[u8Channel].u8HIFStatusCode = KCS_SEQ_NO_INIT;
    
    /* KCS SeqNo init */
    S_aKCSData[u8Channel].u8KCSSeqNo = 0;
    
    /* set initial state to Error state to mit the KCS spec */
    aess_kcs_setstate(u8Channel, KCS_STATUS_ERROR_STATE);
    
    /* record LPC initial status */
    Su8LPCInirSata |= (0x01 << u8Channel);

    DRV_INFO("%s::return(STATUS_OK)::channel %d \n",__FUNCTION__,u8Channel);

	  /* Init H/W */
    aess_kcs_hw_init(u8Channel);
    
    return STATUS_OK;
}

/******************************************************************************
*   FUNCTION        :   aess_kcs_read
******************************************************************************/
/**
 *  @brief      Command handler for AESS_KCSDRV_READ uses to pass the incoming KCS data to user space.
 *
 *  @return     STATUS_OK if read data and transfer it to user space success.
 *              Others if read or transfer data fail.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Internal function \n
 *
 *****************************************************************************/	
static int aess_kcs_read(
                         /** sKCSInfo structure */
                         sKCSInfo *psInfo
                         )
{
    UINT8 u8Count; 
    UINT8 u8HIFDataSize;
    UINT8 u8TempSeqNum;
    
    if (psInfo->u8Channel < KCS_TOTAL_CHANNEL)
    {
        /* started while we are copying */
        u8TempSeqNum = S_aKCSData[psInfo->u8Channel].u8KCSSeqNo;
        u8HIFDataSize = S_aKCSData[psInfo->u8Channel].u8HIFInDataSize;
        
        /* check length of KCS packet to make sure it is a valid message   
           before passing to upper layer for processing.                   
           Siliently discard if KCS packet length not valid.             */
        if ((u8HIFDataSize == 0) || (u8HIFDataSize > (KCS_PACKAGE_MAX + 1)))
        {
            /* invalid KCS packet length */
            DRV_ERROR("%s::return(-EMSGSIZE)::Invalid KCS packet length.\n",__FUNCTION__);         
            return -EMSGSIZE;
        }
        
        /* save incoming messages */
        for (u8Count = 0; u8Count < u8HIFDataSize; u8Count++)
        {
            psInfo->pu8Data[u8Count]
            = S_aKCSData[psInfo->u8Channel].au8HIFReceiveBuf[u8Count];
        }
        
        /* save length of incoming messages */
        *psInfo->pu8ReadLength = u8HIFDataSize;
        
        /* Now compare the temp sequence number to the Global one to make 
           sure another Write Start command hasn't been issued on KCS 
           while we were copying this message.
           If a Write Start was issued, throw away this message and wait 
           for new one.                                                    */ 
        if (u8TempSeqNum == S_aKCSData[psInfo->u8Channel].u8KCSSeqNo)
        {
            DRV_INFO("%s::return(STATUS_OK)::u8HIFDataSize=%d \n",__FUNCTION__,u8HIFDataSize);         
            return (STATUS_OK);
        }
        else
        {
            DRV_ERROR("%s::return(-STATUS_FAIL)::Invalid u8KCSSeqNo number.\n",__FUNCTION__);         
            return (STATUS_FAIL);
        }
    }
    
    /* u8Channel invalid */
    DRV_ERROR("%s::return(-STATUS_FAIL)::Invalid channel number.\n",__FUNCTION__);         
    
    return -ECHRNG;
}


/******************************************************************************
*   FUNCTION        :   aess_kcs_write
******************************************************************************/
/**
 *  @brief      Command handler for AESS_KCSDRV_WRITE uses to set the system 
 *              attention bit or to send out the response KCS data from user space.
 *
 *  @return     STATUS_OK if functun exectue success.
 *              Others if function execute fail.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Internal function \n
 *
 *****************************************************************************/	

static int aess_kcs_write(
                          /** sKCSInfo structure */
                          sKCSInfo *psInfo
                          )
{
    UINT8 u8Count = 0;
    
    if (psInfo->u8Channel < KCS_TOTAL_CHANNEL)
    {
        if (psInfo->u8Control == SMS_ATN_SET)
        {
            /* Set KCS interface system attention bit */
            aess_kcs_setreg(psInfo->u8Channel, KCS_REG_FOCTL, *psInfo->pu8Data);
        }
        else
        {
            if (S_aKCSData[psInfo->u8Channel].u8HIFISRState == KCS_TRANSFER_DATA_STATE )
            {
                /* Compare the sequence number, if it's the response of latest
                   KCS request, then remove the extra byte, if not, drop it */
                if ((S_aKCSData[psInfo->u8Channel].u8KCSSeqNo)
                    != psInfo->pu8Data[psInfo->u8WriteLength - 1])
                {
                    DRV_ERROR("%s::return(-STATUS_FAIL)::Invalid u8KCSSeqNo number.\n",__FUNCTION__);         
                    return (STATUS_FAIL);
                }
                else
                {
                    psInfo->u8WriteLength--;
                }
                
                /* Store output messages in local buffer */
                for (u8Count = 0; u8Count < psInfo->u8WriteLength; u8Count++)
                {
                    S_aKCSData[psInfo->u8Channel].au8HIFTransferBuf[u8Count]
                    = *psInfo->pu8Data++;
                }
                
                /* save pointer of output buffer */
                S_aKCSData[psInfo->u8Channel].pu8HIFOutDataPtr
                = S_aKCSData[psInfo->u8Channel].au8HIFTransferBuf;
                
                /* save length of output buffer */
                S_aKCSData[psInfo->u8Channel].u8HIFOutDataSize 
                = psInfo->u8WriteLength;
                
                /* Send first byte */
                S_aKCSData[psInfo->u8Channel].u8HIFOutDataSize--;
                
                /* OBF (read), put one byte to data out register */
                aess_kcs_setreg(psInfo->u8Channel, KCS_REG_ODR, *S_aKCSData[psInfo->u8Channel].pu8HIFOutDataPtr++);
            }
            else
            {
                /* if KCS machine state is not in transfer data 
                   state then return STATUS_FAIL */
                DRV_ERROR("%s::return(STATUS_FAIL):: It's not in transfer data state.\n",__FUNCTION__);         
                return (STATUS_FAIL);
            }
        }
        
        DRV_INFO("%s::return(STATUS_OK)",__FUNCTION__);         
        return (STATUS_OK);
    }
    else
    {
        DRV_ERROR("%s::return(-STATUS_FAIL)::Invalid channel number.\n",__FUNCTION__);         
        return (STATUS_FAIL);
    }  
}


/******************************************************************************
*   FUNCTION        :   aess_kcs_ioctl
******************************************************************************/
/**
 *  @brief      The API is invoked when user space issues an IOCTL command. 
 *              The following is the list of commands that support currently.
 *                  1.	AESS_KCSDRV_INIT
 *                          Initialize Linux KCS driver according to the given parameters.
 *                  2.	AESS_KCSDRV_READ
 *                          Use to retrieve an incoming KCS message.
 *                  3.	AESS_KCSDRV_WRITE
 *                          Issue a KCS transaction on an assigned KCS channel
 *
 *  @return     STATUS_OK if driver execute the command success.
 *              Others if execute the command fail.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function \n
 *
 *****************************************************************************/
static int aess_kcs_ioctl(
                          /** inode structure point */
                          struct inode *inode, 
       
                          /** file structure point */
                          struct file *filp,
       
                          /** command from user space */
                          unsigned int cmd, 

                          /** sKCSInfo structure point */
                          unsigned long arg
                          )
{
    int err_check;
    sKCSInfo *psInfo = (sKCSInfo *) arg;
   
    /* Check the structure address is OK */
    if(!access_ok(VERIFY_WRITE, (void *) psInfo, sizeof(sKCSInfo)))
    {
        DRV_ERROR("%s::return(-EFAULT)::buffer structure access fail!\n",__FUNCTION__);         
        /* error occur */
        return -EFAULT;
    }
    
    switch(cmd)
    {
        case AESS_KCSDRV_INIT:
            
            /* Check the buffer address exist in the structure is OK */
            err_check = aess_kcs_channel_init(psInfo);
            break;
            
        case AESS_KCSDRV_READ:
           
            /* Check the buffer address exist in the structure is OK */
            if(!access_ok(VERIFY_WRITE, psInfo->pu8Data, 
                          sizeof(UINT8)) * KCS_PACKAGE_MAX)
            {
                /* error occur */
                err_check = -EFAULT;
                DRV_ERROR("%s::return(-EFAULT)::buffer structure access fail!(2)\n",__FUNCTION__);
                break;                
            }
            
            err_check = aess_kcs_read(psInfo);
            break;
            
        case AESS_KCSDRV_WRITE:
           
            /* Check the buffer address exist in the structure is OK */
            if(!access_ok(VERIFY_READ, psInfo->pu8Data, 
                          sizeof(UINT8) * KCS_PACKAGE_MAX))
            {
                /* error occur */
                err_check = -EFAULT;
                DRV_ERROR("%s::return(-EINVAL)::buffer access fail!\n",__FUNCTION__);
                break;
            }
            
            err_check = aess_kcs_write(psInfo);
            break;

        case AESS_KCSDRV_SWSMI:
              #if 0
                  #if SUPPORT_SMI
                  /*Pull down nSMI*/
                  aess_kcs_setreg(psInfo->u8Channel, KCS_REG_IC, SET_SWIB);
                  mdelay(10);
                  /*Pull up nSMI*/
                  aess_kcs_setreg(psInfo->u8Channel, KCS_REG_IC, CLEAR_SWIB);
                  #endif
              #endif
              err_check = STATUS_OK;
              break;    
              
        case AESS_KCSDRV_SETCBID:    
              S_sLPCResetCBID.u16CBFunDriverID = psInfo->u16CBFunDriverID;
              S_sLPCResetCBID.u32CBFunEventID = psInfo->u32CBFunEventID;
              DRV_INFO("S_sLPCResetCBID.u16CBFunDriverID = %x , S_sLPCResetCBID.u32CBFunEventID=%x",psInfo->u16CBFunDriverID ,psInfo->u32CBFunEventID );
         
              err_check = STATUS_OK;
              break;
                        
        default:
            err_check = -EINVAL;
            DRV_ERROR("%s::return(-EINVAL)::command fail!\n",__FUNCTION__);
            break;
    }
    
    /* 0->ok, minus->fail */
    return err_check;
}


/******************************************************************************
*   FUNCTION        :   aess_kcs_clearall
******************************************************************************/
/**
 *  @brief      Clean the S_aKCSData structure and un-register the IRQ .
 *
 *  @return     STATUS_OK never fail.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    Before remove kcs driver , it's necessary to free irq .
 *
 *  @note       None
 *
 *  @internal   Function Type: Internal function \n
 *
 *****************************************************************************/
static int aess_kcs_clearall(void)
{
    /* Set the initial value for data structure */
    memset(S_aKCSData, 0, sizeof(sKCSStruct) * KCS_TOTAL_CHANNEL);  
    
    /* Release IRQ */
    free_irq(IRQ_LPC, (void *) &S_aKCSData[0]);

    DRV_INFO("%s::return(STATUS_OK)\n", __FUNCTION__);
    return (STATUS_OK);
}


/******************************************************************************
*   FUNCTION        :   aess_kcs_errorhandler
******************************************************************************/
/**
 *  @brief      The error handler for Linux KCS driver.
 *
 *              This functon called by aess_kcs_isr when KCS channel error occurs. 
 *              It will depend on the parameter(channel index) to change the KCS 
 *              cahnnel state to KCS_ERROR_STATE.              
 *
 *  @return     None.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    .
 *
 *  @note       None
 *
 *  @internal   Function Type: Internal function \n
 *
 *****************************************************************************/
static void aess_kcs_errorhandler(
                                  /** Channel number */
                                  UINT8 u8Channel
                                 )
{
    /* set BMC state to ERROR state */
    aess_kcs_setstate(u8Channel, KCS_STATUS_ERROR_STATE);

    S_aKCSData[u8Channel].u8HIFISRState = KCS_ERROR_STATE;
    
    /* dummy read IDR register */
    aess_kcs_getreg(u8Channel , KCS_REG_IDR);
    
    /* KCS SeqNo increase*/
    S_aKCSData[u8Channel].u8KCSSeqNo++;
    
    DRV_INFO("%s::return.\n",__FUNCTION__);
}


/******************************************************************************
*   FUNCTION        :   aess_kcs_isr_handler
******************************************************************************/
/**
 *  @brief      The function is an internal function for interrupt handler, aess_kcs_isr.
 *
 *  @return     None.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    Before remove kcs driver , it's necessary to free irq .
 *
 *  @note       None
 *
 *  @internal   Function Type: Internal function \n
 *
 *****************************************************************************/
static void aess_kcs_isr_handler (
                                  /** channel number */  
                                  UINT8 u8Channel
                                  )
{
    volatile sKCSStruct *psInfo = &S_aKCSData[u8Channel];
    volatile UINT8   u8Command = 0;
    volatile UINT8   u8Data = 0;
    
    if (aess_kcs_getreg(psInfo->u8Channel , KCS_REG_IBF))
    {
        /* Command in */
        if (KCS_COMMAND == aess_kcs_getreg(psInfo->u8Channel , KCS_REG_CD))
        {
            switch (psInfo->u8HIFISRState)
            {
                case KCS_IDLE_STATE:
                    
                    /* falls through */
                case KCS_RECEIVE_DATA_END_STATE:
                    
                    /* falls through */
                case KCS_RESPONSE_STATUS_PHASE1_STATE:
                    
                    /* falls through */
                case KCS_RESPONSE_STATUS_PHASE2_STATE:
                    
                    /* BMC sets status to WRITE_STATE */
                    aess_kcs_setstate(psInfo->u8Channel, KCS_STATUS_WRITE_STATE);
                                      
                    /* dummy data to generate OBF (wr_start) */
                    aess_kcs_setreg(u8Channel, KCS_REG_ODR, KCS_DUMMY_DATA);
                    
                    /* get the command byte */
                    u8Command = aess_kcs_getreg(u8Channel, KCS_REG_IDR);
                    
                    if ( u8Command == KCS_WRITE_START )
                    {
                        /* if "WRITE START" command be issued from SMS then set
                        KCS machine state back to receive data state and clear
                        receive buffer size */
                        psInfo->u8HIFISRState = KCS_RECEIVE_DATA_STATE;
                        
                        psInfo->u8HIFInDataSize = 0;
                        ++psInfo->u8KCSSeqNo;
                        
                        DRV_URGENT("%s::KCS_RECEIVE_DATA_STATE::u8Command == KCS_WRITE_START ::psInfo->u8KCSSeqNo==%d [1]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        
                    }
                    else if ( u8Command == KCS_ABORT_CMD )
                    {
                        /* if "ABORT" command be issued from SMS then set BMC
                        and KCS machine state to IDLE state, and record error
                        code [01] */
                        
                        psInfo->u8HIFISRState = KCS_RESPONSE_STATUS_PHASE1_STATE;
                        psInfo->u8HIFStatusCode = KCS_ABORTED_BY_COMMAND;

                        DRV_URGENT("%s::KCS_IDLE_STATE::u8Command == KCS_ABORT_CMD ::psInfo->u8KCSSeqNo==%d [2]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        
                    }
                    else
                    {
                        /* if the command is not expacted then record error
                        code [02] and go to KCS error handler sub-routine */
                        psInfo->u8HIFStatusCode = KCS_ILLEGAL_CONTROL_CODE;

                        DRV_URGENT("%s::KCS_ILLEGAL_CONTROL_CODE::psInfo->u8KCSSeqNo==%d\n [3]",__FUNCTION__,psInfo->u8KCSSeqNo);                        
                        
                        aess_kcs_errorhandler(psInfo->u8Channel);
                    }
                    
                    break;
                    
                case KCS_RECEIVE_DATA_STATE:
                    
                    /* set BMC status to write state */
                    aess_kcs_setstate(psInfo->u8Channel, KCS_STATUS_WRITE_STATE);
                    
                    /* dummy data to generate OBF (wr_end_cmd, wr_start) */
                    aess_kcs_setreg(u8Channel, KCS_REG_ODR, KCS_DUMMY_DATA);
                    u8Command = aess_kcs_getreg(u8Channel , KCS_REG_IDR);
                    
                    if (KCS_WRITE_END == u8Command)
                    {
                        /* if " WRITE END" command be issued from SMS then set
                        KCS machine state to receive last byte state */
                        psInfo->u8HIFISRState = KCS_RECEIVE_DATA_END_STATE;

                        DRV_URGENT("%s::KCS_RECEIVE_DATA_END_STATE::psInfo->u8KCSSeqNo==%d  [4]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        

                    }
                    else if (KCS_WRITE_START == u8Command)
                    {
                        /* if "WRITE START" command be issued from SMS then set
                        KCS machine state back to receive data state and clear
                        receive buffer size */
                        psInfo->u8HIFISRState = KCS_RECEIVE_DATA_STATE;
                        
                        psInfo->u8HIFInDataSize = 0;
                        ++psInfo->u8KCSSeqNo;

                        DRV_URGENT("%s::KCS_RECEIVE_DATA_STATE::KCS_WRITE_START == u8Command::psInfo->u8KCSSeqNo==%d  [5]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        
                        
                    }
                    else if (KCS_ABORT_CMD == u8Command)
                    {
                        /* if "ABORT" command be issued from SMS then set BMC
                        and KCS machine state to IDLE state, and record error
                        code [01] */
                        psInfo->u8HIFISRState = KCS_RESPONSE_STATUS_PHASE1_STATE;
                        psInfo->u8HIFStatusCode = KCS_ABORTED_BY_COMMAND;
                        
                        DRV_URGENT("%s::KCS_IDLE_STATE::u8Command == KCS_ABORT_CMD ::psInfo->u8KCSSeqNo==%d  [6]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        
                        
                    }
                    else
                    {
                        /* if the command is not expacted then record error
                        code [02] and go to KCS error handler sub-routine */
                        psInfo->u8HIFStatusCode = KCS_ILLEGAL_CONTROL_CODE;

                        DRV_URGENT("%s::KCS_ILLEGAL_CONTROL_CODE::psInfo->u8KCSSeqNo==%d  [7]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        

                        aess_kcs_errorhandler(psInfo->u8Channel);
                    }
                    
                    break;
                    
                case KCS_TRANSFER_DATA_STATE:
                    
                    /* falls through */
                case KCS_TRANSFER_DATA_END_STATE:
                    
                    /* BMC sets status to WRITE_STATE */
                    aess_kcs_setstate(psInfo->u8Channel,KCS_STATUS_WRITE_STATE);
                    
                    /* dummy data to generate OBF (wr_start)*/
                    aess_kcs_setreg(u8Channel, KCS_REG_ODR, KCS_DUMMY_DATA);
                    
                    /* get the command byte */
                    u8Command = aess_kcs_getreg(u8Channel , KCS_REG_IDR);
                    
                    if (KCS_WRITE_START == u8Command)
                    {
                        /* if "WRITE START" command be issued from SMS then set
                        KCS machine state back to receive data state and clear
                        receive buffer size */
                        psInfo->u8HIFISRState = KCS_RECEIVE_DATA_STATE;

                        psInfo->u8HIFInDataSize = 0;
                        ++psInfo->u8KCSSeqNo;
                        
                        DRV_URGENT("%s::KCS_RECEIVE_DATA_STATE::KCS_WRITE_START == u8Command::psInfo->u8KCSSeqNo==%d  [8]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        
                        
                    }
                    else if (KCS_ABORT_CMD == u8Command)
                    {
                        /* if "ABORT" command be issued from SMS then set BMC
                        and KCS machine state to IDLE state, and record error
                        code [01] */
                        
                        psInfo->u8HIFISRState = KCS_RESPONSE_STATUS_PHASE1_STATE;
                        psInfo->u8HIFStatusCode = KCS_ABORTED_BY_COMMAND;
                        

                        DRV_URGENT("%s::KCS_IDLE_STATE::KCS_ABORT_CMD == u8Command::psInfo->u8KCSSeqNo==%d  [9]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        
                    }
                    else
                    {
                        /* if the command is not expacted then record error
                        code [02] and go to KCS error handler sub-routine */
                        psInfo->u8HIFStatusCode = KCS_ILLEGAL_CONTROL_CODE;
                        
                        DRV_URGENT("%s::KCS_ILLEGAL_CONTROL_CODE::psInfo->u8KCSSeqNo==%d [10]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        

                        aess_kcs_errorhandler(psInfo->u8Channel);
                    }

                    #ifdef SUPPORT_KCS_TX_EVENT
                        /* Send event to tell high-level firmware that KCS driver
                        has been terminate message transfer */
                        if( (-STATUS_FAIL) == aess_generate_driver_event(psInfo->u16DriverID, psInfo->u32KCSTxFailEvent ) )
                        {
                            DRV_ERROR("%s::generate_driver_event call faill!! (1)\n" );                                              
                        }
                    #endif                    
                    
                    break;
                    
                case KCS_ERROR_STATE:
                    
                    /* BMC sets status to WRITE_STATE */
                    aess_kcs_setstate(psInfo->u8Channel,KCS_STATUS_WRITE_STATE);
                    
                    /* dummy data to generate OBF (error1) */
                    aess_kcs_setreg(u8Channel, KCS_REG_ODR, KCS_DUMMY_DATA);
                    
                    /* get the command byte */
                    u8Command = aess_kcs_getreg(u8Channel , KCS_REG_IDR);
                    
                    if (KCS_GET_STATUS == u8Command)
                    {
                        /* if "GET STATUS" command be issued from 
                        SMS then change KCS machine state to get 
                        status phase1 state */
                        
                        psInfo->u8HIFISRState = KCS_RESPONSE_STATUS_PHASE1_STATE;

                        psInfo->u8HIFStatusCode = KCS_ABORTED_BY_COMMAND;

                        DRV_URGENT("%s::KCS_RESPONSE_STATUS_PHASE1_STATE::KCS_GET_STATUS == u8Command::psInfo->u8KCSSeqNo==%d [11]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        
                    }
                    else if (KCS_WRITE_START == u8Command)
                    {
                        /* if "WRITE START" command be issued from 
                        SMS then set KCS machine state back to receive 
                        data state and clear receive buffer size */
                        psInfo->u8HIFISRState = KCS_RECEIVE_DATA_STATE;
                        
                        psInfo->u8HIFInDataSize = 0;
                        ++psInfo->u8KCSSeqNo;

                        DRV_URGENT("%s::KCS_RECEIVE_DATA_STATE::KCS_WRITE_START == u8Command::psInfo->u8KCSSeqNo==%d [12]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        
                    }
                    else
                    {
                        /* if the command is not expacted then record error
                        code [02] and go to KCS error handler sub-routine */
                        psInfo->u8HIFStatusCode = KCS_ILLEGAL_CONTROL_CODE;

                        DRV_URGENT("%s::KCS_ILLEGAL_CONTROL_CODE::psInfo->u8KCSSeqNo==%d [13]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        

                        aess_kcs_errorhandler(psInfo->u8Channel);
                    }
                    
                    break;
                    
                default:
                    
                    /* if the command is not expacted then record error
                     code [FF] and go to KCS error handler sub-routine */
                    psInfo->u8HIFStatusCode = KCS_UNSPECIFIED_ERROR;
                    
                    /* dummy data to generate OBF (wr_start) */
                    aess_kcs_setreg(u8Channel, KCS_REG_ODR, KCS_DUMMY_DATA);
                    
                    DRV_URGENT("%s::KCS_UNSPECIFIED_ERROR::psInfo->u8KCSSeqNo==%d [14]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        

                    aess_kcs_errorhandler(psInfo->u8Channel);
                    
                    break;
                    
            }   /* end of switch (psInfo->u8HIFISRState) */
        }   /* end of if (KCS_COMMAND == REG_STR.EQU.BIT.CD) */
        else    /* Data in */
        {
            switch (psInfo->u8HIFISRState)
            {
                case KCS_RECEIVE_DATA_STATE:
                    
                    /* BMC sets status to WRITE_STATE */
                    aess_kcs_setstate(psInfo->u8Channel,KCS_STATUS_WRITE_STATE);
                   
                    /* dummy data to generate OBF (wr_data)*/
                    aess_kcs_setreg(u8Channel, KCS_REG_ODR, KCS_DUMMY_DATA);
                    
                    /* get the data byte */
                    u8Data = aess_kcs_getreg(u8Channel , KCS_REG_IDR);
                    
                    if (psInfo->u8HIFInDataSize < KCS_PACKAGE_MAX)
                    {
                        /* save incoming byte from SMS */
                        psInfo->au8HIFReceiveBuf[psInfo->u8HIFInDataSize++] = u8Data;

                        DRV_URGENT("%s::Data_In::KCS_RECEIVE_DATA_STATE::psInfo->u8KCSSeqNo==%d [15]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        
                    }
                    else
                    {
                        /* if the length of incoming message over 40 bytes then
                        record error code [06] */
                        psInfo->u8HIFStatusCode = KCS_LENGTH_ERROR;
                        
                        DRV_URGENT("%s::Data_In::KCS_RECEIVE_DATA_STATE::KCS_LENGTH_ERROR::psInfo->u8KCSSeqNo==%d [16]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        

                        aess_kcs_errorhandler(psInfo->u8Channel);
                    }

                    break;
                    
                case KCS_RECEIVE_DATA_END_STATE:
                    
                    /* BMC sets status to READ_STATE */
                    aess_kcs_setstate(psInfo->u8Channel, KCS_STATUS_READ_STATE);
                    
                    /* get the data byte */
                    u8Data = aess_kcs_getreg(u8Channel , KCS_REG_IDR);
                    
                    /* change KCS machine state to transfer data state */
                    psInfo->u8HIFISRState = KCS_TRANSFER_DATA_STATE;
                    
                    if (psInfo->u8HIFInDataSize < KCS_PACKAGE_MAX)
                    {
                        /* save the last incoming byte from SMS */
                        psInfo->au8HIFReceiveBuf[psInfo->u8HIFInDataSize++] = u8Data;
                        
                        /* Add sequence number to the last byte of 
                        KCS request data */
                        psInfo->au8HIFReceiveBuf[psInfo->u8HIFInDataSize++] = psInfo->u8KCSSeqNo;
                        
                        /* Send event to tell high-level firmware that KCS
                        driver has been got a incoming message from SMS */
                        if ( (-STATUS_FAIL) == aess_generate_driver_event( psInfo->u16DriverID,psInfo->u32KCSRxOKEvent ) )
                        {
                            DRV_ERROR("%s::generate_driver_event call faill!! (2)\n",__FUNCTION__ );                                              
                        }

                        DRV_URGENT("%s::Data_In::KCS_TRANSFER_DATA_STATE::psInfo->u8KCSSeqNo==%d [17]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        
                    }
                    else
                    {
                        /* if the length of incoming message over 40 bytes then  record error code [06] */
                        psInfo->u8HIFStatusCode = KCS_LENGTH_ERROR;

                        DRV_URGENT("%s::Data_In::KCS_LENGTH_ERROR::psInfo->u8KCSSeqNo==%d [18]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        
                        
                        aess_kcs_errorhandler(psInfo->u8Channel);
                    }
                    
                    break;
                    
                case KCS_TRANSFER_DATA_STATE:
                    
                    /* get the data byte */
                    u8Data = aess_kcs_getreg(u8Channel , KCS_REG_IDR);
                    
                    if (KCS_READ == u8Data)
                    {
                        if (psInfo->u8HIFOutDataSize > 0)
                        {
                            psInfo->u8HIFOutDataSize--;
                            
                            if (0 == psInfo->u8HIFOutDataSize)
                            {
                                /* Transfer End */
                                psInfo->u8HIFISRState = KCS_TRANSFER_DATA_END_STATE;

                                DRV_URGENT("%s::Data_In::KCS_TRANSFER_DATA_END_STATE::psInfo->u8KCSSeqNo==%d [19]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        
                            }
                            else
                            {
                                DRV_URGENT("%s::Data_In::KCS_TRANSFER_DATA_STATE::psInfo->u8KCSSeqNo==%d [20]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        
                            }
                            
                            /* OBF (read), put one byte of output message to
                            data out register */
                            aess_kcs_setreg(u8Channel, KCS_REG_ODR, *psInfo->pu8HIFOutDataPtr++);
                        }
                        else
                        {

                            #ifdef SUPPORT_KCS_TX_EVENT
                                /* if the length less than 0 then send event to
                                tell high-level firmware that KCS driver has
                                been terminate message transfer */
                                if ( (-STATUS_FAIL) == aess_generate_driver_event(psInfo->u16DriverID,psInfo->u32KCSTxFailEvent ) )
                                {
                                    DRV_ERROR("%s::generate_driver_event call faill!! (3)\n",__FUNCTION__ );                                              
                                }
                            #endif                            
                            
                            /* record error code [06] */
                            psInfo->u8HIFStatusCode = KCS_LENGTH_ERROR;

                            DRV_URGENT("%s::Data_In::KCS_LENGTH_ERROR::psInfo->u8KCSSeqNo==%d [21]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        
                            
                            aess_kcs_errorhandler(psInfo->u8Channel);
                        }
                    }
                    else
                    {
                        #ifdef SUPPORT_KCS_TX_EVENT
                            /* if receive a not expacted control code then send
                            event to tell high-level firmware that KCS driver has
                            been terminate message transfer */
                            if ( (-STATUS_FAIL) == aess_generate_driver_event(psInfo->u16DriverID,psInfo->u32KCSTxFailEvent )  )
                            {
                                DRV_ERROR("%s::generate_driver_event call faill!! (4)\n",__FUNCTION__ );                                              
                            }
                        #endif
                                                    
                        /* record error code [02] */
                        psInfo->u8HIFStatusCode = KCS_ILLEGAL_CONTROL_CODE;

                        DRV_URGENT("%s::Data_In::KCS_ILLEGAL_CONTROL_CODE::psInfo->u8KCSSeqNo==%d [22]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        
                            
                        aess_kcs_errorhandler(psInfo->u8Channel);
                    }
                    
                    break;
                    
                /* Transfer End */
                case KCS_TRANSFER_DATA_END_STATE:
                    
                    /* set BMC status to IDLE state */
                    aess_kcs_setstate(psInfo->u8Channel, KCS_STATUS_IDLE_STATE);
                    
                    /* OBF (read), write a dummy byte to data out register */
                    
                    aess_kcs_setreg(u8Channel, KCS_REG_ODR, KCS_DUMMY_DATA);
                    
                    /* get the data byte */
                    u8Data = aess_kcs_getreg(u8Channel , KCS_REG_IDR);
                    
                    if (KCS_READ == u8Data)
                    {
                        /* KCS machine state back to idle state */
                        psInfo->u8HIFISRState = KCS_IDLE_STATE;
                        
                        /* record error code [00] */
                        psInfo->u8HIFStatusCode = KCS_NO_ERROR;
                        
                        #ifdef SUPPORT_KCS_TX_EVENT
                            /* Send event to tell high-level firmware that KCS
                            driver has been transfer response data completely */
                            if ( (-STATUS_FAIL) ==  aess_generate_driver_event( psInfo->u16DriverID, psInfo->u32KCSTxOKEvent ) )
                            {
                                DRV_ERROR("%s::generate_driver_event call faill!! (5)\n",__FUNCTION__ );                                              
                            }
                        #endif                             

                        DRV_URGENT("%s::Data_In::KCS_IDLE_STATE::psInfo->u8KCSSeqNo==%d [23]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        
                    }
                    else
                    {
                        #ifdef SUPPORT_KCS_TX_EVENT
                            /* if receive a not expacted control code then send
                            event to tell high-level firmware that KCS driver has
                            been terminate message transfer */
                            if ( (-STATUS_FAIL) == aess_generate_driver_event( psInfo->u16DriverID,psInfo->u32KCSTxFailEvent )  )
                            {
                                DRV_ERROR("%s::generate_driver_event call faill!! (6)\n",__FUNCTION__ );                                              
                            }
                        #endif
                            
                        /* record error code [02] */
                        psInfo->u8HIFStatusCode = KCS_ILLEGAL_CONTROL_CODE;

                        DRV_URGENT("%s::Data_In::KCS_ILLEGAL_CONTROL_CODE::psInfo->u8KCSSeqNo==%d [24]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        

                        aess_kcs_errorhandler(psInfo->u8Channel);
                    }
                    
                    break;

                case KCS_RESPONSE_STATUS_PHASE1_STATE:
                    
                    /* set BMC status to read state */
                    aess_kcs_setstate(psInfo->u8Channel, KCS_STATUS_READ_STATE);
                    u8Data = aess_kcs_getreg(u8Channel , KCS_REG_IDR);
                    
                    if (KCS_ZERO_DATA == u8Data)
                    {
                        aess_kcs_setstate(psInfo->u8Channel, KCS_STATUS_READ_STATE);
                        psInfo->u8HIFISRState  = KCS_RESPONSE_STATUS_PHASE2_STATE;

                        /* OBF (error2)*/
                        aess_kcs_setreg(u8Channel, KCS_REG_ODR, psInfo->u8HIFStatusCode);
                        
                        DRV_URGENT("%s::Data_In::KCS_RESPONSE_STATUS_PHASE2_STATE::psInfo->u8KCSSeqNo==%d [25]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        
                    }
                    else
                    {
                        psInfo->u8HIFStatusCode = KCS_ILLEGAL_CONTROL_CODE;
                        
                        DRV_URGENT("%s::Data_In::KCS_ILLEGAL_CONTROL_CODE::psInfo->u8KCSSeqNo==%d [26]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        
                        
                        aess_kcs_errorhandler(psInfo->u8Channel);
                    }
                    
                    break;
                    
                case KCS_RESPONSE_STATUS_PHASE2_STATE:
                    
                    aess_kcs_setstate(psInfo->u8Channel, KCS_STATUS_IDLE_STATE);
                    
                    /* dummy data to generate OBF (error3)*/
                    aess_kcs_setreg(u8Channel, KCS_REG_ODR, KCS_DUMMY_DATA);
                    
                    u8Data = aess_kcs_getreg(u8Channel , KCS_REG_IDR);
                    
                    if (KCS_READ == u8Data)
                    {
                        psInfo->u8HIFStatusCode = KCS_NO_ERROR;
                        psInfo->u8HIFISRState = KCS_IDLE_STATE;

                        DRV_URGENT("%s::Data_In::KCS_IDLE_STATE::psInfo->u8KCSSeqNo==%d [27]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        
                    }
                    else
                    {
                        psInfo->u8HIFStatusCode = KCS_ILLEGAL_CONTROL_CODE;
                        
                        DRV_URGENT("%s::Data_In::KCS_ILLEGAL_CONTROL_CODE::psInfo->u8KCSSeqNo==%d [28]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        

                        aess_kcs_errorhandler(psInfo->u8Channel);
                    }
                    
                    break;
                    
                default:
                    
                    psInfo->u8HIFStatusCode = KCS_UNSPECIFIED_ERROR;
                    
                    /* dummy data to generate OBF (wr_start) */
                    aess_kcs_setreg(u8Channel, KCS_REG_ODR, KCS_DUMMY_DATA);
                    
                    DRV_URGENT("%s::Data_In::KCS_UNSPECIFIED_ERROR::psInfo->u8KCSSeqNo==%d [29]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        

                    aess_kcs_errorhandler(psInfo->u8Channel);
                    break;
                    
            }  /* end of switch (psInfo->u8HIFISRState) */
        }   /* end of else */
    }   /* end of if (REG_STR.EQU.BIT.IBF) */
    else
    {
        DRV_URGENT("%s::KCS_REG_IBF == 0::psInfo->u8KCSSeqNo==%d [30]\n ",__FUNCTION__,psInfo->u8KCSSeqNo);                        
        return;
    }
}
/******************************************************************************
*   FUNCTION        :   aess_kcs_error_isr_handler
******************************************************************************/
/**
 *  @brief      The function is invoked error isr handler.
 *
 *  @return     None                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Internal function \n
 *
 *****************************************************************************/
void aess_kcs_error_isr_handler(void)
{
    volatile UINT8 u8Tmp = 0;
    
    u8Tmp = *((UINT8 *) LPC_HICR2_REG);
    
    DEBUGP("aess_kcs_error_isr_handler bf(%x)", u8Tmp);
    
    /* clear error condition of LPC hardware reset occurs */
    if (u8Tmp & LPC_LRST)
    {
        DEBUGP(" LRST");
    }
    
    /* clear error condition when a forced termination of an LPC transfer
       cycle occurs */
    if (u8Tmp & LPC_SDWN)
    {
        DEBUGP(" SDWN");
    }
    
    /* clear error condition of LPC hardware shutdown request is generated */
    if (u8Tmp & LPC_ABRT)
    {
        DRV_INFO("%s:: ABRT !!\n",__FUNCTION__);
    }
    
    if (u8Tmp & (LPC_LRST | LPC_SDWN))
    {
        /* if LPC 1 is present */
        if (Su8LPCInirSata & 0x01)
        {
            DRV_INFO("%s:: LPC_LRST | LPC_SDWN :: LPC1  \n",__FUNCTION__);
            aess_kcs_hw_init(KCS_CH_1);
        }
        
        /* if LPC 2 is present */
        if (Su8LPCInirSata & 0x02)
        {
            DRV_INFO("%s:: LPC_LRST | LPC_SDWN :: LPC2  \n",__FUNCTION__);
            aess_kcs_hw_init(KCS_CH_2);
        }
        
        /* if LPC 3 is present */
        if (Su8LPCInirSata & 0x04)
        {
            DRV_INFO("%s:: LPC_LRST | LPC_SDWN :: LPC3  \n",__FUNCTION__);
            aess_kcs_hw_init(KCS_CH_3);
        }
    }
    
    /* Clear LPC interrupt flags */
    *((UINT8 *) LPC_HICR2_REG) = u8Tmp & 0x0F;

}


/******************************************************************************
*   FUNCTION        :   aess_kcs_isr
******************************************************************************/
/**
 *  @brief      The function is the entry function of interrupt handler
 *
 *  @return     IRQ_NONE if the interrupt not occured KCS channel.
 *              IRQ_HANDLED if KCS channel interrupt occured.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Internal function \n
 *
 *****************************************************************************/
 static irqreturn_t aess_kcs_isr (
                                  /** irq number  */
                                  int irq, 
                                  
                                  /**  point to current dev data structure  */
                                  void *dev_id
                                  )
{
   volatile irqreturn_t irqreturn = IRQ_NONE;
   volatile int kcs_index=0; 
   volatile UINT8 u8Tmp=0; 
    
    /* if there is an error interrupt */
    if (*((UINT8 *) LPC_HICR2_REG) & (LPC_SDWN | LPC_ABRT))
    {
        /* invoke error isr handler */
        aess_kcs_error_isr_handler();
        irqreturn = IRQ_HANDLED;
    }
    if (*((UINT8 *) LPC_HICR2_REG) & (LPC_LRST) )
    {
        for( kcs_index =0 ; kcs_index <  Su8LPCInirSata ; kcs_index++ )
        {
            if( Su8LPCInirSata & ( 0x01<<kcs_index) )
            {      
                S_aKCSData[kcs_index].u8HIFISRState = KCS_IDLE_STATE;
                S_aKCSData[kcs_index].u8HIFStatusCode = KCS_NO_ERROR;
                S_aKCSData[kcs_index].u8HIFStatusCode = KCS_SEQ_NO_INIT;
                S_aKCSData[kcs_index].u8KCSSeqNo = 0;
                aess_kcs_setstate(kcs_index, KCS_STATUS_ERROR_STATE);
                aess_kcs_hw_init(kcs_index);        
            }
        } 
        DRV_URGENT("\n        aess_kcs_isr------LPC_LRST      \n");                        
        tasklet_schedule(&lpcreset_tasklet);
        u8Tmp = *((UINT8 *) LPC_HICR2_REG) ;
        *((UINT8 *) LPC_HICR2_REG) = u8Tmp & 0x0F;
       
        irqreturn = IRQ_HANDLED;
        return irqreturn;
    }

    /* if the interrupt is generated by LPC 1 */
    if (aess_kcs_getreg(KCS_CH_1, KCS_REG_IBF))
    {
        /* KCS 0 */
        aess_kcs_isr_handler(KCS_CH_1);
        irqreturn = IRQ_HANDLED;
    }
    
    /* if the interrupt is generated by LPC 2 */
    if (aess_kcs_getreg(KCS_CH_2, KCS_REG_IBF))
    {
        /* KCS 1 */
        aess_kcs_isr_handler(KCS_CH_2);
        irqreturn = IRQ_HANDLED;
    }
    
    /* if the interrupt is generated by LPC 3 */
    if (aess_kcs_getreg(KCS_CH_3, KCS_REG_IBF))
    {
        /* KCS 2 */
        aess_kcs_isr_handler(KCS_CH_3);
        irqreturn = IRQ_HANDLED;
    }
   
    return irqreturn;
}


/******************************************************************************
*   FUNCTION        :   aess_kcsdrv_bh 
******************************************************************************/
/**
 *  @brief      notify event handler driver.
 *
 *  @return     None
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Private function\n
 *
 *****************************************************************************/
static void aess_kcsdrv_bh(
                            /** Unused data */
                            UINT32 u32Data
                          )
{
  /*Set CBFunEventID to event handler driver*/
  aess_generate_driver_event(S_sLPCResetCBID.u16CBFunDriverID, S_sLPCResetCBID.u32CBFunEventID);
}



#ifdef KCS_POLLING
/******************************************************************************
*   FUNCTION        :   aess_kcs_polling_func
******************************************************************************/
/**
 *  @brief      The function is for debug to dump register value.
 *
 *  @return     None.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Internal  function \n
 *
 *****************************************************************************/
void aess_kcs_polling_func(
                           /** reserve */ 
                           unsigned long data
                           )
{
    DRV_MSG("%s :: show the register value.\n", __FUNCTION__ );
    DRV_MSG("LPC_HICR0_REG   %xh\n", *((UINT8 *) LPC_HICR0_REG)); 
    DRV_MSG("LPC_HICR1_REG   %xh \n", *((UINT8 *) LPC_HICR1_REG)); 
    DRV_MSG("LPC_HICR2_REG   %xh \n", *((UINT8 *) LPC_HICR2_REG)); 
    DRV_MSG("LPC_HICR3_REG   %xh \n", *((UINT8 *) LPC_HICR3_REG)); 
    DRV_MSG("LPC_HICR4_REG   %xh \n", *((UINT8 *) LPC_HICR4_REG)); 
    DRV_MSG("LPC_LADR3H_REG  %xh \n",  *((UINT8 *) LPC_LADR3H_REG)); 
    DRV_MSG("LPC_LADR3L_REG  %xh \n",  *((UINT8 *) LPC_LADR3L_REG)); 
    DRV_MSG("LPC_LADR12H_REG %xh \n",  *((UINT8 *) LPC_LADR12H_REG)); 
    DRV_MSG("LPC_LADR12L_REG %xh \n",  *((UINT8 *) LPC_LADR12L_REG)); 
    DRV_MSG("LPC_IDR1_REG    %xh \n",  *((UINT8 *) LPC_IDR1_REG)); 
    DRV_MSG("LPC_IDR2_REG    %xh \n",  *((UINT8 *) LPC_IDR2_REG)); 
    DRV_MSG("LPC_IDR3_REG    %xh \n",  *((UINT8 *) LPC_IDR3_REG)); 
    DRV_MSG("LPC_ODR1_REG    %xh \n",  *((UINT8 *) LPC_ODR1_REG)); 
    DRV_MSG("LPC_ODR2_REG    %xh \n",  *((UINT8 *) LPC_ODR2_REG)); 
    DRV_MSG("LPC_ODR3_REG    %xh \n",  *((UINT8 *) LPC_ODR3_REG)); 
    DRV_MSG("LPC_STR1_REG    %xh \n",  *((UINT8 *) LPC_STR1_REG)); 
    DRV_MSG("LPC_STR2_REG    %xh \n",  *((UINT8 *) LPC_STR2_REG)); 
    DRV_MSG("LPC_STR3_REG    %xh \n",  *((UINT8 *) LPC_STR3_REG)); 
    
    /* timer interval is 10s */
    kcs_polling.expires = jiffies + 5000;
    add_timer(&kcs_polling);
    
    return;
}
#endif

/******************************************************************************
*   FUNCTION        :   aess_kcs_close
******************************************************************************/
/**
 *  @brief      The function is invoked when application close the device note.
 *
 *  @return     0 never fail.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function \n
 *
 *****************************************************************************/
static int aess_kcs_close(
                         /** inode structure point */
                         struct inode *inode, 
                         /** file structure point */
                         struct file *filp
                         )
{
    /* disable all KCS interrupts */
    *((UINT8 *) LPC_HICR0_REG) = 0;
    
#ifdef KCS_POLLING
    del_timer(&kcs_polling);
#endif
    
    DRV_INFO("%s::return(STATUS_OK)\n",__FUNCTION__);    
    return 0;
}

/******************************************************************************
*   FUNCTION        :   aess_kcs_open
******************************************************************************/
/**
 *  @brief      The function is invoked when application open driver note.
 *
 *  @return     STATUS_OK never fail.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function \n
 *
 *****************************************************************************/
static int aess_kcs_open(
                         /** inode structure point */
                         struct inode *inode, 
                         
                         /** file structure point */
                         struct file *filp
                         )
{
    /* Set the initial value for data structure */    
    memset(S_aKCSData, 0, sizeof(sKCSStruct) * KCS_TOTAL_CHANNEL);
    
#ifdef KCS_POLLING
    /* initiate polling timer */
    kcs_polling.data = 0;
    kcs_polling.function = &aess_kcs_polling_func;
    
    /* timer interval is 10s */
    kcs_polling.expires = jiffies + 5000;
    
    init_timer(&kcs_polling);
    add_timer(&kcs_polling);
#endif

        
    DRV_INFO("%s::return(STATUS_OK)\n",__FUNCTION__);    
    return (STATUS_OK);  
}


struct file_operations aess_kcs_fops = {
    .open = aess_kcs_open, 
    .ioctl = aess_kcs_ioctl,
    .release = aess_kcs_close,
};

/******************************************************************************
*   FUNCTION        :   aess_kcs_init
******************************************************************************/
/**
 *  @brief      The function is invoked when KCS module is loaded into the kernel.
 *
 *  @return     STATUS_OK if driver initial success.
 *              Others if initial fail.                   
 *
 *  @dependency EventHandler Driver
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function  \n
 *
 *****************************************************************************/
int __init aess_kcs_init(void)
{
    int result;

#ifdef AST2100
	aess_kcs_hw_reset();
#endif
    kcs_cdev = cdev_alloc();
    
    kcs_cdev->ops = &aess_kcs_fops;
    result = alloc_chrdev_region(&kcs_dev, 0, 1, driver_name);
    
    if (result < 0) 
    {
        printk (KERN_ERR "KN:Registering the kcs device failed with %d\n", MAJOR(kcs_dev));
        return result;
    }

    /* show this message for kcs install shell used */
    printk("mknod /dev/aess_kcsdrv c %d 0\n", MAJOR(kcs_dev));
   
    cdev_add(kcs_cdev, kcs_dev, 1);

    /** Set the IRQ to LEVEL trigger */
    IRQ_SET_LEVEL_TRIGGER(IRQ_LPC);

    /** Set the IRQ to be active high */
    IRQ_SET_HIGH_LEVEL(IRQ_LPC);

    /* Register IRQ, assign channel 0 data structure for dev_id */
    if (request_irq(IRQ_LPC, aess_kcs_isr,   IRQF_SAMPLE_RANDOM | IRQF_SHARED, "KCS",(void *)S_aKCSData) ) 
    {
        DRV_ERROR("%s::return(-EBUSY):: irq reuest fail!\n",__FUNCTION__);        
        return -EBUSY;
    }
    
    /* debug message */
    aess_debugfs_default_create(driver_name,NULL,0,0);
    aess_debugfs_create_file(driver_name,"u32",(u32 *)&u32tmpcounter,DBG_TYPE32,0);
    aess_debugfs_create_file(driver_name,"u16",(u16 *)&u16tmpcounter,DBG_TYPE16,0);
    aess_debugfs_create_file(driver_name,"u8",(u8 *)&u8tmpcounter,DBG_TYPE8,0);
   
    DRV_INFO("%s::return(STATUS_OK)\n",__FUNCTION__);    
    return (STATUS_OK);
}

/******************************************************************************
*   FUNCTION        :   aess_kcs_exit
******************************************************************************/
/**
 *  @brief      The function is invoked when KCS driver be removed from kernal.
 *
 *  @return     None.                   
 *
 *  @dependency None
 *
 *  @limitation None
 *
 *  @warning    None
 *
 *  @note       None
 *
 *  @internal   Function Type: Global function \n
 *
 *****************************************************************************/
void __exit aess_kcs_exit(void)
{
    aess_kcs_clearall();
    cdev_del(kcs_cdev);
    unregister_chrdev_region(kcs_dev, 1);
    
    DRV_INFO("%s:: return\n",__FUNCTION__);    
    aess_debugfs_remove(driver_name,NULL,0,0);  
    return;
}


MODULE_DESCRIPTION("AESS KCS Driver");
MODULE_AUTHOR("Taiyi Kuo<taiyi.kuo@avocent.com>");
MODULE_LICENSE("GPL");
MODULE_VERSION("0.1.0.0");
module_param(driver_name, charp, S_IRUGO);
module_init(aess_kcs_init);
module_exit(aess_kcs_exit);


/* End of code */
