/*
 *  linux/arch/arm/mach-ast2000/irq.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <linux/init.h>
#include <linux/stddef.h>
#include <linux/list.h>
#include <linux/timer.h>

#include <asm/hardware.h>
#include <asm/irq.h>
#include <asm/io.h>

#include <asm/mach/irq.h>

static unsigned int intCtrl[NR_IRQS] =
{
	#if 	NR_IRQS > 0
	(INTC_TRIG_0 << INTC_TRIG_SHIFT) | (INTC_PRIO_0 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 0 */
	#if 	NR_IRQS > 1
	(INTC_TRIG_1 << INTC_TRIG_SHIFT) | (INTC_PRIO_1 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 1 */
	#if 	NR_IRQS > 2
	(INTC_TRIG_2 << INTC_TRIG_SHIFT) | (INTC_PRIO_2 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 2 */
	#if 	NR_IRQS > 3
	(INTC_TRIG_3 << INTC_TRIG_SHIFT) | (INTC_PRIO_3 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 3 */
	#if 	NR_IRQS > 4
	(INTC_TRIG_4 << INTC_TRIG_SHIFT) | (INTC_PRIO_4 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 4 */
	#if 	NR_IRQS > 5
	(INTC_TRIG_5 << INTC_TRIG_SHIFT) | (INTC_PRIO_5 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 5 */
	#if 	NR_IRQS > 6
	(INTC_TRIG_6 << INTC_TRIG_SHIFT) | (INTC_PRIO_6 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 6 */
	#if 	NR_IRQS > 7
	(INTC_TRIG_7 << INTC_TRIG_SHIFT) | (INTC_PRIO_7 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 7 */
	#if 	NR_IRQS > 8
	(INTC_TRIG_8 << INTC_TRIG_SHIFT) | (INTC_PRIO_8 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 8 */
	#if 	NR_IRQS > 9
	(INTC_TRIG_9 << INTC_TRIG_SHIFT) | (INTC_PRIO_9 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 9 */
	#if 	NR_IRQS > 10
	(INTC_TRIG_10 << INTC_TRIG_SHIFT) | (INTC_PRIO_10 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 10 */
	#if 	NR_IRQS > 11
	(INTC_TRIG_11 << INTC_TRIG_SHIFT) | (INTC_PRIO_11 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 11 */
	#if 	NR_IRQS > 12
	(INTC_TRIG_12 << INTC_TRIG_SHIFT) | (INTC_PRIO_12 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 12 */
	#if 	NR_IRQS > 13
	(INTC_TRIG_13 << INTC_TRIG_SHIFT) | (INTC_PRIO_13 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 13 */
	#if 	NR_IRQS > 14
	(INTC_TRIG_14 << INTC_TRIG_SHIFT) | (INTC_PRIO_14 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 14 */
	#if 	NR_IRQS > 15
	(INTC_TRIG_15 << INTC_TRIG_SHIFT) | (INTC_PRIO_15 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 15 */
	#if 	NR_IRQS > 16
	(INTC_TRIG_16 << INTC_TRIG_SHIFT) | (INTC_PRIO_16 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 16 */
	#if 	NR_IRQS > 17
	(INTC_TRIG_17 << INTC_TRIG_SHIFT) | (INTC_PRIO_17 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 17 */
	#if 	NR_IRQS > 18
	(INTC_TRIG_18 << INTC_TRIG_SHIFT) | (INTC_PRIO_18 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 18 */
	#if 	NR_IRQS > 19
	(INTC_TRIG_19 << INTC_TRIG_SHIFT) | (INTC_PRIO_19 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 19 */
	#if 	NR_IRQS > 20
	(INTC_TRIG_20 << INTC_TRIG_SHIFT) | (INTC_PRIO_20 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 20 */
	#if 	NR_IRQS > 21
	(INTC_TRIG_21 << INTC_TRIG_SHIFT) | (INTC_PRIO_21 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 21 */
	#if 	NR_IRQS > 22
	(INTC_TRIG_22 << INTC_TRIG_SHIFT) | (INTC_PRIO_22 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 22 */
	#if 	NR_IRQS > 23
	(INTC_TRIG_23 << INTC_TRIG_SHIFT) | (INTC_PRIO_23 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 23 */
	#if 	NR_IRQS > 24
	(INTC_TRIG_24 << INTC_TRIG_SHIFT) | (INTC_PRIO_24 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 24 */
	#if 	NR_IRQS > 25
	(INTC_TRIG_25 << INTC_TRIG_SHIFT) | (INTC_PRIO_25 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 25 */
	#if 	NR_IRQS > 26
	(INTC_TRIG_26 << INTC_TRIG_SHIFT) | (INTC_PRIO_26 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 26 */
	#if 	NR_IRQS > 27
	(INTC_TRIG_27 << INTC_TRIG_SHIFT) | (INTC_PRIO_27 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 27 */
	#if 	NR_IRQS > 28
	(INTC_TRIG_28 << INTC_TRIG_SHIFT) | (INTC_PRIO_28 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 28 */
	#if 	NR_IRQS > 29
	(INTC_TRIG_29 << INTC_TRIG_SHIFT) | (INTC_PRIO_29 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 29 */
	#if 	NR_IRQS > 30
	(INTC_TRIG_30 << INTC_TRIG_SHIFT) | (INTC_PRIO_30 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 30 */
	#if 	NR_IRQS > 31
	(INTC_TRIG_31 << INTC_TRIG_SHIFT) | (INTC_PRIO_31 & INTC_PRIO_MASK),
	#endif /* INTC_MAX_IRQ > 31 */
/*
	#if 	NR_IRQS > 32
	#error "Insufficient entries are defined!"
	#endif 
*/
};

  
void __init uP_intc_init(void)
{
	int		n;

	/* disable all interrupts */

        IO_WRITE(0, INTC_IECR); //+grant
	IO_WRITE(0xffffffff, INTC_ICCR ); /* clear all pending intr*/
	 
	/* config the interrupt controller */
	for (n = 0; n < NR_IRQS; n++)
	{
		/* kernel only need IRQ no, for it maintains irqaction, i.e. ISR
		* and it will be called in do_IRQ
		*/
                IO_WRITE(intCtrl[n]    ,INTC_SCR0+INTC_REG_SIZE*n ); //+grant
                IO_WRITE(n ,INTC_SVR0+INTC_REG_SIZE*n  ); //+grant
	}

	/* clear any pending interrupts */
	IO_WRITE(0xffffffff, INTC_ICCR ); /* clear all pending intr*/
	IO_WRITE(0, INTC_IMR );/* mask all interrupt */

}


static void uP_unmask_irq(unsigned int irq)
{
        unsigned int iecr;
      
        iecr = IO_READ(INTC_IMR);
      
        iecr |= (1 << (irq) );
      
        IO_WRITE(iecr, INTC_IMR);
        IO_WRITE(iecr, INTC_IECR);

}

static void uP_mask_irq(unsigned int irq)
{
        unsigned int iecr;

        iecr = IO_READ(INTC_IMR);
        iecr &= ~(1 << (irq) );
        IO_WRITE(iecr, INTC_IECR);
        IO_WRITE(iecr, INTC_IMR);
      
        IO_WRITE(1 << (irq),INTC_ICCR);
}

static void uP_ack_irq(unsigned int irq)
{
        unsigned int iecr;    
      
        uP_mask_irq(irq); //+grant

        IO_WRITE(1 << (irq),INTC_ICCR);
}

static struct irqchip sc_chip = {
	.ack	= uP_ack_irq,
	.mask	= uP_mask_irq,
	.unmask = uP_unmask_irq,
}; 
 
void __init ast2000_init_irq(void)
{
	unsigned int i;

	uP_intc_init();

	for (i = 0; i < NR_IRQS; i++) 
	{
	    set_irq_chip(i, &sc_chip);
	    set_irq_handler(i, do_level_IRQ);
	    set_irq_flags(i, IRQF_VALID);
	}

 	 IO_WRITE(0x00000000, INTC_IMR );/* mask all interrupt */
 	 IO_WRITE(0x00000000, INTC_IECR );/* mask all interrupt */
	 IO_WRITE(0xffffffff, INTC_ICCR ); /* clear all pending intr*/
}
