/*
 *  linux/arch/arm/arch-ast2000/timer.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/device.h>
#include <linux/spinlock.h>
#include <linux/interrupt.h>
#include <linux/sched.h>

#include <asm/system.h>
#include <asm/io.h>
#include <asm/arch/hardware.h>
#include <asm/arch/irqs.h>
#include <asm/arch/time.h>

/*
 * Returns number of ms since last clock interrupt.  Note that interrupts
 * will have been disabled by do_gettimeoffset()
 */
static unsigned long ast2000_gettimeoffset(void)
{

	volatile TimerStruct_t *timer1 = (volatile TimerStruct_t *) ASPEED_TIMER1_VIRTUAL;
        unsigned long ticks1, ticks2, status;
 
	ticks2 = IO_READ(ASPEED_TIMER1_VIRTUAL + ASPEED_TIMER_TICK);	
	do {
		ticks1 = ticks2;
		status = IO_READ(INTC_IPR);
		ticks2 = IO_READ(ASPEED_TIMER1_VIRTUAL + ASPEED_TIMER_TICK);
	} while (ticks2 >  ticks1);

	/*
	 * Number of ticks since last interrupt.
	 */
	ticks1 = TIMER_RELOAD - ticks2;

	/*
	 * Interrupt pending?  If so, we've reloaded once already.
	 */
	if ((status & IRQMASK_TIMERINT1)){ //+grant
		ticks1 += TIMER_RELOAD;
	        //timer1->TimerControl = (timer1->TimerControl & 0x1f7); //+grant clear timer intr
         }
         
	/*
	 * Convert the ticks to usecs
	 */
	return TICKS2USECS(ticks1);
	 
}


/*
 * IRQ handler for the timer
 */
static irqreturn_t 
ast2000_timer_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	volatile TimerStruct_t *timer1 = (volatile TimerStruct_t *) ASPEED_TIMER1_VIRTUAL;
	 
	write_seqlock(&xtime_lock);

        timer1->TimerControl   |= 0x08;
 
        timer1->TimerControl   &= 0x1fb;
	timer1->TimerLoad    = TIMER_RELOAD;
	timer1->TimerValue   = TIMER_RELOAD;
	do_timer(regs);

        timer1->TimerControl   &= 0x1f7;
	
	/*
	 * clear the interrupt
	 */
	IRQ_EDGE_CLEAR(IRQ_TIMER1);

	/*
	 * the clock tick routines are only processed on the
	 * primary CPU
	 */
	if (hard_smp_processor_id() == 0) {
		timer_tick(regs);
#ifdef CONFIG_SMP
		smp_send_timer();
#endif
	}

#ifdef CONFIG_SMP
	/*
	 * this is the ARM equivalent of the APIC timer interrupt
	 */
	update_process_times(user_mode(regs));
#endif /* CONFIG_SMP */

	write_sequnlock(&xtime_lock);

	return IRQ_HANDLED;	
}


static struct irqaction ast2000_timer_irq = {
	.name		= "AST2000 Timer Tick",
	.flags		= SA_INTERRUPT | SA_TIMER,
	.handler	= ast2000_timer_interrupt,
};


/*
 * Set up timer interrupt, and return the current time in seconds.
 */
static void ast2000_setup_timer(void)
{

	volatile TimerStruct_t *timer0 = (volatile TimerStruct_t *) ASPEED_TIMER0_VIRTUAL;
	volatile TimerStruct_t *timer1 = (volatile TimerStruct_t *) ASPEED_TIMER1_VIRTUAL;
	volatile TimerStruct_t *timer2 = (volatile TimerStruct_t *) ASPEED_TIMER2_VIRTUAL;

        timer0->TimerControl = 0;
        timer1->TimerControl = 0;
        timer2->TimerControl = 0;
        
	/*
	 * Initialise to a known state (all timers off)
	 */
	timer1->TimerLoad    = TIMER_RELOAD;
	timer1->TimerValue   = TIMER_RELOAD;

	/* 
	 * Make irqs happen for the system timer
	 */
	setup_irq(IRQ_TIMER1, &ast2000_timer_irq);

        timer1->TimerControl =0x182 ;
}

struct sys_timer ast2000_timer = {
	.init		= ast2000_setup_timer,
	.offset		= ast2000_gettimeoffset,
};
