// --------------------------------------------------------------------

#ifndef FTMAC100_H
#define FTMAC100_H

#define CACHE_ABLE    1

#define IP_COUNT                2
#define BUFPOOL_SIZE		0x20000
#define BUFPOOL_OFFSET_RXDESC	0
#define BUFPOOL_OFFSET_RXBUF	(BUFPOOL_OFFSET_RXDESC+0x1000)
#define BUFPOOL_OFFSET_TXDESC	(BUFPOOL_OFFSET_RXBUF+0x10000)
#define BUFPOOL_OFFSET_TXBUF	(BUFPOOL_OFFSET_TXDESC+0x1000)


typedef unsigned char			byte;
typedef unsigned short			word;
typedef unsigned long int 		dword;


// --------------------------------------------------------------------
//		FTMAC100 hardware related defenition
// --------------------------------------------------------------------
#define CONFIG_FTGMAC_DES_FMT_NC1
#ifdef CONFIG_FTGMAC_DES_FMT_NC1
#define ZeroCopy
// Defining _PHY_MARVELL includes code that isn't needed for Broadcom.
#define CONFIG_FTGMAC_PHY_MARVELL
#define HAVE_MULTICAST
#endif

#ifdef CONFIG_CPU_FA52x_DCE
#define DescpCacheable
#endif


//base address and interrupt information
#define NC_GMAC1_BASE		0xE0400000
#define IRQ_GMAC1		6
#define NC_GMAC2_BASE		0xE0500000
#define IRQ_GMAC2		5

// --------------------------------------------------------------------
//		ISR_REG  IMR_REG 
// --------------------------------------------------------------------
#define HPTXBUF_UNAVA_BIT	(1UL<<10)
#define PHYSTS_CHG_BIT		(1UL<<9)
#define AHB_ERR_BIT			(1UL<<8)
#define TPKT_LOST_BIT		(1UL<<7)
#define NPTXBUF_UNAVA_BIT	(1UL<<6)
#define TPKT2F_BIT			(1UL<<5)
#define TPKT2E_BIT			(1UL<<4)
#define RPKT_LOST_BIT		(1UL<<3)
#define RXBUF_UNAVA_BIT		(1UL<<2)
#define RPKT2F_BIT			(1UL<<1)
#define RPKT2B_BIT			(1UL<<0)


// --------------------------------------------------------------------
//	APTC_REG 
// --------------------------------------------------------------------

#ifndef CONFIG_CPU_BIG_ENDIAN 

typedef struct
{
	u32 RXPOLL_CNT:4;
	u32 RXPOLL_TIME_SEL:1;
	u32 Reserved1:3;
	u32 TXPOLL_CNT:4;
	u32 TXPOLL_TIME_SEL:1;
	u32 Reserved2:19;
}FTGMAC100_APTCR_Status;

#else

typedef struct
{
	u32 Reserved2:19;
	u32 TXPOLL_TIME_SEL:1;
	u32 TXPOLL_CNT:4;
	u32 Reserved1:3;
	u32 RXPOLL_TIME_SEL:1;
	u32 RXPOLL_CNT:4;
}FTGMAC100_APTCR_Status;

#endif

// --------------------------------------------------------------------
//		PHYCR_REG
// --------------------------------------------------------------------
#define PHY_RE_AUTO_BIT			(1UL<<9)
#define PHY_READ_BIT			(1UL<<26)
#define PHY_WRITE_BIT			(1UL<<27)
// --------------------------------------------------------------------
//		PHYCR_REG 
// --------------------------------------------------------------------
#define PHY_AUTO_OK_BIT			(1UL<<5)
#define PHY_LINK_BIT  			(1UL<<2)
// --------------------------------------------------------------------
//		PHY INT_STAT_REG 
// --------------------------------------------------------------------
#define PHY_SPEED_CHG_BIT		(1UL<<14)
#define PHY_DUPLEX_CHG_BIT		(1UL<<13)
#define PHY_LINK_CHG_BIT		(1UL<<10)
#define PHY_AUTO_COMP_BIT		(1UL<<11)
// --------------------------------------------------------------------
//		PHY SPE_STAT_REG 
// --------------------------------------------------------------------
#define PHY_RESOLVED_BIT		(1UL<<11)
#define PHY_SPEED_mask			0xC000
#define PHY_SPEED_10M			0x0
#define PHY_SPEED_100M			0x1
#define PHY_SPEED_1G			0x2
#define PHY_DUPLEX_mask                 0x2000
//#define PHY_FULLDUPLEX                  0x1
#define PHY_SPEED_DUPLEX_MASK           0x01E0
#define PHY_100M_DUPLEX                 0x0100
#define PHY_100M_HALF                   0x0080
#define PHY_10M_DUPLEX                  0x0040
#define PHY_10M_HALF                    0x0020
#define LINK_STATUS                     0x04
// PHY Register Description
#define PHY_CNTL_REG      0x00
#define PHY_STATUS_REG    0x01
#define PHY_ID1_REG       0x02
#define PHY_ID2_REG       0x03
#define PHY_ANA_REG       0x04
#define PHY_ANLPA_REG     0x05
#define PHY_ANE_REG       0x06
// PHY Status Register
#define LINK_STATUS_VALID  (1 << 2)
#define PHY_REMOTE_FAULT   (1 << 4)
#define PHY_AN_COMPLETE    (1 << 5)

// --------------------------------------------------------------------
//		MACCR_REG
// --------------------------------------------------------------------

#define SW_RST_BIT			(1UL<<31)				// software reset/
#define DIRPATH_BIT			(1UL<<21)
#define RX_IPCS_FAIL_BIT	(1UL<<20)	//
#define SPEED_100_BIT   	        (1UL<<19)	//
#define RX_UDPCS_FAIL_BIT	(1UL<<18)	//
#define RX_BROADPKT_BIT		(1UL<<17)				// Receiving broadcast packet
#define RX_MULTIPKT_BIT		(1UL<<16)				// receiving multicast packet
#define RX_HT_EN_BIT		(1UL<<15)
#define RX_ALLADR_BIT		(1UL<<14)				// not check incoming packet's destination address
#define JUMBO_LF_BIT		(1UL<<13)	//
#define RX_RUNT_BIT			(1UL<<12)				// Store incoming packet even its length is les than 64 byte
#define CRC_CHK_BIT			(1UL<<11)	//
#define CRC_APD_BIT			(1UL<<10)				// append crc to transmit packet
#define GMAC_MODE_BIT		(1UL<<9)	//
#define FULLDUP_BIT			(1UL<<8)				// full duplex
#define ENRX_IN_HALFTX_BIT	(1UL<<7)	//
#define LOOP_EN_BIT			(1UL<<6)				// Internal loop-back
#define HPTXR_EN_BIT		(1UL<<5)	//
#define REMOVE_VLAN_BIT		(1UL<<4)	//
//#define MDC_SEL_BIT		(1UL<<13)				// set MDC as TX_CK/10
//#define RX_FTL_BIT		(1UL<<11)				// Store incoming packet even its length is great than 1518 byte
#define RXMAC_EN_BIT		(1UL<<3)				// receiver enable
#define TXMAC_EN_BIT		(1UL<<2)				// transmitter enable
#define RXDMA_EN_BIT		(1UL<<1)				// enable DMA receiving channel
#define TXDMA_EN_BIT		(1UL<<0)				// enable DMA transmitting channel


// --------------------------------------------------------------------
//		SCU_REG
// --------------------------------------------------------------------
//#define  SCU_PROTECT_KEY_REG            0x0
#define  SCU_PROT_KEY_MAGIC             0x1688a8a8
#define  SCU_RESET_MAC1                 (1u << 11)
#define  SCU_RESET_MAC2                 (1u << 12)


#define  SCU_HT_MAC_INTF_LSBIT          6
#define  SCU_HT_MAC_INTERFACE           (0x7u << SCU_HT_MAC_INTF_LSBIT)
#define  MAC_INTF_SINGLE_PORT_MODES     (1u<<0/*GMII*/ | 1u<<3/*MII_ONLY*/ | 1u<<4/*RMII_ONLY*/)
#define  SCU_HT_MAC_GMII                0x0u

// MII and MII mode
#define  SCU_HT_MAC_MII_MII             0x1u
#define  SCU_HT_MAC_MII_ONLY            0x3u
#define  SCU_HT_MAC_RMII_ONLY           0x4u

#define  SCU_MULTIFUNCTION_PIN_REG      0x74
#define  SCU_MFP_MAC2_PHYLINK           (1u << 26)
#define  SCU_MFP_MAC1_PHYLINK           (1u << 25)
#define  SCU_MFP_MAC2_MII_INTF          (1u << 21)
#define  SCU_MFP_MAC2_MDC_MDIO          (1u << 20)

#define  SCU_SCRATCH_REG	        0x40

// --------------------------------------------------------------------
//		Receive Ring descriptor structure
// --------------------------------------------------------------------


#ifdef CONFIG_CPU_BIG_ENDIAN 
typedef struct
{
	// RXDES0
	u32 RXPKT_RDY:1;		// 1 ==> owned by FTMAC100, 0 ==> owned by software
	u32 EDORR:1;
	u32 FRS:1;
	u32 LRS:1;
	u32 Reserved2:2;
	u32 PAUSE_FRAME:1;
	u32 PAUSE_OPCODE:1;
	u32 FIFO_FULL:1;
	u32 RX_ODD_NB:1;
	u32 RUNT:1;
	u32 FTL:1;
	u32 CRC_ERR:1;
	u32 RX_ERR:1;
	u32 BROARDCAST:1;
	u32 MULTICAST:1;
	u32 Reserved3:1;
	u32 Reserved1:1;   
	u32 VDBC:14;
	
	
	// RXDES1
	u32 Reserved5:4;
	u32 IPCS_FAIL:1;
	u32 UDPCS_FAIL:1;
	u32 TCPCS_FAIL:1;
	u32 VLAN_AVA:1;	
	u32 DF:1;
	u32 LLC_PKT:1;
	u32 PROTL_TYPE:2;
	u32 Reserved4:4;
	u32 VLAN_TAGC:16;
	
	
	// RXDES2
	u32 Reserved6:32;
	
	// RXDES3
	u32 RXBUF_BADR;
				
	u32 VIR_RXBUF_BADR;			// not defined, the virtual address of receive buffer is placed here
	
	u32 RESERVED; 
	u32 RESERVED1; 
	u32 RESERVED2; 
}RX_DESC;

#else // little endian

#ifdef CONFIG_FTGMAC_DES_FMT_NC1
typedef struct
{
	// RXDES0
	u32 VDBC:14;//0~10
	u32 Reserved1:1;          //11~15
	u32 Reserved3:1;
	u32 MULTICAST:1;          //16
	u32 BROARDCAST:1;         //17
	u32 RX_ERR:1;             //18
	u32 CRC_ERR:1;            //19
	u32 FTL:1;
	u32 RUNT:1;
	u32 RX_ODD_NB:1;
	u32 FIFO_FULL:1;
	u32 PAUSE_OPCODE:1;
	u32 PAUSE_FRAME:1;
	u32 Reserved2:2;
	u32 LRS:1;
	u32 FRS:1;
	u32 EDORR:1;
	u32 RXPKT_RDY:1;		// 1 ==> owned by FTMAC100, 0 ==> owned by software
	
	// RXDES1
	u32 VLAN_TAGC:16;
	u32 Reserved4:4;
	u32 PROTL_TYPE:2;
	u32 LLC_PKT:1;
	u32 DF:1;
	u32 VLAN_AVA:1;	
	u32 TCPCS_FAIL:1;
	u32 UDPCS_FAIL:1;
	u32 IPCS_FAIL:1;
	u32 Reserved5:4;
	
	// RXDES2
	u32 Reserved6:32;
	
	// RXDES3
	u32 RXBUF_BADR;
				
	u32 VIR_RXBUF_BADR;			// not defined, the virtual address of receive buffer is placed here
	
	u32 RESERVED; 
	u32 RESERVED1; 
	u32 RESERVED2;	
}RX_DESC;

#else //NC WholeChip

typedef struct
{
	// RXDES0
	u32 VDBC:14;//0~10
	u32 Reserved1:2;          //11~15
	u32 MULTICAST:1;          //16
	u32 BROARDCAST:1;         //17
	u32 RX_ERR:1;             //18
	u32 CRC_ERR:1;            //19
	u32 FTL:1;
	u32 RUNT:1;
	u32 RX_ODD_NB:1;
	u32 FIFO_FULL:1;
	u32 PAUSE_OPCODE:1;
	u32 PAUSE_FRAME:1;
	u32 Reserved2:2;
	u32 LRS:1;
	u32 FRS:1;
	u32 Reserved3:1;
	u32 RXPKT_RDY:1;		// 1 ==> owned by FTMAC100, 0 ==> owned by software
	
	
	// RXDES1
	u32 VLAN_TAGC:16;
	u32 VLAN_AVA:1;	
	u32 TCPCS_FAIL:1;
	u32 UDPCS_FAIL:1;
	u32 IPCS_FAIL:1;
	u32 PROTL_TYPE:2;
	u32 LLC_PKT:1;
	u32 Reserved4:9;
	
	// RXDES2
	u32 RXBUF_Size:14;			
	u32 Reserved5:17;
	u32 EDORR:1;
	
	// RXDES3
	u32 RXBUF_BADR;
				
	u32 VIR_RXBUF_BADR;			// not defined, the virtual address of receive buffer is placed here
	
	u32 RESERVED; 
	u32 RESERVED1; 
	u32 RESERVED2; 
}RX_DESC;

#endif // end of #ifdef CONFIG_FTGMAC_DES_FMT_NC1

#endif


#ifdef CONFIG_CPU_BIG_ENDIAN 

typedef struct
{
	// TXDES0
	u32 TXDMA_OWN:1;
	u32 EDOTR:1;
	u32 FTS:1;
	u32 LTS:1;
	u32 Reserved4:8;
	u32 CRC_ERR:1;
	u32 Reserved3:3;
	u32 Reserved2:1;
	u32 Reserved1:1;
	u32 TXBUF_Size:14;
	
	// TXDES1
	u32 TXIC:1;	
	u32 TX2FIC:1;
	u32 Reserved6:7;
	u32 LLC_PKT:1;
	u32 Reserved5:2;
	u32 IPCS_EN:1;
	u32 UDPCS_EN:1;
	u32 TCPCS_EN:1;
	u32 INS_VLAN:1;
	u32 VLAN_TAGC:16;
	
	// TXDES2
	u32 Reserved7:32;

	// TXDES3
	u32 TXBUF_BADR;

	u32 VIR_TXBUF_BADR;			// Reserve, the virtual address of transmit buffer is placed here

	u32 RESERVED;
	u32 RESERVED1; 
	u32 RESERVED2; 
}TX_DESC;

#else //little endian

#ifdef CONFIG_FTGMAC_DES_FMT_NC1

typedef struct
{
	// TXDES0
	u32 TXBUF_Size:14;
	u32 Reserved1:1;
	u32 Reserved2:1;
	u32 Reserved3:3;
	u32 CRC_ERR:1;
	u32 Reserved4:8;
	u32 LTS:1;
	u32 FTS:1;
	u32 EDOTR:1;
	u32 TXDMA_OWN:1;
	
	// TXDES1
	u32 VLAN_TAGC:16;
	u32 INS_VLAN:1;
	u32 TCPCS_EN:1;
	u32 UDPCS_EN:1;
	u32 IPCS_EN:1;
	u32 Reserved5:2;
	u32 LLC_PKT:1;
	u32 Reserved6:7;
	u32 TX2FIC:1;
	u32 TXIC:1;	
	
	// TXDES2
	u32 Reserved7:32;

	// TXDES3
	u32 TXBUF_BADR;

	u32 VIR_TXBUF_BADR;			// Reserve, the virtual address of transmit buffer is placed here

	u32 RESERVED;
	u32 RESERVED1; 
	u32 RESERVED2; 

}TX_DESC;

#else //NC WholeChip

typedef struct
{
	// TXDES0
	u32 TXBUF_Size:14;
	u32 Reserved1:14;
	u32 LTS:1;
	u32 FTS:1;
	u32 Reserved2:1;
	u32 TXDMA_OWN:1;
	
	// TXDES1
	u32 VLAN_TAGC:16;
	u32 INS_VLAN:1;
	u32 TCPCS_EN:1;
	u32 UDPCS_EN:1;
	u32 IPCS_EN:1;
	u32 LLC_PKT:1;
	u32 Reserved3:11;
	
	// TXDES2
	u32 Reserved4:29;
	u32 TX2FIC:1;
	u32 TXIC:1;
	u32 EDOTR:1;

	// TXDES3
	u32 TXBUF_BADR;

	u32 VIR_TXBUF_BADR;			// Reserve, the virtual address of transmit buffer is placed here

	u32 RESERVED;
	u32 RESERVED1; 
	u32 RESERVED2; 

}TX_DESC;

#endif //#ifdef CONFIG_FTGMAC_DES_FMT_NC1

#endif
#define MAX_PHYADDR		    0x1f
#define INVALID_PHYADDR		0xff

// waiting to do:
#define	TXPOLL_CNT			8
#define RXPOLL_CNT			0

#define TX_OWNBY_SOFTWARE		0
#define TX_OWNBY_FTGMAC100		1


#define RX_OWNBY_SOFTWARE		1
#define RX_OWNBY_FTGMAC100		0

// --------------------------------------------------------------------
//		driver related definition
// --------------------------------------------------------------------


//#define RXDES_NUM			64//64 // we defined 32 descriptor for OTG issue 
#define RXDES_NUM			32

//#define RX_BUF_SIZE			512					//  receive buffer  512 byte
#define RX_BUF_SIZE			1536

//#define TXDES_NUM			64//64 //define 32 for OTG working
//#define TX_BUF_SIZE			2048

#define TXDES_NUM			32
#define TX_BUF_SIZE			1536

#define PHYID_VENDOR_MASK		0xfffffc00
#define PHYID_VENDOR_MARVELL		0x01410c00
#define PHYID_VENDOR_BROADCOM		0x00406000

#define MAXIMUM_MAC_INTF    8
#define MAC_INTF_NONE       0x00
#define MAC_INTF_RMII_PHY   0x01
#define MAC_INTF_NCSI       0x02
#define MAC_INTF_MII_PHY    0x10
#define MAC_INTF_GMII_PHY   0x20

#define MAC_INTF_IS_PHY   (MAC_INTF_RMII_PHY | MAC_INTF_MII_PHY | MAC_INTF_GMII_PHY)
#define MAC_INTF_IS_VAILD   (MAC_INTF_RMII_PHY | MAC_INTF_NCSI | MAC_INTF_MII_PHY |MAC_INTF_GMII_PHY)

#define MAC0    0
#define MAC1    1

#define MAC_ACT_STATUS_OFFSET  7
#define MAC_ACTIVATE    1
#define MAC_DEACTIVATE  0


/* store this information for the driver.. */

struct AstMacHwConfig {
    unsigned int  miiPhyId;

    // Values 0..31 are the PHY chip address on the MDC+MDIO bus driven by this
    // MAC.  Values 32..63 mean this MAC's PHY is on the _MAC1_ MDC+MDIO bus.
    // Value 255 prevents this MAC from accessing PHY chip; assume 100 Mbit/s FDX.
    unsigned char phyAddr;	
    unsigned char isRevA0;
    unsigned char isRevA2;
    unsigned char pad[1];
};

struct ftgmac100_local {

    // these are things that the kernel wants me to keep, so users
    // can find out semi-useless statistics of how well the card is
    // performing
    struct net_device_stats stats;

    struct AstMacHwConfig ids;

    // Set to true during the auto-negotiation sequence
    int	autoneg_active;

    // Last contents of PHY Register 18
    u32		lastPhy18;

    spinlock_t lock;

    struct sk_buff *rx_skbuff[RXDES_NUM];

    volatile RX_DESC *rx_descs;			// receive ring base address
    u32		rx_descs_dma;				// receive ring physical base address
    char	*rx_buf;					// receive buffer cpu address
    int		rx_buf_dma;					// receive buffer physical address
    int		rx_idx;						// receive descriptor

    volatile TX_DESC *tx_descs;
    u32		tx_descs_dma;
    char	*tx_buf;
    int		tx_buf_dma;
    int		tx_idx;
    int		old_tx;
    struct sk_buff *tx_skbuff[TXDES_NUM];
    int     maccr_val;
    struct timer_list    timer;

    u32		GigaBit_MAHT0;
    u32		GigaBit_MAHT1;
    u32		Not_GigaBit_MAHT0;
    u32		Not_GigaBit_MAHT1;
    unsigned int 	InstanceID;
    unsigned int 	Retry;
    unsigned int 	NCSI_Support;
    unsigned char	Payload_Data[16];
    unsigned char	Payload_Pad[4];
    unsigned long	Payload_Checksum;
    unsigned long	tx_free;
    struct mii_if_info mii_if;

    u8 mac_num;
    u8 mac_intf;
    u8 mac_act;
    u8 reserved;
};


#define FTGMAC100_STROBE_TIME			(10*HZ)
///#define FTMAC100_STROBE_TIME			1
 
#endif  /* _SMC_91111_H_ */


