/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __address_h
#define __address_h                     1

/* Base Address */
#define ASPEED_SM_BASE                  0x10000000	/* STATIC MEMORY */
#define ASPEED_SMC_CONTROLLER_BASE      0x16000000	/* SMC CONTROLLER */
#define ASPEED_AHB_CONTROLLER_BASE      0x1E600000	/* AHB CONTROLLER */
#define ASPEED_MAC1_BASE                0x1E660000      /* MAC1 */ 
#define ASPEED_MAC2_BASE                0x1E680000      /* MAC2 */
#define ASPEED_USB20_BASE               0x1E6A0000	/* USB 2.0 */
#define ASPEED_VIC_BASE                 0x1E6C0000	/* VIC */
//#define ASPEED_APB1_BASE                0x1E6E0000	/* APB BRIDGE 1 */
#define ASPEED_MMC_BASE                	0x1E6E0000	/* MMC */
#define ASPEED_USB11_BASE              	0x1E6E1000	/* USB11 */
#define ASPEED_SCU_BASE                	0x1E6E2000	/* SCU */
#define ASPEED_CRYPTO_BASE              0x1E6E3000	/* Crypto */
#define ASPEED_ADPCM_BASE               0x1E6E4000	/* ADPCM */
#define ASPEED_I2S_BASE                	0x1E6E5000	/* I2S */
#define ASPEED_GRAPHIC_BASE             0x1E6E6000	/* Graphics */
#define ASPEED_VIDEO_BASE               0x1E700000	/* VIDEO ENGINE */
#define ASPEED_AHB_TO_PBUS_BASE         0x1E720000	/* APB -> PBUS */
#define ASPEED_MDMA_BASE                0x1E740000	/* MDMA */
#define ASPEED_2D_BASE                  0x1E760000	/* 2D */
//#define ASPEED_APB2_BASE                0x1E780000	/* APB BRIDGE 2 */
#define ASPEED_GPIO_BASE               	0x1E780000	/* MMC */
#define ASPEED_RTC_BASE               	0x1E781000	/* RTC */
#define ASPEED_TIMER_BASE               0x1E782000	/* TIMER */
#define ASPEED_UART0_BASE               0x1E783000	/* UART1 */
#define ASPEED_UART1_BASE               0x1E784000	/* UART2 */
#define ASPEED_WDT_BASE               	0x1E785000	/* WDT */
#define ASPEED_PWM_BASE               	0x1E786000	/* PWM */
#define ASPEED_VUART0_BASE              0x1E787000	/* VUART1 */
#define ASPEED_VUART1_BASE              0x1E788000	/* VUART2 */
#define ASPEED_LPC_BASE               	0x1E789000	/* LPC */
#define ASPEED_I2C_BASE               	0x1E78A000	/* I2C */
#define ASPEED_PECI_BASE               	0x1E78B000	/* PECI */
#define ASPEED_PCIARBITER_BASE          0x1E78C000	/* PCI ARBITER */
#define ASPEED_PCI_MASTER1_BASE         0x60000000	/* PCI MASTER 1 */
#define ASPEED_PCI_MASTER2_BASE         0x80000000	/* PCI MASTER 2 */

#define ASPEED_SDRAM_BASE               0x40000000	/* SDRAM */

#define ASPEED_IO_START                 0x10000000
#define ASPEED_IO_SIZE                  0x0F000000


/*
 * VIC
 */
/* Interrupt Controllers */
#define ASPEED_VIC_STATUS_OFFSET	0x00
#define ASPEED_VIC_RAW_STATUS_OFFSET    0x08
#define ASPEED_VIC_INT_SEL_OFFSET	0x0C
#define ASPEED_VIC_ENABLE_SET_OFFSET    0x10
#define ASPEED_VIC_ENABLE_CLEAR_OFFSET  0x14
#define ASPEED_VIC_SENSE_OFFSET         0x24
#define ASPEED_VIC_BOTH_EDGE_OFFSET	0x28
#define ASPEED_VIC_EVENT_OFFSET         0x2C
#define ASPEED_VIC_EDGE_CLEAR_OFFSET    0x38

#define ASPEED_VIC_STATUS               (ASPEED_VIC_BASE+ASPEED_VIC_STATUS_OFFSET)
#define ASPEED_VIC_RAW_STATUS           (ASPEED_VIC_BASE+ASPEED_VIC_RAW_STATUS_OFFSET)
#define ASPEED_VIC_ENABLE_SET           (ASPEED_VIC_BASE+ASPEED_VIC_ENABLE_SET_OFFSET)
#define ASPEED_VIC_ENABLE_CLEAR         (ASPEED_VIC_BASE+ASPEED_VIC_ENABLE_CLEAR_OFFSET)
#define ASPEED_VIC_SENSE                (ASPEED_VIC_BASE+ASPEED_VIC_SENSE_OFFSET)
#define ASPEED_VIC_BOTH_EDGE		(ASPEED_VIC_BASE+ASPEED_VIC_BOTH_EDGE_OFFSET)
#define ASPEED_VIC_EVENT                (ASPEED_VIC_BASE+ASPEED_VIC_EVENT_OFFSET)
#define ASPEED_VIC_EDGE_CLEAR           (ASPEED_VIC_BASE+ASPEED_VIC_EDGE_CLEAR_OFFSET)

/* Interrupt Controllers 2 */
#define ASPEED_VIC2_STATUS_OFFSET	0x38
#define ASPEED_VIC2_RAW_STATUS_OFFSET   0x38		/* no use */
#define ASPEED_VIC2_INT_SEL_OFFSET	0x0C		/* no use */
#define ASPEED_VIC2_ENABLE_SET_OFFSET   0x28
#define ASPEED_VIC2_ENABLE_CLEAR_OFFSET 0x28
#define ASPEED_VIC2_SENSE_OFFSET        0x30
#define ASPEED_VIC2_BOTH_EDGE_OFFSET	0x34
#define ASPEED_VIC2_EVENT_OFFSET        0x2C
#define ASPEED_VIC2_EDGE_CLEAR_OFFSET   0x38

#define ASPEED_VIC2_STATUS              (ASPEED_GPIO_BASE+ASPEED_VIC2_STATUS_OFFSET)
#define ASPEED_VIC2_RAW_STATUS          (ASPEED_GPIO_BASE+ASPEED_VIC2_RAW_STATUS_OFFSET)
#define ASPEED_VIC2_ENABLE_SET          (ASPEED_GPIO_BASE+ASPEED_VIC2_ENABLE_SET_OFFSET)
#define ASPEED_VIC2_ENABLE_CLEAR        (ASPEED_GPIO_BASE+ASPEED_VIC2_ENABLE_CLEAR_OFFSET)
#define ASPEED_VIC2_SENSE               (ASPEED_GPIO_BASE+ASPEED_VIC2_SENSE_OFFSET)
#define ASPEED_VIC2_BOTH_EDGE		(ASPEED_GPIO_BASE+ASPEED_VIC2_BOTH_EDGE_OFFSET)
#define ASPEED_VIC2_EVENT               (ASPEED_GPIO_BASE+ASPEED_VIC2_EVENT_OFFSET)
#define ASPEED_VIC2_EDGE_CLEAR          (ASPEED_GPIO_BASE+ASPEED_VIC2_EDGE_CLEAR_OFFSET)

/* Interrupt numbers and bit positions */
/* on VIC1 */
#define INT_ECC				0
#define INTMASK_ECC			(1 << INT_ECC)

#define INT_MIC				1
#define INTMASK_MIC			(1 << INT_MIC)

#define INT_MAC1           		2
#define INTMASK_MAC1       		(1 << INT_MAC1)

#define INT_MAC2           		3
#define INTMASK_MAC2       		(1 << INT_MAC2)

#define INT_HAC           		4
#define INTMASK_HAC       		(1 << INT_HAC)

#define INT_USB20           		5
#define INTMASK_USB20       		(1 << INT_USB20)

#define INT_MDMA           		6
#define INTMASK_MDMA       		(1 << INT_MDMA)

#define INT_VIDEO           		7
#define INTMASK_VIDEO       		(1 << INT_VIDEO)

#define INT_LPC				8
#define INTMASK_LPC			(1 << INT_LPC)

#define INT_UARTINT0            	9
#define INTMASK_UARTINT0        	(1 << INT_UARTINT0)

#define INT_UARTINT1            	10
#define INTMASK_UARTINT1        	(1 << INT_UARTINT1)

#define INT_I2S           	        11
#define INTMASK_I2S       		(1 << INT_I2S)

#define INT_I2C				12
#define INTMASK_I2C			(1 << INT_I2C)

#define INT_USB11           		13
#define INTMASK_USB11       		(1 << INT_USB11)

#define INT_ADPCM           		14
#define INTMASK_ADPDM       		(1 << INT_ADPCM)

#define INT_PECI           		15
#define INTMASK_PECI       		(1 << INT_PECI)

#define INT_TIMERINT0           	16
#define INTMASK_TIMERINT0       	(1 << INT_TIMERINT0)

#define INT_TIMERINT1           	17
#define INTMASK_TIMERINT1       	(1 << INT_TIMERINT1)

#define INT_TIMERINT2           	18
#define INTMASK_TIMERINT2       	(1 << INT_TIMERINT2)

#define INT_FLASH		      	19
#define INTMASK_FLASH       		(1 << INT_FLASH)

#define INT_GPIO           		20
#define INTMASK_GPIO       		(1 << INT_GPIO)

#define INT_SCU          		21
#define INTMASK_SCU      		(1 << INT_SCU)

#define INT_RTC_SEC			22
#define INTMASK_RTC_SEC			(1 << INT_RTC_SEC)

#define INT_RTC_DAY			23
#define INTMASK_RTC_DAY			(1 << INT_RTC_DAY)

#define INT_RTC_HOUR			24
#define INTMASK_RTC_HOUR		(1 << INT_RTC_HOUR)

#define INT_RTC_MIN			25
#define INTMASK_RTC_MIN			(1 << INT_RTC_MIN)

#define INT_RTC_ALARM			26
#define INTMASK_RTC_ALARM		(1 << INT_RTC_ALARM)

#define INT_WDT           		27
#define INTMASK_WDT        		(1 << INT_WDT)

#define INT_TACHO           		28
#define INTMASK_TACHO       		(1 << INT_TACHO)

#define INT_2D           		29
#define INTMASK_2D       		(1 << INT_2D)

#define INT_PCI           		30
#define INTMASK_PCI       		(1 << INT_PCI)

#define INT_AHBC			31
#define INTMASK_AHBC			(1 << INT_AHBC)

/* Mask of valid system controller interrupts */
/* VIC means VIC1 */
#define ASPEED_VIC_VALID_INTMASK	( \
					 INTMASK_ECC | \
                                         INTMASK_MIC | \
                                         (INTMASK_MAC1 | INTMASK_MAC2) | \
                                         INTMASK_HAC | \
                                         INTMASK_USB20 | \
                                         INTMASK_MDMA | \
                                         INTMASK_VIDEO | \
                                         INTMASK_LPC | \
                                         (INTMASK_UARTINT0 | INTMASK_UARTINT1) | \
                                         INTMASK_I2S | \
                                         INTMASK_I2C | \
                                         INTMASK_USB11 | \
                                         INTMASK_ADPDM | \
                                         INTMASK_PECI | \
                                         (INTMASK_TIMERINT0 | INTMASK_TIMERINT1 | INTMASK_TIMERINT2) | \
                                         INTMASK_FLASH | \
                                         INTMASK_GPIO | \
                                         INTMASK_SCU | \
					 (INTMASK_RTC_SEC | INTMASK_RTC_DAY | INTMASK_RTC_HOUR | INTMASK_RTC_MIN | INTMASK_RTC_ALARM) | \
                                         INTMASK_WDT | \
                                         INTMASK_TACHO | \
                                         INTMASK_2D | \
                                         INTMASK_PCI | \
                                         INTMASK_AHBC \
					)

#define MAXIRQNUM                       31
#define MAXFIQNUM                       29

/*
 * Timer
 */
#define ASPEED_TIMER0_OFFSET		0x0000		/* Timer0 Offset */
#define ASPEED_TIMER1_OFFSET		0x0010		/* Timer1 Offset */
#define ASPEED_TIMER2_OFFSET		0x0020		/* Timer2 Offset */
#define ASPEED_TIMERRC_OFFSET		0x0030		/* Timer RC Offset */

#define ASPEED_TIMER0_VA_BASE		(IO_ADDRESS(ASPEED_TIMER_BASE)+ASPEED_TIMER0_OFFSET)
#define ASPEED_TIMER1_VA_BASE		(IO_ADDRESS(ASPEED_TIMER_BASE)+ASPEED_TIMER1_OFFSET)
#define ASPEED_TIMER2_VA_BASE		(IO_ADDRESS(ASPEED_TIMER_BASE)+ASPEED_TIMER2_OFFSET)
#define ASPEED_TIMERC_VA_BASE		(IO_ADDRESS(ASPEED_TIMER_BASE)+ASPEED_TIMERRC_OFFSET)

#define ASPEED_TIMER_CLKRATE		(ASPEED_EXTCLK)

/*
 * AVOCENT - AndyC begin
 * modifiedd for kernel upgrade from 2.6.18.6 to 2.6.23.1
 *
 */
#define ASPEED_EXTCLK			(1*1000*1000)	/* 1M */
//#define ASPEED_EXTCLK			(1*1024*1024)	/* 1M */
/* AVOCENT - AndyC end */
#define ASPEED_CRYPTO_VA_BASE       IO_ADDRESS(ASPEED_CRYPTO_BASE)
/*
 * UART
 */
#define ASPEED_UART0_VIRTUAL		(IO_ADDRESS(ASPEED_UART0_BASE))
#define ASPEED_UART1_VIRTUAL		(IO_ADDRESS(ASPEED_UART1_BASE))

/*
 * Ticks
 */
//#define TICKS_PER_uSEC                  40 // IP Cam
//#define TICKS_PER_uSEC                  24 // FPGA
#define TICKS_PER_uSEC                  1		/* ASPEED_EXTCLK / 10 ^ 6 */

#define mSEC_1                          1000
#define mSEC_5                          (mSEC_1 * 5)
#define mSEC_10                         (mSEC_1 * 10)
#define mSEC_25                         (mSEC_1 * 25)
#define SEC_1                           (mSEC_1 * 1000)

/*
 * RTC
 */
#define ASPEED_RTC_VA_BASE		(IO_ADDRESS(ASPEED_RTC_BASE))
 
/*
 * Watchdog
 */
#define ASPEED_WDT_VA_BASE		(IO_ADDRESS(ASPEED_WDT_BASE))

#endif
