/**
 * \n<em>
 *               Avocent Corporation. Proprietary Information.
 *      This software is supplied under the terms of a license agreement or
 *      nondisclosure agreement with Avocent Corporation, or one of its affiliates, and
 *      may not be used, copied, disseminated, or distributed except in accordance
 *      with the terms of that agreement.
 *
 *      3541 Gateway Boulevard, Fremont, California, 94538 U.S.A.
 *                  US Phone: 510-771-6100
 *      Copyright 2001-2009 Avocent Corporation. All rights reserved.
 *</em> </center>
 *----------------------------------------------------------------------------\n
 *  MODULES     aess pwm driver\n
 *----------------------------------------------------------------------------\n
 * @file    aess_pwmdrv.c 
 * @brief   fantach sensor driver
 *----------------------------------------------------------------------------
 */
/******************************************************************************
 * Content
 * ----------------
 *    aess_pwm_config_init() -	pwm ioctl config initializtion function
 *    aess_pwm_config_set() -   pwm ioctl config set function
 *    aess_pwm_open() -		pwm driver open node function
 *    aess_pwm_release() -	pwm driver realase function
 *    aess_pwm_ioctl() -	pwm driver ioctl function
 *    aess_pwm_init() -		pwm init function (called when insert module)
 *    aess_pwm_exit() -		pwm exit function (called when remove module)
*******************************************************************************/

#define AESSPWMDRV_C

#include <linux/module.h>
#include <linux/fs.h>
#include <linux/cdev.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <linux/delay.h>
//#include <asm/arch/map.h>

#include "aess_pwmdrv.h"
#include <linux/aess_debugfs.h>

#define PWM_DEBUG
#undef  PWM_DEBUG

/* Debugging Message */
#ifdef PWM_DEBUG
#define DEBUG_MSG(fmt, args...) printk( "PWM: " fmt, ## args)
#else
#define DEBUG_MSG(fmt, args...)
#endif

#define PWM_DEBUG
#define PWM_DEBUG2
//Debug Message
static u8   u8tmpcounter = 0;
static u16 u16tmpcounter = 0;
static u32 u32tmpcounter = 0;

/* driver name will passed by insmod command */
static char *driver_name = "aess_pwmdrv"; 

/* The init flag for aess_pwm_open */
static UINT8 S_u8init_flag = AESSPWM_NOT_INIT;
static UINT8 S_u8Reginit_flag = AESSPWM_NOT_INIT;

#define ASPEED_A0_WORKAROUND
#ifdef ASPEED_A0_WORKAROUND
static UINT8 S_u8DutyCycle2Value = 0;
#endif

/* Linux device data structure */
dev_t pwm_dev;
struct cdev *pwm_cdev;

/******************************************************************************
 *   FUNCTION        :   aess_pwm_config_init
 ******************************************************************************/
/** @brief 	pass configuration to driver to do initialization
 * @return	STATUS_OK when success, otherwise when fail
 * @dependency     none
 * @limitation     none
 * @warning        none
 * @note           none
 * @internal       Function type: Local function \n 
 ******************************************************************************/
static int aess_pwm_config_init(
				/** pwm structure */
				sPWMDevConfig *PWMDevConfig
				)
{
	UINT8  u8PWMChannel = PWMDevConfig->u8PWMChannelNum;
	UINT32 u32TmpBuf = 0, u32TmpBuf2 = 0;
	UINT32 u32DestAddr = 0;  
	sPWMRegTable *PWMRegMapTemp;	
	int i;
#ifdef ASPEED_A0_WORKAROUND
	int a0_wa_flag = 0;
#endif

	if (u8PWMChannel >= PWM_MAX_CHN_NUM)
		return (-ENODEV);
		
  /****************************************************************
	* Init depend on chipset
	****************************************************************/
	//set pwm clock division is 32768 * 4 and 32768 * 16
	if (S_u8Reginit_flag == AESSPWM_NOT_INIT)
	{
		//SCU disable protection
		iowrite32(0x1688A8A8, (void *) SCU_PKR);
		// disable PWM reset function
		u32TmpBuf = ioread32((void *) SCU_RESET_CONTROL_ADDR);
		u32TmpBuf |= (1 << 9);
		iowrite32(u32TmpBuf, (void *) SCU_RESET_CONTROL_ADDR);
		u32TmpBuf &= !(1 << 9);
		iowrite32(u32TmpBuf, (void *) SCU_RESET_CONTROL_ADDR);
		// enable PWM function
		u32TmpBuf = ioread32((void *) SCU_MULTI_FUNTION1_ADDR);
		u32TmpBuf |= (0xf << 8);
		iowrite32(u32TmpBuf, (void *) SCU_MULTI_FUNTION1_ADDR);
		//SCU enable protection
		iowrite32(0x0, (void *) SCU_PKR);
	
		//set PWM clock frequency for type N and type M
		u32TmpBuf = 0xff << 24; //type N period = 255
		u32TmpBuf |= (0x0 << 20); // type N division H bit
		u32TmpBuf |= (3 << 16); //type N division L bit = 6
		u32TmpBuf |= (0xff << 8); //type M period = 255
		u32TmpBuf |= (0x0 << 4); // type M division H bit
		u32TmpBuf |= (3 << 0); //type M division L bit = 6
		iowrite32(u32TmpBuf, (void *) PWM_REG_CLOCK_CONTROL_ADDR);

		//set pwm clock source
		//fan1 use pwm1 (pwmA) and fan2 use pwm3 (pwmC) on evb
		u32TmpBuf = ioread32((void *) PWM_REG_GEN_CONTROL_ADDR);

		// set pwmA use type M and pwmC use type N
		u32TmpBuf &= (CLK_SRC_TYPE_N << PWMA_TYPE);
		u32TmpBuf |= (CLK_SRC_TYPE_M << PWMC_TYPE);
		iowrite32(u32TmpBuf, (void *) PWM_REG_GEN_CONTROL_ADDR);	

	
		// duty period start from zero
		iowrite32(0x0, (void *) PWM_REG_DUTY_CTRL0_ADDR);
		iowrite32(0x0, (void *) PWM_REG_DUTY_CTRL1_ADDR);
	
		S_u8Reginit_flag = AESSPWM_INIT_OK;
	}
  /**********************************************************************
	* Config PWMD register for setting frequency divider during initialize
	**********************************************************************/
	for (i = 0; i < (PWM_REG_MAX - 1); i ++)
	{
		PWMRegMapTemp = &PWMRegMap[i][u8PWMChannel];
		if (PWMRegMapTemp->enable == 0)
			continue;

		switch(i)
		{
			case PWM_REG_BASE_FREQUENCY:
				u32TmpBuf2 = (UINT32)PWMDevConfig->u8PWMBaseCycleFrequency;
				break;
			case PWM_REG_RESOLUTION:
				u32TmpBuf2 = (UINT32)PWMDevConfig->u8PWMFrequencyDivider;
				break;
			case PWM_REG_DUTY_CYCLE:
				u32TmpBuf2 = (UINT32)PWMDevConfig->u8PWMDutyCycle;
#ifdef ASPEED_A0_WORKAROUND
				if (u8PWMChannel == 2)
					S_u8DutyCycle2Value = (UINT8)(PWMDevConfig->u8PWMDutyCycle & 0xff);
				if (u8PWMChannel == 3)
					a0_wa_flag = 1;
#endif
				break;
			case PWM_REG_OTHER0_SET1:
			case PWM_REG_ENABLE:
			case PWM_REG_OTHER1_SET1:
			case PWM_REG_ENABLE_PIN:
				u32TmpBuf2 = 1;
				break;
			default:
				u32TmpBuf2 = 0;
				break;
		}

		u32DestAddr = (UINT32) (PWMRegMapTemp->address);
		u32TmpBuf = ioread32((void *) u32DestAddr);
		
		if ((PWMRegMapTemp->mask == 0x1) && (PWMRegMapTemp->negative == 1))
		{
			u32TmpBuf2 = !u32TmpBuf2;
		}
		u32TmpBuf2 = (u32TmpBuf2 & PWMRegMapTemp->mask) << PWMRegMapTemp->bitnumber;
		u32TmpBuf &= ~(((UINT32)PWMRegMapTemp->mask) << PWMRegMapTemp->bitnumber);
		u32TmpBuf |= (UINT32)u32TmpBuf2;
#ifdef ASPEED_A0_WORKAROUND
		if (a0_wa_flag == 1)
			u32TmpBuf |= (S_u8DutyCycle2Value << 8);
#endif
		iowrite32(u32TmpBuf, (void *) u32DestAddr); 
	}
	return (STATUS_OK);
}

/******************************************************************************
 *   FUNCTION        :   aess_pwm_config_set
 ******************************************************************************/
/** @brief  	set configuration
 * @return	STATUS_OK when success, otherwise when fail
 * @dependency     none
 * @limitation     none
 * @warning        none
 * @note           none
 * @internal       Function type: Local function \n 
 ******************************************************************************/
static int aess_pwm_config_set(
				/** pwm structure */
				sPWMDevConfig *PWMDevConfig
				)
{
	UINT32 u32TmpBuf = 0, u32TmpBuf2;
	UINT32 u32DestAddr = 0;  
	UINT8  u8tmpDutyCycle = 0;  
	sPWMRegTable *PWMRegMapTemp;	

	if (PWMDevConfig->u8PWMChannelNum >= PWM_MAX_CHN_NUM)
		return (-ENODEV);
		
  /****************************************************************
	* Config PWMC register for setting duty cycle
	****************************************************************/
	PWMRegMapTemp = &PWMRegMap[PWM_REG_DUTY_CYCLE][PWMDevConfig->u8PWMChannelNum];
	u8tmpDutyCycle = PWMDevConfig->u8PWMDutyCycle;
	u32DestAddr = (UINT32) (PWMRegMapTemp->address);

	{       
		/* PWP (Pulse Width Percentage) is the 0-base */
		u32TmpBuf2 = (UINT32)(u8tmpDutyCycle & 0xFF);
	}

	u32TmpBuf = ioread32((void *) u32DestAddr);
		
	u32TmpBuf2 = (u32TmpBuf2 & PWMRegMapTemp->mask) << PWMRegMapTemp->bitnumber;
	u32TmpBuf &= ~(((UINT32)PWMRegMapTemp->mask) << PWMRegMapTemp->bitnumber);
	u32TmpBuf |= (UINT32)u32TmpBuf2;
#ifdef ASPEED_A0_WORKAROUND
		if (PWMDevConfig->u8PWMChannelNum == 3)
			u32TmpBuf |= (S_u8DutyCycle2Value << 8);
#endif
	iowrite32(u32TmpBuf, (void *) u32DestAddr); 
	
	DRV_INFO("PWMCAddr=0x%lx, u8PWMBaseCycleFrequency=%lx, u8PWMDutyCycle=%lx, PWMCTmpBuf=%lx\n",
		u32DestAddr, PWMDevConfig->u8PWMBaseCycleFrequency, PWMDevConfig->u8PWMDutyCycle, u32TmpBuf);


	return (STATUS_OK);
}

/******************************************************************************
 *   FUNCTION        :   aess_pwm_open
 ******************************************************************************/
/** @brief	   function for open node
 * @return	   always STATUS_OK 
 * @dependency     none
 * @limitation     none
 * @warning        none
 * @note           none
 * @internal       Function type: Local function \n 
 ******************************************************************************/
static int aess_pwm_open(
			/** device node */
			struct inode *inode, 
			
			/** file descriptor */
			struct file *filp
			)

{
	if (S_u8init_flag == AESSPWM_NOT_INIT)
	{
		DRV_MSG("start init aess_pwm_driver\n");

		S_u8init_flag = AESSPWM_INIT_OK;
	
		DRV_MSG("finish init aess_pwm_driver!\n");

	}
	return (STATUS_OK);
}

/******************************************************************************
 *   FUNCTION        :   aess_pwm_release
 ******************************************************************************/
/** @brief         release resource
 * @return	   always 0
 * @dependency     none
 * @limitation     none
 * @warning        none
 * @note           none
 * @internal       Function type: Local function \n 
 ******************************************************************************/
static int aess_pwm_release(
			/** device node */
			struct inode *inode, 
			
			/** file descriptor */
			struct file *filp
			)
{
	/* Change the rotational speed of FAN to full speed */
	iowrite32(0x0, (void *) PWM_REG_DUTY_CTRL0_ADDR); 
	iowrite32(0x0, (void *) PWM_REG_DUTY_CTRL1_ADDR); 

	DRV_MSG("aess_pwm_release exit!\n");
	
	return 0;
}

/******************************************************************************
 *   FUNCTION        :   aess_pwm_ioctl
 ******************************************************************************/
/** @brief         ioctl function
 * @return	   STATUS_OK when success, otherwise when fail
 * @dependency     none
 * @limitation     none
 * @warning        none
 * @note           none
 * @internal       Function type: Local function \n 
 ******************************************************************************/
static int aess_pwm_ioctl(
			/** device node */
			struct inode *inode, 
			
			/** file descriptor */
			struct file *filp,
			
			/** ioctl cmannd */
			unsigned int cmd, 
			
			/** ioctl arguments */
			unsigned long arg
			)
{
	int err_check;
	sPWMDevConfig *PWMDevConfig;

	DRV_MSG( "aess_pwm_ioctl\n");
	
	PWMDevConfig = (sPWMDevConfig *) arg;

	/* Check the structure address is OK */
	if(!access_ok(VERIFY_READ, (void *) PWMDevConfig, sizeof(sPWMDevConfig)))
	{
		DRV_ERROR(" struct access error!!\n");
		
		/* error occur */
		return -EFAULT;
	}
	switch(cmd)
	{
		case AESS_PWM_CONFIG_INIT: 
			DRV_MSG( " Start PWM Init config process!!\n");
			DRV_INFO("aess_pwm_ioctl[%d]\n", PWMDevConfig->u8PWMChannelNum);
			
			err_check = aess_pwm_config_init(PWMDevConfig);

			DRV_MSG( "Finish PWM Init config process!!\n");
			break;
			
		case AESS_PWM_CONFIG_SET:
			DRV_MSG( " Start PWM Set config process!!\n");
			DRV_INFO("aess_pwm_ioctl[%d]\n", PWMDevConfig->u8PWMChannelNum);
			
			err_check = aess_pwm_config_set(PWMDevConfig);
			
			DRV_MSG( "Finish PWM Set config process!!\n");
			break;
			
			
		default:
			printk("\n default ?? ..");
			DRV_ERROR("aess_pwm_ioctl, NEWCMD=%lx OLDCMD=0xlx command error\n", cmd, AESS_PWM_CONFIG_SET);
			err_check = -EINVAL;
			break;
	}
	
	/* 0->ok, minus->fail */
	return err_check;
}

static struct file_operations aess_pwm_fops = {
	.open = aess_pwm_open, 
	.ioctl = aess_pwm_ioctl,
	.release = aess_pwm_release,     
};

/******************************************************************************
 *   FUNCTION        :   aess_pwm_init
 ******************************************************************************/
/** @brief         initial function for insert module 
 * @return	   STATUS_OK when success, otherwise when fail
 * @dependency     none
 * @limitation     none
 * @warning        none
 * @note           none
 * @internal       Function type: Global function \n 
 ******************************************************************************/
int __init aess_pwm_init(void)
{
	int result;
	
	DRV_MSG( "init_aess_pwm_module\n");
	
	pwm_cdev = cdev_alloc();
   
	pwm_cdev->ops = &aess_pwm_fops;
	result = alloc_chrdev_region(&pwm_dev, 0, 1, driver_name);

	printk("mknod /dev/aess_pwmdrv c %d 0\n", MAJOR(pwm_dev));


	if (result < 0) {
		DRV_ERROR("Registering the PWM character device failed with %d\n", MAJOR(pwm_dev));
		return result;
	}

	cdev_add(pwm_cdev, pwm_dev, 1);

        aess_debugfs_default_create(driver_name,NULL,0,0);
        aess_debugfs_create_file(driver_name,"u32",(u32 *)&u32tmpcounter,DBG_TYPE32,0);
        aess_debugfs_create_file(driver_name,"u16",(u16 *)&u16tmpcounter,DBG_TYPE16,0);
        aess_debugfs_create_file(driver_name,"u8",(u8 *)&u8tmpcounter,DBG_TYPE8,0);	
	
	return (STATUS_OK);
}

/******************************************************************************
 *   FUNCTION        :   aess_pwm_exit
 ******************************************************************************/
/** @brief         exit function for remove module  
 * @return	   STATUS_OK when success, otherwise when fail
 * @dependency     none
 * @limitation     none
 * @warning        none
 * @note           none
 * @internal       Function type: Local function \n 
 ******************************************************************************/
static void __exit aess_pwm_exit(void)
{
    DRV_MSG("aess_pwmdrv_exit\n");
    aess_debugfs_remove(driver_name,NULL, 0 , 0);
    cdev_del(pwm_cdev);
    unregister_chrdev_region(pwm_dev, 1);
}


MODULE_DESCRIPTION("AESS PWM Driver");
MODULE_AUTHOR("Steven Lai <steven.lai@avocent.com");
MODULE_LICENSE("GPL");
MODULE_VERSION("0.1.0.0");
module_init(aess_pwm_init);
module_exit(aess_pwm_exit);
module_param(driver_name, charp, S_IRUGO);

/* End of code */
