/*
 * $RCSfile: ast2050_fwupgrade.h,v $
 * $Revision: 1.11 $
 * $Date: 2008/06/13 21:04:00 $
 * $Author: jlee $
 *
 * AST2050 firmware upgrade driver.
 *  
 * Copyright (C) 2008 Avocent Corp.
 *
 * This file is subject to the terms and conditions of the GNU 
 * General Public License. This program is distributed in the hope 
 * that it will be useful, but WITHOUT ANY WARRANTY; without even 
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 */


/* 
 image layout
+---------------------+
| image header        | struct image_header + structimage_info * n
+---------------------+
| image 1             | each image offset is recorded in the image header
+---------------------+
| image 2             |
+---------------------+
|       :             |
+---------------------+
| image n             |
+---------------------+
*/
#pragma pack(1)

/* structure of a primary partition, 16 bytes */
typedef struct primary_partition_table
{
    UINT8 status;           /* 0x80 - bootable, active */
    UINT8 start_chs[3];     /* cyliner-head-sector address of the first sector */
    UINT8 type;             /* partition type */
    UINT8 end_chs[3];       /* cyliner-head-sector address of the last sector */
    UINT8 lba[4];           /* logical block address of the first secotr in the partition */
    UINT8 len[4];           /* length of the partition in sectors */
} partition_table_type;


/* structure of a Master Boot Record, MBR, 512 bytes */
typedef struct fwu_mbr
{
    UINT8 code[440];                    /* 0x000 - 0x1B7 */
    UINT8 disk_signature[4];            /* 0x1B8, 0x41 0x56 0x43 0x54 (AVCT) */
    UINT8 reserved[2];                  /* 0x1BC, 0x00 0x00 */
    partition_table_type partition[4];  /* 0x1BE */
    UINT8 mbr_signature[2];             /* 0x1FE, 0x55 0xAA */
} fwu_mbr_type;


/* structure of an Extended Boot Record, EBR, 512 bytes */ 
typedef struct fwu_ebr
{
    UINT8 reserved[446];                /* 0x000, generally unused */
    partition_table_type partition[2];  /* 0x1BE, first and seconed entries */
    partition_table_type res_part[2];   /* 0x1DE */
    UINT8 ebr_signature[2];             /* 0x1FE, 0x55 0xAA */
} fwu_ebr_type;


/* structure of a image information, 12 */
typedef struct fwu_image_info
{
    UINT32 offset;
    UINT32 size;
    UINT32 crc32;
} fwu_image_info_type;


#if 0
/* structure of a image header, 512 bytes */
typedef struct fwu_image_header 
{
    UINT32 crc32;
    UINT16 header_ver;
    UINT16 num_of_image;
    UINT32 version;
    UINT32 img_size;
} fwu_image_header_type;
#endif


/* structure of a image header, 512 bytes */
typedef struct fwu_image_header 
{
    UINT32 crc32;           /* checksum */
    UINT8 header_ver;       /* header version, 1 */
    UINT8 img_type;         /* image type, 1 is for iBMC */
    UINT8 num_of_image;     /* number of image */
    UINT8 reserved_0;
    UINT32 version;         /* use for application */
    UINT32 img_size;        /* total size of image file */
    /* UINT32 reserved_1; */
    /* UINT32 reserved_2; */
    UINT8 uboot_ver[8];     /* 0-2: major 4-6: minor, in BCD */
    /* UINT32 reserved_3; */
    UINT8 platform_id[4];   /* 0: platfrom id, 0xFFFFFFFF -> unspecified */
    UINT32 reserved_4;
} fwu_image_header_type;


#pragma pack()


/* partition number for kernel and rootfs */
#define FWU_KERNEL_1            0       /* partition 1 */
#define FWU_ROOTFS_1            1       /* partition 2 */
#define FWU_KERNEL_2            2       /* partition 3 */
#define FWU_ROOTFS_2            4       /* partition 5 */

/* default partition size for kernel and rootfs */
#define FWU_KERNEL_PART_SIZE    8       /* 8 MB */
#define FWU_ROOTFS_PART_SIZE    56      /* 56 MB */

/* default partition size for kernel and rootfs in blocks, block = 512 bytes */
#define FWU_KERNEL_PART_BLOCKS  (FWU_KERNEL_PART_SIZE * 1024 * 1024 / 512)
#define FWU_ROOTFS_PART_BLOCKS  (FWU_ROOTFS_PART_SIZE * 1024 * 1024 / 512)

/* firmware upgarde device */
#define FWU_DEV_UNKNOWN         0
#define FWU_DEV_EVB             1
#define FWU_DEV_MASER           2
#define FWU_DEV_AMEA            3
#define FWU_DEV_SPI             4
#define FWU_DEV_AMEA_NO_MUX     5
#define FWU_DEV_LAST            5

/* partition type */
#define FWU_PRIMARY_PART        0
#define FWU_EXTEND_PART         1
#define FWU_LOGIC_PART          2

#define FWU_ADD_PART            1
#define FWU_DEL_PART            0

#define FWU_SPI_BOOTLOADER      0
#define FWU_SPI_KERNEL          1
#define FWU_SPI_ROOTFS          2

#define FWU_CRAMFS_MAGIC        0x28cd3d45
#define FWU_CRAMFS_SIGNATURE    "Compressed ROMFS"


typedef struct fwu_cramfs_super {
  unsigned int magic, size, flags, future;
  unsigned char signature[16];
} fwu_cramfs_super_type;


/* structure of firmware upgarde information */
typedef struct fwu_info
{
    /* partition */
    UINT32 part_lba[12];    /* partition logical block address, 3P 1E 8L */
    UINT32 part_size[12];   /* partition size */
    UINT32 ebr_lba[8];      /* EBR's logical block address, max 8 */
    UINT32 size;            /* size of device */
    UINT32 free_space;      /* free space of device */
    UINT8 bootable[12];     /* partition status byte, 0x80=bootable */
    UINT8 type[12];         /* partition type */
    UINT8 num_of_pp;        /* number of primary partition */
    UINT8 num_of_ep;        /* number of extended partition */
    UINT8 num_of_lp;        /* number of logical partition */ 
    UINT8 init;             /* init status */
    int dev;                /* device */
} fwu_info_type;
