/*
 * (C) Copyright 2004
 * Peter Chen <peterc@socle-tech.com.tw>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef __CONFIG_H
#define __CONFIG_H

#define CONFIG_USE_128MB_SDRAM

#define CONFIG_AST2050_PLATFORM_ID "AEVB"

/* should be the same as U_BOOT_VERSION */
#define CONFIG_UBOOT_VERSION            "1"
#define CONFIG_UBOOT_PATCHLEVEL         "1"
#define CONFIG_UBOOT_SUBLEVEL           "4"


#define CONFIG_AVCT_VERSION             "0"
#define CONFIG_AVCT_PATCHLEVEL          "0"
#define CONFIG_AVCT_SUBLEVEL            "2"

#define CONFIG_IDENT_STRING_VERSION     " (" CONFIG_AVCT_VERSION "." \
                                             CONFIG_AVCT_PATCHLEVEL "." \
                                             CONFIG_AVCT_SUBLEVEL ")"

#define CONFIG_IDENT_STRING_PLATFORM    " EVB"

#ifdef CONFIG_USE_128MB_SDRAM
#define CONFIG_IDENT_STRING_MISC        " [AST2050 0.19-128M (VGA: 8MB)]"
#else
#define CONFIG_IDENT_STRING_MISC        " [AST2050 pre.0.19-64M]"
#endif

#define CONFIG_IDENT_STRING     " Avocent" \
                                CONFIG_IDENT_STRING_VERSION \
                                CONFIG_IDENT_STRING_PLATFORM \
                                CONFIG_IDENT_STRING_MISC

/*
 * High Level Configuration Options
 * (easy to change)
 */
//#define CONFIG_INIT_CRITICAL			/* define for U-BOOT 1.1.1 */
#undef  CONFIG_INIT_CRITICAL			/* undef for  U-BOOT 1.1.4 */ 
#define CONFIG_ARM926EJS	1		/* This is an arm926ejs CPU */
#define CONFIG_AST1100		1
//#define CONFIG_AST1100_FPGA
#undef CONFIG_AST1100_FPGA			/* undef if real chip */
#define CONFIG_2SPIFLASH			/* Boot SPI: CS2, 2nd SPI: CS0 */
#undef CONFIG_2SPIFLASH

#define CONFIG_USE_IRQ				/* we don't need IRQ/FIQ stuff */
#define CONFIG_MISC_INIT_R 
#define CONFIG_CMDLINE_EDITING 1 /*support command line history*/

/* 
 * Environment Config
 */
#define CONFIG_CMDLINE_TAG	 1		/* enable passing of ATAGs	*/
#define CONFIG_SETUP_MEMORY_TAGS 1
#define CONFIG_INITRD_TAG	 1
#define CONFIG_BOOTARGS 	"root=/dev/mtdblock3 mem=80M console=ttyS0"
#define CONFIG_BOOTDELAY	1		/* autoboot after 1 seconds	*/
#define CONFIG_BOOTCOMMAND	"fwu boot"
#define CONFIG_BOOTFILE		"all.bin"
#define CONFIG_ENV_OVERWRITE
#define CONFIG_COMMANDS		((CONFIG_CMD_DFL  | CFG_CMD_NET) \
                             | CFG_CMD_ENV                   \
                             | CFG_CMD_FLASH               \
                             | CFG_CMD_DHCP                \
                             | CFG_CMD_PING                 \
                             | CONFIG_CMD_ETHACT            \
                             | CFG_CMD_I2C                  \
                             | CFG_CMD_EEPROM) & ~CFG_CMD_PCMCIA

/* 
 * CPU Setting
 */
#define CPU_CLOCK_RATE		18000000	/* 16.5 MHz clock for the ARM core */

/*
 * Size of malloc() pool
 */
#define CFG_MALLOC_LEN		(CFG_ENV_SIZE + 128*1024)
#define CFG_GBL_DATA_SIZE	128	/* size in bytes reserved for initial data */

/*
 * Stack sizes,  The stack sizes are set up in start.S using the settings below
 */
#define CONFIG_STACKSIZE	(128*1024)	/* regular stack */
#define CONFIG_STACKSIZE_IRQ	(4*1024)	/* IRQ stack */
#define CONFIG_STACKSIZE_FIQ	(4*1024)	/* FIQ stack */

/*
 * Memory Configuration
 */
#define CONFIG_NR_DRAM_BANKS	1	   	/* we have 1 bank of DRAM */
#define PHYS_SDRAM_1		0x40000000 	/* SDRAM Bank #1 */
#ifdef CONFIG_USE_128MB_SDRAM
#define PHYS_SDRAM_1_SIZE	0x8000000 	/* 128 MB */
#else
#define PHYS_SDRAM_1_SIZE	0x4000000 	/* 64 MB */
#endif

#define CFG_SDRAM_BASE		0x40000000

/*
 * FLASH Configuration
 */
#define PHYS_FLASH_1		0x14000000 	/* Flash Bank #1 */
#define PHYS_FLASH_2		0x10000000 	/* Flash Bank #2 */

#ifdef CONFIG_2SPIFLASH
#define CFG_FLASH_BASE          PHYS_FLASH_1
#define CFG_FLASH_BANKS_LIST    { PHYS_FLASH_1, PHYS_FLASH_2 }
#define CFG_MAX_FLASH_BANKS     2
#define CFG_MAX_FLASH_SECT      (256)   /* max number of sectors on one chip */
#else
#define CFG_FLASH_BASE          PHYS_FLASH_1
#define CFG_FLASH_BANKS_LIST    { PHYS_FLASH_1 }
#define CFG_MAX_FLASH_BANKS     1
#define CFG_MAX_FLASH_SECT      (256)   /* max number of sectors on one chip */
#endif

/*-----------------------------------------------------------------------
 * U-boot Environment
 *-----------------------------------------------------------------------*/
#define CFG_ENV_IS_IN_FLASH     1
#define CFG_ENV_SECT_SIZE       0x10000 /* the size of one sector */
#define CFG_ENV_SIZE	        0x02000  /* Total Size of Environment Sector, 8K */
#define CFG_ENV_OFFSET	        0x20000 /* environment starts here  */
#define CFG_ENV_ADDR            (CFG_FLASH_BASE + CFG_ENV_OFFSET) /* addr of environment */

/*-----------------------------------------------------------------------
 * U-boot Persistent Storage
 *-----------------------------------------------------------------------*/
#define CFG_PS_SECT_SIZE        0x10000 /* the size of one sector */
#define CFG_PS_SIZE             0x00100 /* Total Size of PS Sector */
#define CFG_PS_OFFSET           0x24000 /* PS starts here  */
#define CFG_PS_ADDR             (CFG_FLASH_BASE + CFG_PS_OFFSET) /* addr of PS */

#define __LITTLE_ENDIAN  1
#define CFG_FLASH_SPI

#define CFG_MONITOR_BASE	TEXT_BASE
#define CFG_MONITOR_LEN		(192 << 10)

/* timeout values are in ticks */
#define CFG_FLASH_ERASE_TOUT	(20*CFG_HZ) 	/* Timeout for Flash Erase */
#define CFG_FLASH_WRITE_TOUT	(20*CFG_HZ) 	/* Timeout for Flash Write */

/*
 * Miscellaneous configurable options
 */
#define CFG_LONGHELP				/* undef to save memory		*/

#define CFG_PROMPT		"uboot# " 	/* Monitor Command Prompt	*/
#define CFG_CBSIZE		256		/* Console I/O Buffer Size	*/
#define CFG_PBSIZE (CFG_CBSIZE+sizeof(CFG_PROMPT)+16) /* Print Buffer Size */
#define CFG_MAXARGS		16		/* max number of command args	*/
#define CFG_BARGSIZE		CFG_CBSIZE	/* Boot Argument Buffer Size	*/

#define CFG_MEMTEST_START	0x40000000	/* memtest works on	*/
#ifdef CONFIG_USE_128MB_SDRAM
#define CFG_MEMTEST_END		0x47FFFFFF	/* 128 MB in DRAM	*/
#else
#define CFG_MEMTEST_END		0x43FFFFFF	/* 64 MB in DRAM	*/
#endif

#define	CFG_CLKS_IN_HZ      /* everything, incl board info, in Hz */
#undef	CFG_CLKS_IN_HZ      /* everything, incl board info, in Hz */

#define CFG_LOAD_ADDR		0x41400000	/* default load address */

#define CFG_TIMERBASE		0x1E782000	/* use timer 1 */
#define CFG_HZ			(1*1000*1000)	/* use external clk (1M) */

#define CONFIG_CMDLINE_EDITING

/*
 * Serial Configuration
 */
#define CFG_NS16550
#define CFG_NS16550_SERIAL
#define CFG_NS16550_REG_SIZE	4
#define CFG_NS16550_CLK		24000000
#define CFG_NS16550_COM1	0x1e783000
#define CFG_NS16550_COM2	0x1e784000
#define	CFG_LOADS_BAUD_CHANGE
#define CONFIG_SERIAL1		1
#define CONFIG_CONS_INDEX	2
#define CONFIG_BAUDRATE		115200
#define CFG_BAUDRATE_TABLE	{ 9600, 19200, 38400, 57600, 115200 }

/*
 * USB device configuration
 */
/*
#define CONFIG_USB_DEVICE	1
#define CONFIG_USB_TTY		1

#define CONFIG_USBD_VENDORID		0x1234
#define CONFIG_USBD_PRODUCTID		0x5678
#define CONFIG_USBD_MANUFACTURER	"Siemens"
#define CONFIG_USBD_PRODUCT_NAME	"SX1"
*/

/*
 * I2C configuration
 */
#define CONFIG_HARD_I2C
#define CFG_I2C_SPEED		100000
#define CFG_I2C_SLAVE		1
#define CONFIG_DRIVER_ASPEED_I2C

/*
* EEPROM configuration
*/

#define CFG_I2C_EEPROM_ADDR_LEN 2
#define CFG_I2C_EEPROM_ADDR 0xa0


/*
 * NIC configuration
 */
#define __BYTE_ORDER __LITTLE_ENDIAN
#define __LITTLE_ENDIAN_BITFIELD
#define CONFIG_MAC_PARTITION
#define CONFIG_FARADAYNIC
//#define CONFIG_MAC1_PHY_LINK_INTERRUPT
#define CONFIG_MAC2_ENABLE
//#define CONFIG_MAC2_MII_ENABLE
//#define CONFIG_MAC2_PHY_LINK_INTERRUPT	
#define CONFIG_NET_MULTI
#define CONFIG_ETHACT		faradaynic#0
#define CONFIG_GATEWAYIP	192.168.0.1
#define CONFIG_NETMASK		255.255.255.0
#define CONFIG_IPADDR		192.168.0.188
#define CONFIG_SERVERIP		192.168.0.126
#define CONFIG_ETHADDR		00:C0:A8:12:34:56
#define CONFIG_ETH1ADDR		00:C0:A8:12:34:57

//#define CONFIG_AESS_SUPPORT_NCSI		

#define CFG_AU "au=run spi_au\0"
#define CFG_BU "bu=run spi_bu\0"
#define CFG_KU "ku=run spi_ku\0"
#define CFG_RU "ru=run spi_ru\0"
#define CFG_QU "qu=run spi_ku spi_ru\0"

#define CONFIG_EXTRA_ENV_SETTINGS \
    "flash_0_start=14000000\0" \
    "flash_1_start=10000000\0" \
    "env_start=14020000\0" \
    "env_size=10000\0" \
    "uboot_start=14000000\0" \
    "ps_start=14160000\0" \
    "ps_size=80000\0" \
    "kernel_load=41400000\0" \
    "kernel_start=14030000\0" \
    "kernel_size=130000\0" \
    "rootfs_load=41530000\0" \
    "rootfs_start=141E0000\0" \
    "rootfs_part1_start=141E0000\0" \
    "rootfs_part2_start=10000000\0" \
    "rootfs_size=1E20000\0" \
    "fwu_kernel_offset=0\0" \
    "fwu_kernel_load=41400000\0" \
    "fwu_kernel_start=0\0" \
    "fwu_kernel_size=0\0" \
    "fwu_rootfs_offset=0\0" \
    "fwu_rootfs_load=0\0" \
    "fwu_rootfs_start=0\0" \
    "fwu_rootfs_size=0\0" \
    "fwu_uboot_offset=0\0" \
    "fwu_uboot_size=0\0" \
    "erase_eeprom=mw.b 41400000 ff 4000;eeprom write 41400000 0 4000\0" \
    "erase_env=ps erase env\0" \
    "erase_ps=set fl_start $(ps_start);" \
             "set fl_size $(ps_size);" \
             "run erase\0" \
    "erase=protect off $(fl_start) +$(fl_size);" \
          "erase $(fl_start) +$(fl_size);" \
          "protect on $(fl_start) +$(fl_size)\0" \
    CFG_AU \
    CFG_BU \
    CFG_KU \
    CFG_RU \
    CFG_QU \
    "mu=set bootfile mac.bin;" \
       "$(interface) $(offset);" \
       "eeprom write 41400000 0 6\0" \
    "spi_au=set bootfile $(rootpath)all.bin.part1;" \
           "set fl_start $(flash_0_start);" \
           "run uploadfile;" \
           "set filesize 0;" \
           "set bootfile $(rootpath)all.bin.part2;" \
           "set fl_start $(flash_1_start);" \
           "run uploadfile\0" \
    "spi_bu=set bootfile $(rootpath)bootloader.bin;" \
           "set fl_start $(uboot_start);" \
           "run uploadfile\0" \
    "spi_ku=set bootfile $(rootpath)kernel.bin;" \
           "set fl_start $(kernel_start);" \
           "run uploadfile\0" \
    "spi_ru=set bootfile $(rootpath)rootfs.cramfs.part1;" \
           "set fl_start $(rootfs_part1_start);" \
           "run uploadfile;" \
           "set filesize 0;" \
           "set bootfile $(rootpath)rootfs.cramfs.part2;" \
           "set fl_start $(rootfs_part2_start);" \
           "run uploadfile\0" \
    "fwu_au=tftp $(offset) $(rootpath)$(fwuimage);" \
           "fwu update $(offset)\0" \
    "spi_boot=bootm $(kernel_start) $(rootfs_start)\0" \
    "nfs_boot=run nfsargs;" \
             "run load_kernel\0" \
    "spi_kernel=bootm $(kernel_start)\0" \
    "fwu_kernel=run fwu_boot\0" \
    "load_kernel=run spi_kernel\0" \
    "interface=tftp\0" \
    "offset=41400000\0" \
    "uploadfile=$(interface) $(offset);" \
               "protect off $(fl_start) +$(filesize);" \
               "erase $(fl_start) +$(filesize);" \
               "cp.b $(offset) $(fl_start) $(filesize);" \
               "protect on $(fl_start) +$(filesize)\0" \
    "fwudev=spi\0" \
    "fwuimage=firmimg.ast2050\0" \
    "console=ttyS0\0" \
    "mem=104M\0" \
    "netdev=eth0\0" \
    "nfsip=\0" \
    "nfsroot=/nfsroot\0" \
    "initrdargs=set bootargs console=$(console) root=$(rootpath);" \
               "bootm $(kernel_start) $(rootfs_start)\0" \
    "nfsargs=set bootargs " \
                "console=$(console) " \
                "mem=$(mem) " \
                "nfsroot=$(nfsip):$(nfsroot) root=/dev/nfs rw " \
                "ip=$(ipaddr):$(nfsip):$(gatewayip):$(netmask):$(hostname):$(netdev):off\0" \

/* this must be included AFTER the definition of CONFIG_COMMANDS (if any) */
#include <cmd_confdefs.h>

#endif	/* __CONFIG_H */
