/*
 * ASPEED ast2050 USB device controller driver
 *
 * Copyright (C) 2008 Avocent Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef __ASPEED_USB_UDC_H_
#define __ASPEED_USB_UDC_H_
	
#include "ch11.h"

/*
 * Register definitions
 */

/* SCU base */
#define	SCU_BASE				0x1e6e2000

/* protection key register */
#define	SCU_PROTECTION_KEY			(SCU_BASE + 0x0)
#define	PROTECTION_KEY_UNLOCK			0x1688a8a8

/* system reset register */
#define	SCU_RESET_CONTROL			(SCU_BASE + 0x4)
#define	RESET_USB20				0x00004000

/* clock control register */
#define	SCU_CLOCK_CONTROL			(SCU_BASE + 0xc)
#define	ENABLE_USB20_CLOCK			0x00004000

/* USB 2.0 base */
#define	USBCORE_BASE				0x1e6a0000

/* root / global registers */
#define	USBCORE_ROOT_BASE			(USBCORE_BASE + 0x0)

/* root control / status register */
#define	ROOT_CONTROL				(USBCORE_ROOT_BASE + 0x0)
#define	PHYCLOCK				0x80000000
#define	ISOCHRONOUS_IN_NULL_DATA_RESPONSE	0x00020000
#define	COMPLETE_SPLIT_IN_AFTER_SOF		0x00010000
#define	LOOPBACK_RESULT				0x00008000
#define	LOOPBACK_FINISHED			0x00004000
#define	PHY_BIST_RESULT				0x00002000
#define	PHY_BIST_CONTROL			0x00001000
#define	ENABLE_PHY_RESET			0x00000800
#define	REMOTE_WAKEUP_SIGNAL			0x00000020
#define	ENABLE_MANUAL_REMOTE_WAKEUP		0x00000010
#define	ENABLE_AUTO_REMOTE_WAKEUP		0x00000008
#define	ENABLE_CLOCK_STOPPING_SUSPEND		0x00000004
#define	UPSTREAM_SPEED_SELECTION		0x00000002
#define	ENABLE_UPSTREAM_PORT_CONNECTION		0x00000001

/* root configuration setting register */
#define	ROOT_CONF				(USBCORE_ROOT_BASE + 0x4)
#define	DMA_PAGE_BUFFER_STATUS			0xffff0000
#define	ROOT_FUNCTION_DEVICE_ADDRESS		0x0000007f

/* interrupt control register */
#define	INT_CONTROL				(USBCORE_ROOT_BASE + 0x8)
#define	ENABLE_ENDPOINT_POOL_NAK_INTR		0x00020000
#define	ENABLE_ENDPOINT_POOL_ACK_STALL_INTR	0x00010000
#define	ENABLE_CONTROLLER_DEVICE7_INTR		0x00008000
#define	ENABLE_CONTROLLER_DEVICE6_INTR		0x00004000
#define	ENABLE_CONTROLLER_DEVICE5_INTR		0x00002000
#define	ENABLE_CONTROLLER_DEVICE4_INTR		0x00001000
#define	ENABLE_CONTROLLER_DEVICE3_INTR		0x00000800
#define	ENABLE_CONTROLLER_DEVICE2_INTR		0x00000400
#define	ENABLE_CONTROLLER_DEVICE1_INTR		0x00000200
#define	ENABLE_CONTROLLER_DEVICE1_INTR_SHFT	8
#define	ENABLE_SUSPEND_RESUME_INTR		0x00000100
#define	ENABLE_SUSPEND_ENTRY_INTR		0x00000080
#define	ENABLE_BUS_RESET_INTR			0x00000040
#define	ENABLE_HUB_EP1_IN_ACK_INTR		0x00000020
#define	ENABLE_HUB_EP0_IN_NAK_INTR		0x00000010
#define	ENABLE_HUB_EP0_IN_ACK_STALL_INTR	0x00000008
#define	ENABLE_HUB_EP0_OUT_NAK_INTR		0x00000004
#define	ENABLE_HUB_EP0_OUT_ACK_STALL_INTR	0x00000002
#define	ENABLE_HUB_EP0_SETUP_ACK_INTR		0x00000001
//#define	INT_CONTROL_INIT			0x000101ef
#define	INT_CONTROL_INIT			0x0001006f

/* interrupt status register */
#define	INT_STATUS				(USBCORE_ROOT_BASE + 0xc)
#define	BUS_DEADLOCKED				0x00040000
#define	ENDPOINT_POOL_NAK_INTR			0x00020000
#define	ENDPOINT_POOL_ACK_STALL_INTR		0x00010000
#define	CONTROLLER_DEVICE7_INTR			0x00008000
#define	CONTROLLER_DEVICE6_INTR			0x00004000
#define	CONTROLLER_DEVICE5_INTR			0x00002000
#define	CONTROLLER_DEVICE4_INTR			0x00001000
#define	CONTROLLER_DEVICE3_INTR			0x00000800
#define	CONTROLLER_DEVICE2_INTR			0x00000400
#define	CONTROLLER_DEVICE1_INTR			0x00000200
#define	CONTROLLER_DEVICE1_INTR_SHFT		8
#define	CONTROLLER_DEVICE_INTR_ALL		0x0000fe00
#define	SUSPEND_RESUME_INTR			0x00000100
#define	SUSPEND_ENTRY_INTR			0x00000080
#define	BUS_RESET_INTR				0x00000040
#define	EP1_IN_ACK_INTR				0x00000020
#define	EP0_IN_NAK_INTR				0x00000010
#define	EP0_IN_ACK_STALL_INTR			0x00000008
#define	EP0_OUT_NAK_INTR			0x00000004
#define	EP0_OUT_ACK_STALL_INTR			0x00000002
#define	EP0_SETUP_ACK_INTR			0x00000001
#define	INT_STATUS_INIT				0x000001ff

/* endpoint pool ACK interrupt enable register */
#define	EP_ACK_INT_CONTROL			(USBCORE_ROOT_BASE + 0x10)

/* endpoint pool NAK interrupt enable register */
#define	EP_NAK_INT_CONTROL			(USBCORE_ROOT_BASE + 0x14)

/* endpoint pool ACK interrupt status register */
#define	EP_ACK_INT_STATUS			(USBCORE_ROOT_BASE + 0x18)

/* endpoint pool NAK interrupt status register */
#define	EP_NAK_INT_STATUS			(USBCORE_ROOT_BASE + 0x1c)

/* controller soft reset enable register */
#define	SW_RESET				(USBCORE_ROOT_BASE + 0x20)
#define	ENDPOINT_POOL_SOFT_RESET_ENABLE		0x00000200
#define	DMA_CONTROLLER_RESET_ENABLE		0x00000100
#define	CONTROLLER_DEVICE7_SOFT_RESET_ENABLE	0x00000080
#define	CONTROLLER_DEVICE6_SOFT_RESET_ENABLE	0x00000040
#define	CONTROLLER_DEVICE5_SOFT_RESET_ENABLE	0x00000020
#define	CONTROLLER_DEVICE4_SOFT_RESET_ENABLE	0x00000010
#define	CONTROLLER_DEVICE3_SOFT_RESET_ENABLE	0x00000008
#define	CONTROLLER_DEVICE2_SOFT_RESET_ENABLE	0x00000004
#define	CONTROLLER_DEVICE1_SOFT_RESET_ENABLE	0x00000002
#define	HUB_CONTROLLER_SOFT_RESET_ENABLE	0x00000001

/* status register */
#define	USB_STATUS				(USBCORE_ROOT_BASE + 0x24)
#define	SUSPEND_STATE				0x80000000
#define	RESET_STATE				0x40000000
#define	BUS_LINE_STATE_DN			0x20000000
#define	BUS_LINE_STATE_DP			0x10000000
#define	BUS_SPEED				0x08000000
#define	LAST_FRAME_NUMBER			0x07ff0000
#define	UTMI_STATE_XCVR_SELECT			0x00008000
#define	UTMI_STATE_TERM_SELECT			0x00004000
#define	UTMI_STATE_OPMODE			0x00003000
#define	LAST_ENDPOINT_NUMBER			0x00000f00
#define	LAST_ENDPOINT_NUMBER			0x00000f00
#define	LAST_DEVICE_NUMBER			0x0000007f

/* endpoint pool data toggle value set register */
#define	EP_TOGGLE				(USBCORE_ROOT_BASE + 0x28) 
#define	ENDPOINT_DATA_TOGGLE_VALUE_SET		0x00000100
#define	ENDPOINT_INDEX				0x0000001f

/* isochronous fail accumulator register */
#define	ISO_FAIL_ACC				(USBCORE_ROOT_BASE + 0x2c)
#define	ISOCHRONOUS_OUT_FAIL_COUNTER		0x03ff0000
#define	ISOCHRONOUS_IN_FAIL_COUNTER		0x000003ff

/* endpoint 0 control / status register */
#define	EP0_CONTROL				(USBCORE_ROOT_BASE + 0x30)
#define	EP0_OUT_RECEIVED_COUNT			0x007f0000
#define	EP0_OUT_RECEIVED_COUNT_SHFT		16
#define	EP0_IN_TRANSFER_COUNT			0x00007f00
#define	EP0_IN_TRANSFER_COUNT_SHFT		8	
#define	EP0_OUT_READY_TO_RECEIVE		0x00000004
#define	EP0_IN_READY_TO_TRANSFER		0x00000002
#define	EP0_STALL_CONTROL			0x00000001

/* endpoint 0 buffer address register */
#define	EP0_BASE				(USBCORE_ROOT_BASE + 0x34)

/* endpoint 1 control / status register */
#define	EP1_CONTROL				(USBCORE_ROOT_BASE + 0x38)
#define	EP1_RESET_DATA_TOGGLE			0x00000004
#define	EP1_STALL_CONTROL			0x00000002
#define	EP1_ENABLE				0x00000001

/* endpoint 1 status change register */
#define	EP1_BITMAP				(USBCORE_ROOT_BASE + 0x3c)
#define	EP1_DEVICE7_STATUS_CHANGE		0x00000080
#define	EP1_DEVICE6_STATUS_CHANGE		0x00000040
#define	EP1_DEVICE5_STATUS_CHANGE		0x00000020
#define	EP1_DEVICE4_STATUS_CHANGE		0x00000010
#define	EP1_DEVICE3_STATUS_CHANGE		0x00000008
#define	EP1_DEVICE2_STATUS_CHANGE		0x00000004
#define	EP1_DEVICE1_STATUS_CHANGE		0x00000002
#define	EP1_HUB_PORT_STATUS_CHANGE		0x00000001

/* setup data buffer registers */
#define	USBCORE_SETUP_BASE			(USBCORE_BASE + 0x80)

/* root device setup buffer */
#define	ROOT_SETUP_LOW				(USBCORE_SETUP_BASE + 0x0)
#define	ROOT_SETUP_HIGH				(USBCORE_SETUP_BASE + 0x4)

/* device 1-7 setup buffer */
#define	DEVICE_SETUP_LOW			(USBCORE_SETUP_BASE + 0x8)
#define	DEVICE_SETUP_HIGH			(USBCORE_SETUP_BASE + 0xc)
#define	DEVICE_SETUP_SIZE			0x8

/* device 1-7 registers */
#define	USBCORE_DEVICE_BASE			(USBCORE_BASE + 0x100)
#define	USBCORE_DEVICE_SIZE			0x10	

/* downstream function enable control register */
#define	DEVICE_CONTROL				(USBCORE_DEVICE_BASE + 0x0)
#define	DOWNSTREAM_DEVICE_ADDRESS		0x00007f00
#define	ENABLE_EP0_IN_NAK_INTR			0x00000040
#define	ENABLE_EP0_IN_ACK_STALL_INTR		0x00000020
#define	ENABLE_EP0_OUT_NAK_INTR			0x00000010
#define	ENABLE_EP0_OUT_ACK_STALL_INTR		0x00000008
#define	ENABLE_EP0_SETUP_INTR			0x00000004
#define	DEVICE_PORT_SPEED			0x00000002
#define	ENABLE_DEVICE_PORT			0x00000001
#define	DEVICE_CONTROL_INIT			0x0000003d

/* device interrupt status register */
#define	DEVICE_INT_STATUS			(USBCORE_DEVICE_BASE + 0x4)
#define	EP0_IN_NAK_INTR				0x00000010
#define	EP0_IN_ACK_STALL_INTR			0x00000008
#define	EP0_OUT_NAK_INTR			0x00000004
#define	EP0_OUT_ACK_STALL_INTR			0x00000002
#define	EP0_SETUP_INTR				0x00000001
#define	DEVICE_INT_STATUS_INIT			0x0000001f

/* device control/ status register */
#define	DEVICE_EP0_CTRL				(USBCORE_DEVICE_BASE + 0x8)
#define	CSPLIT_IN_WAIT				0x10000000
#define	NORMAL_IN_WAIT				0x08000000
#define	START_SPLIT				0x04000000
#define	TRANSMIT_DMA_STATE			0x03000000
#define	EP0_OUT_RECEIVED_COUNT			0x007f0000
#define	EP0_IN_TRANSFER_COUNT			0x00007f00
#define	EP0_OUT_READY_TO_RECEIVE		0x00000004
#define	EP0_IN_READY_TO_TRANSFER		0x00000002
#define	EP0_STALL_CONTROL			0x00000001

/* device buffer address register */
#define	DEVICE_EP0_BUFFER			(USBCORE_DEVICE_BASE + 0x0c)

/* programmable endpoint 0-20 registers */
#define	USBCORE_ENDPOINT_BASE			(USBCORE_BASE + 0x200)
#define	USBCORE_ENDPOINT_SIZE			0x10	

/* endpoint configuration register */
#define	EP_CONTROL				(USBCORE_ENDPOINT_BASE + 0x0)
#define EP_ISOCHRONOUS_DATA_STAGES		0x0000c000
#define EP_AUTO_DATA_TOGGLE			0x00002000
#define EP_STALL_CONTROL			0x00001000 
#define EP_NUMBER				0x00000f00
#define EP_NUMBER_SHFT				8
#define EP_TYPE					0x00000070
#define	EP_TYPE_DISABLE				0x00000000
#define	EP_TYPE_BULK_IN				0x00000020
#define	EP_TYPE_BULK_OUT			0x00000030
#define	EP_TYPE_INTERRUPT_IN			0x00000040
#define	EP_TYPE_INTERRUPT_OUT			0x00000050
#define	EP_TYPE_ISO_IN				0x00000060
#define	EP_TYPE_ISO_OUT				0x00000070
#define EP_DEVICE_PORT				0x0000000e
#define EP_DEVICE_PORT_SHFT			1	
#define EP_ENABLE				0x00000001

/* endpoint descriptor list control/status register */
#define	EP_LIST_CONTROL				(USBCORE_ENDPOINT_BASE + 0x4)
#define	TRANSMIT_BUFFER_IN_STATUS		0x00100000
#define	TRANSMIT_BUFFER_IN_INDEX		0x000f0000
#define	INTERRUPT_GENERATION_FLAG		0x00001000
#define	EP_CSPLIT_IN_WAIT			0x00000800
#define	AUTO_DATA_TOGGLE_COUNT			0x00000600
#define	START_SPLIT_CYCLE			0x00000100
#define	DESCRIPTOR_PROCESS_STATUS		0x000000f0
#define	DESCRIPTOR_LIST_OPERATION_RESET		0x00000004
#define	SINGLE_STAGE_DESCRIPTOR_MODE		0x00000002
#define	DESCRIPTOR_LIST_OPERATION_ENABLE	0x00000001

/* DMA descriptor/buffer base register */
#define	EP_BASE					(USBCORE_ENDPOINT_BASE + 0x8)

/* DMA descriptor list pointer and status register */
#define	EP_LIST_PTRS				(USBCORE_ENDPOINT_BASE + 0xc)
#define	EP_DESCRIPTOR_LIST_EMPTY		0x80000000
#define	EP_DATA_TOGGLE_SEQUENCE_VALUE		0x30000000
#define	EP_PACKET_SIZE				0x07ff0000
#define	EP_PACKET_SIZE_SHFT			16
#define	EP_DESCRIPTOR_LIST_DMA_READ_POINTER	0x00001f00
#define	EP_DESCRIPTOR_LIST_DMA_READ_POINTER_SHFT 8
#define	EP_DESCRIPTOR_LIST_DMA_WRITE_POINTER	0x0000001f

/* Endpoint Descriptor Buffer Base Address */
#define	DESCRIPTOR_ADDRESS_INDEX		0

/* Endpoint Descriptor Control/Status */
#define	DESCRIPTOR_CONTROL_INDEX		1
#define EP_ENABLE_DESCRIPTOR_INTERRUPT		0x80000000
#define EP_DEVICE_PORT_NUM			0x70000000
#define EP_ENDPOINT_NUM				0x0f000000
#define EP_DEVICE_ADDRESS_NUM			0x007f0000
#define EP_PID					0x0000c000
#define EP_END_OF_PACKET			0x00002000
#define EP_START_OF_PACKET			0x00001000
#define EP_OUT_PACKET_VALID			0x00000800
#define EP_PACKET_LENGTH			0x000007ff

#define	UDC_MAX_ENDPOINTS	15
#define UDC_NUM_PORTS           5

#define	USBCORE_IRQ		5

/* control transfers */
#define WAIT_FOR_SETUP          	0
#define DATA_STATE_CONTROL_READ		1
#define DATA_STATE_CONTROL_READ_ZLP	2
#define DATA_STATE_CONTROL_WRITE	3
#define	WAIT_FOR_CONTROL_WRITE_STATUS	4

#define	EP0_PACKETSIZE		8

#define	MAX_PACKET_SIZE_CONTROL	64
#define	MAX_PACKET_SIZE_INT	64
#define	MAX_PACKET_SIZE_BULK	512

typedef enum ep_type {
	ep_control, ep_bulk_in, ep_bulk_out, ep_interrupt
} ep_type_t;

struct aspeed_ep;
struct aspeed_device;

struct transfer_ops {
	int (*recv_out_packet) (struct aspeed_ep *, u8 *, int);
	int (*data_in_fifo) (struct aspeed_ep *);
	int (*transmit_in_packet) (struct aspeed_ep *, u8 *, int);
	void (*terminate_in_packet) (struct aspeed_ep *);
	void (*ready_to_receive) (struct aspeed_ep *);
};

struct aspeed_ep {
	struct usb_ep ep;
	struct aspeed_device *dev;

	struct transfer_ops *xfer_ops;

	const struct usb_endpoint_descriptor *desc;
	struct list_head queue;

	ep_type_t ep_type;

	u8 stopped;
	u8 endpointAddress;
	int endpointPoolId;  // index into endpoint pool (1-15)
	u8 bmAttributes;
	u8 transmitBusy;
	u8 enabled;
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,23)
	struct work_struct work;
#else
	struct delayed_work work;
#endif

	/* buffer descriptors */
	void * buffer;
	dma_addr_t buffer_phy;
};

struct aspeed_request {
	struct usb_request req;
	struct list_head queue;
};

struct aspeed_device {
	struct usb_gadget        gadget;
	struct usb_gadget_driver *driver;
	struct device            *dev;
	spinlock_t               lock;
	u8                       index;          // Device index
	int                      ep0state;
	u8                       control_buf[64];
	int                      control_buf_count;
	u8                       initialized;
	int                      device_address;
	int                      device_address_change;
	u32                      *setup_low;
	u32                      *setup_high;
	struct aspeed_ep         ep0;
	void                     *ep0_buffer;
	dma_addr_t               ep0_buffer_phy;
	struct aspeed_ep         ep[UDC_MAX_ENDPOINTS];  // Endpoint list
	u8                       numEndpointsUsed;       // number of endpoints used by this device
	u8                       isRegistered;
	struct usb_port_status   portStatus;             // Port status for this device
};

struct aspeed_udc {
	u8                     nextDeviceIndex;
	struct aspeed_device   device[UDC_NUM_PORTS + 1];          // 5 devices + the HUB device
	u8                     numEndpointsAvailable;              // Number or available endpoints from the pool
	u8                     endpointPoolIds[UDC_MAX_ENDPOINTS]; // Available endpoint pool id's
	//u8                     nextEndpointPoolId;                 // Next available endpoint pool index (1-15)
	struct usb_hub_status  hubStatus;
};
			
extern struct aspeed_udc *the_controller;

#define device_index(DEV) ((DEV)->index)

#define ep_is_in(EP)	(((EP)->endpointAddress&USB_DIR_IN)==USB_DIR_IN)
#define ep_address(EP)	((EP)->endpointAddress&0xF)
#define ep_pool_id(EP)	((EP)->endpointPoolId)
#define ep_maxpacket(EP)	((EP)->ep.maxpacket)

#define DMA_ADDR_INVALID        (~(dma_addr_t)0)

#endif
